<?php

	include_once( PLOG_CLASS_PATH."class/action/blogaction.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articlecomments.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/addcommentview.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/users.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articles.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articlecategories.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articlecomments.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articlenotifications.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/integervalidator.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/stringvalidator.class.php" );
    include_once( PLOG_CLASS_PATH."class/net/client.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/textfilter.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/errorview.class.php" );
    include_once( PLOG_CLASS_PATH."class/bayesian/bayesianfiltercore.class.php" );

    /**
     * Takes care of validating the form to add new comments to an article
     */
    class AddCommentAction extends BlogAction 
    {

    	var $_articleId;
        var $_blogId;
        var $_opId;
        var $_userName;
        var $_userEmail;
        var $_userUrl;
        var $_commentText;
        var $_commentTopic;
        var $_parentId;


    	/**
         * Constructor
         */
        function AddCommentAction( $blogInfo, $request )
        {
        	$this->BlogAction( $blogInfo, $request );

            $this->_fetchFields();
        }

        function _fetchFields()
        {
            $this->_articleId = $this->_request->getValue( "articleId" );
            $this->_blogId    = $this->_request->getValue( "blogId" );
            $this->_opId      = $this->_request->getValue( "op" );
            $this->_parentId  = $this->_request->getValue( "parentId" );

            $this->_userEmail = trim($this->_request->getValue( "userEmail" ));
            $this->_userUrl   = trim($this->_request->getValue( "userUrl" ));
            $this->_userName  = trim($this->_request->getValue( "userName" ));

            $this->_parentId  = $this->_request->getValue( "parentId" );
            if( $this->_parentId == "" )
            	$this->_parentId = 0;

            $this->_commentText = trim($this->_request->getValue( "commentText" ));
            $this->_commentTopic = trim($this->_request->getValue( "commentTopic" ));
            // remove all straneous stuff from the text and topic
            $tf = new TextFilter();
            $this->_commentTopic = $tf->xhtmlize($tf->filterHTML( $this->_commentTopic ));

            // and also from the text
            $this->_commentText = $tf->xhtmlize($tf->filterHTML( $this->_commentText ));

            // now, if the option is set, we 'beautify' the text typed by users
            if( $this->_config->getValue( "beautify_comments_text" )) {
            	$this->_commentText = $tf->autop($this->_commentText);
            }

        }

        /**
         * Since this function contains this method, the controller will automatically
         * call it before calling perform()
         *
         * @return True if all fields ok or false otherwise.
         */
        function validate()
        {
            // check if comments are enabled
            $blogSettings = $this->_blogInfo->getSettings();
            if( !$blogSettings->getValue( "comments_enabled" )) {
            	$this->_view = new ErrorView( $this->_blogInfo, "error_comments_not_enabled" );
                $this->setCommonData();

                return false;
            }

            // if they are, proceed with everything else
        	$this->_fetchFields();
        	// we have to check all the important fields first
            $validator = new IntegerValidator( $this->_articleId );
            if( !$validator->validate()) {
            	$this->_view = new ErrorView( $this->_blogInfo, "error_incorrect_article_id" );
                $this->setCommonData();

                return false;
            }

            // check the blog id
            $blogIdVal = new IntegerValidator( $this->_blogId );
            if( !$blogIdVal->validate()) {
            	$this->_view = new ErrorView( $this->_blogInfo, "error_incorrect_blog_id" );
                $this->setCommonData();
                return false;
            }

            // check the post text
            $strValidator = new StringValidator( $this->_commentText );
            if( !$strValidator->validate()) {
                $this->_view = new ErrorView( $this->_blogInfo, "error_comment_without_text" );
                $this->setCommonData();
                return false;
            }

            // check the post owner
            $strUserVal = new StringValidator( $this->_userName );
            if( !$strUserVal->validate()) {
                $this->_view = new ErrorView( $this->_blogInfo, "error_comment_without_name" );
                $this->setCommonData();
                return false;
            }

            // sanity check
            if( $this->_parentId == "" || $this->_parentId < 0 )
            	$this->_parentId = 0;

	    // this is a kind of a hack, so that we can somehow choose which view to show after
	    // adding the post. the only valid value at the moment is "commentarticle"
	    $this->_nextView = $this->_request->getValue( "nextView" );

            // otherwise, we can return everything OK
            return true;
        }

        /**
         * Carries out the action
         */
        function perform()
        {
        	// need to check the ip of the client
            $clientIp = Client::getIp();

            // fetch the same article again so that we can have all the comments from
            // the database, plus this last one
            $articles = new Articles();
            $article  = $articles->getBlogArticle( $this->_articleId, $this->_blogInfo->getId());

            // check if the user wanted to receive comments for this article
            // or not...
            if( $article->getCommentsEnabled() == false ) {
                 $this->_view = new ErrorView( $this->_blogInfo );
                 $this->_view->setValue( "message", "Comments have been disabled for this article." );
                 $this->setCommonData();
                 return false;
            }

        	// we have already checked all the data, so we are sure that everything's in place
            $comments = new ArticleComments();

            // check if there is already a comment with the same text, topic and made from the same
            // IP already in the database because if so, then we will not add the comment that
            // the user is trying to add (a reload button mistake, perhaps?)
            if( !$comments->getIdenticalComment( $this->_commentTopic, $this->_commentText,
                                                 $this->_articleId, $this->_parentId,
                                                 $this->_userName, $this->_userEmail,
                                                 $this->_userUrl, $clientIp )) {
            	if( !$comments->addComment( $this->_commentTopic, $this->_commentText, $this->_articleId,
                                            $this->_parentId, $this->_userName, $this->_userEmail, $this->_userUrl, $clientIp )) {
                	// show an error message if problems
                    $this->_view = new ErrorView( $this->_blogInfo );
                    $this->_view->setValue( "message", "error_adding_comment" );
                    $this->setCommonData();
                    return false;
                }
            }

            // fetch the comments
            $blogSettings = $this->_blogInfo->getSettings();
            $articleComments = $comments->getPostComments( $article->getId(), $blogSettings->getValue( "comments_order" ), COMMENT_STATUS_NONSPAM );

            // if everything's fine, we set up the article object for the view
            if( $this->_nextView == "commentarticle" )
            	$this->_view = new CommentView( $this->_blogInfo );
            else
            	$this->_view = new AddCommentView( $this->_blogInfo );

            $this->_view->setValue( "category", $category );
            $article->setTotalComments( $article->getTotalComments() + 1 );
            $this->_view->setValue( "post", $article );
            $this->_view->setValue( "user", $user );
            $this->_view->setValue( "comments", $articleComments );
            $this->setCommonData();

            // finally, check if there was any user who wanted to be notified of new comments
            // to this post...
            $notifier = new ArticleNotifications();
            $notifier->notifyUsers( $article->getId(), $this->_blogInfo);

            // and update the bayesian filter table, telling it that this message
            // made it through the filter and that it contains content that is ok...
            // we can always undo this later on anyway!
            /*$bayesian = new BayesianFilterCore();
            $bayesian->train( $this->_blogInfo->getId(),
                              $this->_commentTopic,
                              $this->_commentText,
                              $this->_userName,
                              $this->_userEmail,
                              $this->_userUrl,
                              false );*/

            return true;
        }
    }
?>
