<?php

	include_once( PLOG_CLASS_PATH."class/action/action.class.php" );
    include_once( PLOG_CLASS_PATH."class/locale/locales.class.php" );
    include_once( PLOG_CLASS_PATH."class/net/httpvars.class.php" );
    include_once( PLOG_CLASS_PATH."class/template/templateservice.class.php" );
    include_once( PLOG_CLASS_PATH."class/template/template.class.php" );
    include_once( PLOG_CLASS_PATH."class/misc/version.class.php" );

    /**
     * In the same way BlogAction sets some predefined information to be available for every action
     * for the "normal" side of the blog, this one does the same but for the administrative interface.
     * So far it fetches information from the session, such as the UserInfo and the BlogInfo objects
     * so that they are available for every template.
     */
    class AdminAction extends Action {

    	var $_blogInfo;
        var $_userInfo;
        var $_session;
        var $_config;
        var $_locale;

        /**
         * Constructor.
         */
        function AdminAction( $actionInfo, $request )
        {
        	$this->Action( $actionInfo, $request );

            // get information about the session
            $session = HttpVars::getSession();
            $this->_session = $session["SessionInfo"];

            $this->_config  =& Config::getConfig();

            // get the information about the user and quit if we don't have it...
            $this->_getUserInfo();
            if( $this->_userInfo == "" ) {
            	header( "HTTP/1.0 403 Forbidden" );
                print($this->mustAuthenticatePage());
                die();
            }

            // do the same with the information about the blog
            $this->_getBlogInfo();
            if( $this->_blogInfo == "" ) {
            	if( $this->_actionInfo->getActionParamValue() != "blogSelect" ) {
                	header( "HTTP/1.0 403 Forbidden" );
                	print($this->mustAuthenticatePage());
                    die();
                }
            }

            $this->_locale = $this->getLocale();
        }

        /**
         * Retrieves the blogInfo object from the session
         */
        function _getBlogInfo()
        {
        	//$sessionInfo = $_SESSION["SessionInfo"];

            $session = HttpVars::getSession();
            $sessionInfo = $session["SessionInfo"];

            $this->_blogInfo = $sessionInfo->getValue( "blogInfo" );
            //$this->_blogSettings = $this->_blogInfo->getSettings();
        }

        /**
         * Retrieves the userInfo object from the session
         */
        function _getUserInfo()
        {
            $session = HttpVars::getSession();
            $sessionInfo = $session["SessionInfo"];
            $this->_userInfo = $sessionInfo->getValue("userInfo");
        }

        /**
         * sets the default locale, in case we want to send localized messages to the user.
         */
        function getLocale()
        {
        	// don't like this so much...
        	if( $this->_blogInfo != "" ) {
        		$this->_blogSettings = $this->_blogInfo->getSettings();
            	$locale =& Locales::getLocale( $this->_blogSettings->getValue("locale"));
            }
            else {
            	$locale =& Locales::getLocale( $this->_config->getValue("default_locale"));
            }
            
            return $locale;
        }

        /**
         * Adds some common data to the view. this function must be manually called once
         * we've set up a view.
         */
        function setCommonData()
        {
        	$this->_view->setValue( "user", $this->_userInfo );
            $this->_view->setValue( "blog", $this->_blogInfo );
            $this->_view->setValue( "blogsettings", $this->_blogInfo->getSettings());
            $this->_view->setValue( "baseurl", $this->_config->getValue( "base_url"));
            $this->_view->setValue( "op", $this->_actionInfo->_actionParamValue );
            $this->_view->setValue( "version", Version::getVersion());
            $this->_view->setValue( "uploads_enabled", $this->_config->getValue( "uploads_enabled" ));
        }

        /**
         * Saves the session data
         */
        function saveSession()
        {
        	$this->_session->setValue( "blogInfo", $this->_blogInfo );
            $this->_session->setValue( "userInfo", $this->_userInfo );
        	//$_SESSION["SessionInfo"] = $this->_session;
            $session = HttpVars::getSession();
            $session["SessionInfo"] = $this->_session;
            HttpVars::setSession( $session );
        }

        /**
         * Generates a page which shows an "access forbidden" message, prompting the user to
         * authenticate first using the login page.
         */
        function mustAuthenticatePage()
        {
        	$ts = new TemplateService();
            $mustAuthenticate = $ts->Template( "simpleerror", "admin" );
            $locale = Locales::getLocale( $this->_config->getValue( "default_locale" ));
            $mustAuthenticate->assign( "locale", $locale );
            $mustAuthenticate->assign( "message", $locale->tr("error_access_forbidden"));

            return $mustAuthenticate->fetch();
        }
    }
?>
