<?php

    include_once( PLOG_CLASS_PATH."class/action/blogowneradminaction.class.php" );
    include_once( PLOG_CLASS_PATH."class/action/adminaddtemplateaction.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/admintemplatedview.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminerrorview.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminmessageview.class.php" );    
    include_once( PLOG_CLASS_PATH."class/file/unpacker.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/stringvalidator.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/templatevalidator.class.php" );
    include_once( PLOG_CLASS_PATH."class/template/templatesandbox.class.php" );
    include_once( PLOG_CLASS_PATH."class/file/fileuploads.class.php" );

    /**
     * Shows a form to add a new locale file
     */
    class AdminAddBlogTemplateAction extends BlogOwnerAdminAction {

        var $_templateCode;

        function AdminAddBlogTemplateAction( $actionInfo, $request )
        {
            $this->BlogOwnerAdminAction( $actionInfo, $request );
        }

        function validate()
        {
            //
            // first of all, let's make sure that users are allowed to
            // add new templates
            //
            $config =& Config::getConfig();
            if( !$config->getValue( "users_can_add_templates" )) {
                $this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_add_template_disabled"));
                $this->setCommonData();

                return false;
            }

            $this->_subOp = $this->_request->getValue( "subOp" );

            if( $this->_subOp == "uploadTemplate" ) {
                // check that we have a file!!
                $files = HttpVars::getFiles();
                if( count($files) == 0 || $files["templateFile"]["name"] == "") {
                    $this->_view = new AdminErrorView( $this->_blogInfo );
                    $this->_view->setMessage( $this->_locale->tr("error_must_upload_file"));
                    $this->setCommonData();

                    return false;
                }
            }
            elseif( $this->_subOp == "addTemplateCode" ) {
                $this->_templateCode = $this->_request->getValue( "templateCode" );
                $val = new StringValidator( $this->_templateCode );
                if( !$val->validate()) {
                    $this->_view = new AdminErrorView( $this->_blogInfo );
                    $this->_view->setMessage( $this->_locale->tr("error_no_template_name"));
                    $this->setCommonData();

                    return false;
                }
            }

            return true;
        }

        function _performUploadTemplate()
        {
            // get the temporary folder
            $config =& Config::getConfig();
            $tmpFolder = $config->getValue( "temp_folder" );

            // move it to the temporary folder
            $files    = HttpVars::getFiles();
            $uploads  = new FileUploads( $files );
            
            _debug("Procesing uploaded files to $tmpFolder");
            $result = $uploads->process( $tmpFolder );
            if( $result < 0 ) {
                $this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_uploads_disabled"));
                $this->setCommonData();

                return false;
            }

            $upload   = new FileUpload( $files["templateFile"] );

            // and make it go through the template sandbox to check if
            // we're dealing with a 'healthy' file
            _debug("Executing the template sandbox to check that files are healthy enough...");
            $templateSandbox = new TemplateSandbox();
            $valid = $templateSandbox->checkTemplateSet( $upload->getFileName(), $tmpFolder."/");

            if( $valid < 0 ) {
                _debug("There was an error in the sandbox!");
                $this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( AdminAddTemplateAction::_checkTemplateSandboxResult( $valid ));
                $this->setCommonData();
                File::delete( $tmpFolder."/".$upload->getFileName());
                return false;
            }

            //
            // :KLUDGE:
            //
            // maybe we should simply move the files rather than unpacking the whole
            // thing again, but this indeed makes things easier! ;)
            //

            // since it is a local template, the path has to be $template_folder/blog_x/$templateName
            $ts = new TemplateService();
            $blogTemplateFolder = $ts->createBlogTemplateFolder( $this->_blogInfo->getId());
            _debug("Creating the template folder $blogTemplateFolder");            

            // it should be there now... we can continue
            $destFolder = $blogTemplateFolder."/";
            
            _debug("Unpacking file ".$upload->getFileName());
            $unpacker = new Unpacker();
            if( !$unpacker->unpack( $tmpFolder."/".$upload->getFileName(), $destFolder )) {
                _debug("There was an error unpacking the file!");
                $this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_installing_template") );
                $this->setCommonData();

                // remove the file before returning!
                _debug("Removing incorrect file $tmpFolder/".$upload->getFileName());
                File::delete( $tmpFolder."/".$upload->getFileName());

                return false;
            }

            // if the template set was installed ok in the template folder, we can record
            // it as a valid set
            $fileParts = explode( ".", $upload->getFileName());
            $templateName = $fileParts[0];
            _debug("Everything ok, adding template $templateName to the list");
            $ts->addTemplate( $templateName, $this->_blogInfo->getId());

            // remove the file
            File::delete( $tmpFolder."/".$upload->getFileName());

            $this->_view = new AdminMessageView( $this->_blogInfo );
            $this->_view->setMessage( $this->_locale->pr("template_installed_ok", $templateName));
            $this->setCommonData();

            return true;
        }


        //
        // adds a template manually
        //
        function _performAddTemplateCode()
        {
            $config =& Config::getConfig();
            $templateFolder = $config->getValue( "template_folder" );

            $templateName = $this->_request->getValue( "templateCode" );

            $ts = new TemplateService();

            // make sure that the template is valid
            $templateSandbox = new TemplateSandbox();
            $valid = $templateSandbox->checkTemplateFolder( $templateName, $ts->getBlogBaseTemplateFolder( $this->_blogInfo->getId()));
            if( $valid < 0 ) {
                $this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( AdminAddTemplateAction::_checkTemplateSandboxResult( $valid ));
                $this->setCommonData();

                return false;
            }

            // otherwise, we can add it without problems
            $ts->addTemplate( $templateName, $this->_blogInfo->getId());

            $this->_view = new AdminMessageView( $this->_blogInfo );
            $this->_view->setMessage( $this->_locale->pr("template_installed_ok", $templateName));
            $this->setCommonData();

            return true;
        }

        function perform()
        {
            if( $this->_subOp == "uploadTemplate" )
                $result = $this->_performUploadTemplate();
            elseif( $this->_subOp == "addTemplateCode" )
                $result = $this->_performAddTemplateCode();
            else {
                throw( new Exception( "You shouldn't be seeing this!!! :)" ));
                die();
            }

            return $result;
        }
    }
?>
