<?php

	include_once( PLOG_CLASS_PATH."class/action/blogowneradminaction.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminmessageview.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminerrorview.class.php" );
    include_once( PLOG_CLASS_PATH."class/template/templateservice.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/uploadvalidator.class.php" );
    include_once( PLOG_CLASS_PATH."class/net/httpvars.class.php" );
    include_once( PLOG_CLASS_PATH."class/file/fileuploads.class.php" );

	class AdminAddBlogTemplateFileAction extends BlogOwnerAdminAction {

    	var $_files;
        var $_templateId;

    	function AdminAddBlogTemplateFileAction( $actionInfo, $request )
        {
        	$this->BlogOwnerAdminAction( $actionInfo, $request );
        }

        function validate()
        {
        	// make sure that the user is uploading a file
            $this->_files = HttpVars::getFiles();

            // check if there's any file to upload
            if( count($this->_files) == 0 ) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale-tr("error_must_upload_file") );
                $this->setCommonData();

                return false;
            }

            // get the template id
            $this->_templateId = $this->_request->getValue( "templateId" );

            return true;
        }

        function perform()
        {
        	// uploads the file and moves it to the correct folder
            $uploads  = new FileUploads( $this->_files );

            // make sure were it should go
            $ts = new TemplateService();
            $destFolder = $ts->getTemplateFolder( $this->_templateId, $this->_blogInfo->getId());

            // and move it there
            $processedUploads = $uploads->process( $destFolder );

            // check for the different error conditions we can have
            if( $processedUploads == FILE_UPLOADS_NOT_ENABLED ) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
            	$this->_view->setMessage( $this->_locale->tr("error_uploads_disabled"));
            	$this->setCommonData();

                return false;
            }

            $fileResult = $processedUploads[0];
            if( $fileResult->getError()) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
            	$this->_view->setMessage( $this->_locale->tr("error_adding_template_file"));
            	$this->setCommonData();

                return false;
            }

            // check if the file is valid
            $uv = new UploadValidator();
            foreach( $processedUploads as $upload ) {
            	$validateCode = $uv->validate($upload);
                if( $validateCode < 0 ) {
                	$this->_view = new AdminErrorView( $this->_blogInfo );
                    if( $validateCode == UPLOAD_VALIDATOR_ERROR_UPLOAD_TOO_BIG )
                    	$message = $this->_locale->pr("error_file_too_big", $this->_config->getValue( "maximum_file_upload_size" ));
                    elseif( $validateCode == UPLOAD_VALIDATOR_ERROR_FORBIDDEN_EXTENSION )
                    	$message = $this->_locale->tr("error_file_forbidden");

                    $this->_view->setMessage( $message );
                    $this->setCommonData();
                    return false;
                }
            }

            // now, check if the template is still valid with this new
            // file included
            $this->_view = new AdminMessageView( $this->_blogInfo );
            $this->_view->setMessage( $this->_locale->pr("template_file_added_ok", $this->_templateId));
            $this->setCommonData();

            return true;
        }
    }
?>
