<?php

	include_once( PLOG_CLASS_PATH."class/action/blogowneradminaction.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/admintemplatedview.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/stringvalidator.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/users.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminmessageview.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminerrorview.class.php" );
    include_once( PLOG_CLASS_PATH."class/mail/emailservice.class.php" );

    /**
     * Adds a user to the blog
     */
    class AdminAddBlogUserAction extends BlogOwnerAdminAction {

    	var $_sendNotification;
        var $_notificationText;
        var $_newUsername;

    	function AdminAddBlogUserAction( $actionInfo, $request )
        {
        	$this->BlogOwnerAdminAction( $actionInfo, $request );
        }

        function validate()
        {
        	// get the user and validate it
        	$this->_newUsername = $this->_request->getValue( "newBlogUsername" );
            $val = new StringValidator( $this->_newUsername );
            if( !$val->validate()) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_user_id_incorrect") );
                $this->setCommonData();

                return false;
            }

            // check if the user wishes to send a notification message
            if( $this->_request->getValue( "sendNotification" ) == "on" )
            	$this->_sendNotification = true;
            else
            	$this->_sendNotification = false;

            // check if the user typed anything or not
            $this->_notificationText = $this->_request->getValue( "newBlogUserText" );
            if( $this->_notificationText != "" )
            	$this->_sendNotificationEmail = true;
            else
            	$this->_sendNotificationEmail = false;

            return true;
        }

        function sendNotificationEmail( $userInfo )
        {
            // if everything went fine, we can now send the confirmation email
            // only if the user specified a valid email address
            if( $userInfo->getEmail() != "" ) {
            	// build an email message
                $message = new EmailMessage();
                $message->setBody( $this->_notificationText );
                $message->setSubject( "pLog Notification" );
                $message->addTo( $userInfo->getEmail());
                $message->setFrom( $this->_userInfo->getEmail());
                // and finally send it
                $emailService = new EmailService();
                $emailService->sendMessage( $message );
            }

            return true;
        }

        function perform()
        {
        	// see if the user exists
            $users = new Users();
            $userInfo = $users->getUserInfoFromUsername( $this->_newUsername );
            if( !$userInfo ) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->pr("error_could_not_find_user"), $this->_newUsername );
                $this->setCommonData();

                return false;
            }

            // now we can add this user to the blog
            $userPerms = new UserPermissions();
            $res = $userPerms->grantPermission( $userInfo->getId(), $this->_blogInfo->getId(), PERMISSION_BLOG_USER );
            if( !$res ) {
            	// there was an error adding the user to the blog
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->pr("error_adding_user_to_blog", $userInfo->getUsername()));
                $this->setCommonData();

                return false;
            }

            // send a notification if enabled
            if( $this->_sendNotification ) {
            	$this->sendNotificationEmail( $userInfo );
            }

            $this->_view = new AdminMessageView( $this->_blogInfo );
            $this->_view->setMessage( $this->_locale->pr("user_added_to_blog_ok", $userInfo->getUsername()));
            $this->setCommonData();

            return true;
        }
    }
?>
