<?php

    include_once( PLOG_CLASS_PATH."class/action/adminaction.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/mylinks.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/mylink.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminerrorview.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminmessageview.class.php" );

    /**
     * Adds a new link to the database, given some information
     */
    class AdminAddLinkAction extends AdminAction {

        var $_linkName;
        var $_linkUrl;
        var $_linkDescription;
        var $_linkCategoryId;

        /**
         * Constructor. If nothing else, it also has to call the constructor of the parent
         * class, BlogAction with the same parameters
         */
        function AdminAddLinkAction( $actionInfo, $request )
        {
            $this->AdminAction( $actionInfo, $request );
        }

        /**
         * Check the data coming from the user
         */
        function validate()
        {
            $this->_linkName = $this->_request->getValue( "linkName" );
            $this->_linkUrl  = $this->_request->getValue( "linkUrl" );
            $this->_linkCategoryId = $this->_request->getValue( "linkCategoryId" );
            $this->_linkDescription = $this->_request->getValue( "linkDescription" );

            // check everything and exit if not valid
            if( $this->_linkName == "" || $this->_linkUrl == "" || $this->_linkCategoryId == "" || $this->_linkCategoryId <= 0 ) {
                $this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_provide_name_and_url"));
                $this->setCommonData();

                return false;
            }

            return true;
        }

        /**
         * Carries out the specified action
         */
        function perform()
        {
            // check if the link given by the user has http:// or https:// in front.
            // if not there, we will then add it...
            if( !preg_match("/^https?:\/\/.*$/i", $this->_linkUrl))
                $this->_linkUrl = "http://".$this->_linkUrl;

            // adds the new link to the database
            $myLinks = new MyLinks();
            $myLink  = new MyLink( $this->_linkName, $this->_linkDescription, $this->_linkUrl, $this->_blogInfo->getId(), $this->_linkCategoryId );
            if( !$myLinks->addMyLink( $myLink, $this->_blogInfo->getId())) {
                $this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_adding_link"));
                $this->setCommonData();

                return false;
            }

            $this->_view = new AdminMessageView( $this->_blogInfo );
            $this->_view->setMessage( $this->_locale->tr("link_added_ok"));
            $this->setCommonData();

            // better to return true if everything fine
            return true;
        }
    }
?>
