<?php

	include_once( PLOG_CLASS_PATH."class/action/siteadminaction.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminerrorview.class.php" );
    include_once( PLOG_CLASS_PATH."class/locale/locales.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminmessageview.class.php" );
    include_once( PLOG_CLASS_PATH."class/file/fileupload.class.php" );
    include_once( PLOG_CLASS_PATH."class/file/fileuploads.class.php" );

    define( 'ADD_LOCALE_CODE', "addLocaleCode" );
    define( 'UPLOAD_LOCALE', "uploadLocale" );

    /**
     * Takes care of adding new locales to the system
     */
    class AdminAddLocaleAction extends SiteAdminAction {

    	function AdminAddLocaleAction( $actionInfo, $request )
        {
        	$this->SiteAdminAction( $actionInfo, $request );
        }

        function _validateAddLocaleCode()
        {
            // fetch the code of the locale
            $this->_localeCode = $this->_request->getValue( "localeCode" );
            // the code has to have a very specific format, otherwise
            // it's not valid
            if( !preg_match("/[a-z][a-z]_[A-Z][A-Z]/", $this->_localeCode)) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_incorrect_locale_code") );
                $this->setCommonData();

                return false;
            }

            return true;
        }

        //
        // :KLUDGE:
        //
        // this should maybe be moved to the Locales class, so that if in the future
        // we change the naming policy of the locale files, we don't have to come
        // here and make the necesary changes...
        //
        function _checkFileName( $fileName )
        {
        	$localeFileRegexp = "/locale_[a-z][a-z]_[A-Z][A-Z]\.php/";

            return preg_match( $localeFileRegexp, $fileName );
        }

        function _validateUploadLocale()
        {
        	// first of all, we have to see if the name of the file is valid
            $files = HttpVars::getFiles();
            $upload = new FileUpload( $files["localeFile"] );

            if( !$this->_checkFileName( $upload->getFileName())) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_invalid_locale_file") );
                $this->setCommonData();

                return false;
            }

            return true;
        }

        function validate()
        {
        	$this->_subOp = $this->_request->getValue( "subOp" );

            if( $this->_subOp == ADD_LOCALE_CODE ) {
            	$result = $this->_validateAddLocaleCode();
            }
            else if( $this->_subOp == UPLOAD_LOCALE ) {
            	$result = $this->_validateUploadLocale();
            }
            else {
            	throw( new Exception( "Unknown value of the subOp parameter! You should never see this, anyway :)" ));
                die();
            }

            return $result;
        }

        function _performAddLocaleCode()
        {
        	$locales = new Locales();

            // make sure that the locale does not already exist
            /*if( $locales->isValidLocale( $this->_localeCode )) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( "This locale is already available in the system." );
                $this->setCommonData();

                return false;
            }*/

            $res = $locales->addLocale( $this->_localeCode );

            if( !$res ) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_adding_locale_file"));
                $this->setCommonData();

                $result = false;
            }
            else {
            	$this->_view = new AdminMessageView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->pr("locale_added_ok", $this->_localeCode));
                $this->setCommonData();

                $result = true;
           }

           return result;
        }

        function _performUploadLocale()
        {
        	// since we are here, the file name was validated to be ok, so we can
            // continue with the operation
            $files = HttpVars::getFiles();
            $uploads = new FileUploads( $files );

            // we can first of all move the file to the destionation folder
            $result = $uploads->process( $this->_config->getValue( "locale_folder" ));

            // the only thing that can happen is that the file was not correctly saved
            if( $result[0]->getError() != 0 ) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_saving_locale"));
                $this->setCommonData();

                return false;
            }

            // and once it's there, we can do as if we were adding a locale code
            $upload = new FileUpload( $files["localeFile"] );
            $regexp = "/locale_([a-z][a-z]_[A-Z][A-Z])\.php/";

            $res = preg_match( $regexp, $upload->getFileName(), $matches );

            $this->_localeCode = $matches[1];

            return $this->_performAddLocaleCode();
        }

        function perform()
        {
        	if( $this->_subOp == ADD_LOCALE_CODE ) {
            	$result = $this->_performAddLocaleCode();
            }
            else if( $this->_subOp == UPLOAD_LOCALE ) {
            	$result = $this->_performUploadLocale();
            }
            else {
            	$result = false;
            }

            return $result;
        }
    }
?>
