<?php

    include_once( PLOG_CLASS_PATH."class/action/siteadminaction.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/admintemplatedview.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminerrorview.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminmessageview.class.php" );
    include_once( PLOG_CLASS_PATH."class/file/unpacker.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/templatevalidator.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/stringvalidator.class.php" );
    include_once( PLOG_CLASS_PATH."class/template/templatesandbox.class.php" );
    include_once( PLOG_CLASS_PATH."class/file/fileuploads.class.php" );

    /**
     * Shows a form to add a new locale file
     */
    class AdminAddTemplateAction extends SiteAdminAction {

        var $_templateCode;

        function AdminAddTemplateAction( $actionInfo, $request )
        {
            $this->SiteAdminAction( $actionInfo, $request );
        }

        function validate()
        {
            $this->_subOp = $this->_request->getValue( "subOp" );

            if( $this->_subOp == "uploadTemplate" ) {
                // check that we have a file!!
                $files = HttpVars::getFiles();
                if( count($files) == 0 || $files["templateFile"]["name"] == "" ) {
                    $this->_view = new AdminErrorView( $this->_blogInfo );
                    $this->_view->setMessage( $this->_locale->tr("error_must_upload_file"));
                    $this->setCommonData();

                    return false;
                }
            }
            elseif( $this->_subOp == "addTemplateCode" ) {
                $this->_templateCode = $this->_request->getValue( "templateCode" );
                $val = new StringValidator( $this->_templateCode );
                if( !$val->validate()) {
                    $this->_view = new AdminErrorView( $this->_blogInfo );
                    $this->_view->setMessage( $this->_locale->tr("error_no_template_name"));
                    $this->setCommonData();

                    return false;
                }
            }

            return true;
        }

        function _checkTemplateSandboxResult( $result )
        {
            switch( $result ) {
                case ERROR_TEMPLATE_NOT_INSIDE_FOLDER:
                    $errorMessage = $this->_locale->tr("error_template_not_inside_folder");
                    break;
                case ERROR_MISSING_BASE_FILES:
                    $errorMessage =  $this->_locale->tr("error_missing_base_files");
                    break;
                case TEMPLATE_SANDBOX_ERROR_UNPACKING:
                    $errorMessage =  $this->_locale->tr("error_unpacking");
                    break;
                case TEMPLATE_SANDBOX_ERROR_FORBIDDEN_EXTENSIONS:
                    $errorMessage =  $this->_locale->tr("error_forbidden_extensions");
                    break;
                case TEMPLATE_SANDBOX_ERROR_CREATING_WORKING_FOLDER:
                    $errorMessage = $this->_locale->tr("error_creating_working_folder");
                    break;
                default:
                    $errorMessage = $this->_locale->pr("error_checking_template", $result);
                    break;
            }

            return $errorMessage;
        }

        function _performUploadTemplate()
        {
            // handle the uploaded file
            $files    = HttpVars::getFiles();
            $uploads  = new FileUploads( $files );

            $config =& Config::getConfig();

            $tmpFolder = $config->getValue( "temp_folder" );

            // move it to the temporary folder
            $result = $uploads->process( $tmpFolder );

            // and from there, unpack it
            $upload   = new FileUpload( $files["templateFile"] );

            $templateSandbox = new TemplateSandbox();
            $valid = $templateSandbox->checkTemplateSet( $upload->getFileName(), $tmpFolder."/");

            if( $valid < 0 ) {
                $this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_checkTemplateSandboxResult( $valid ));
                $this->setCommonData();

                return false;
            }

            // the template was ok, so then we can proceed and move it to the main
            // template folder, add it to our array of templates

            //
            // :KLUDGE:
            //
            // maybe we should simply move the files rather than unpacking the whole
            // thing again, but this indeed makes things easier! ;)
            $unpacker = new Unpacker();
            $templateFolder = $config->getValue( "template_folder" );
            if( !$unpacker->unpack( $tmpFolder."/".$upload->getFileName(), $templateFolder )) {
                $this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_installing_template"));
                $this->setCommonData();

                return false;
            }

            // if the template set was installed ok in the template folder, we can record
            // it as a valid set
            $ts = new TemplateService();
            $fileParts = explode( ".", $upload->getFileName());
            $templateName = $fileParts[0];
            $ts->addTemplate( $templateName );

            $this->_view = new AdminMessageView( $this->_blogInfo );
            $this->_view->setMessage( $this->_locale->pr("template_installed_ok", $templateName));
            $this->setCommonData();

            return true;
        }


        //
        // adds a template manually
        //
        function _performAddTemplateCode()
        {
            $templateName = $this->_request->getValue( "templateCode" );

            $ts = new TemplateService();

            // make sure that the template is valid
            $templateSandbox = new TemplateSandbox();
            $valid = $templateSandbox->checkTemplateFolder( $templateName, $ts->getBaseTemplateFolder());
            if( $valid < 0 ) {
                $this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_checkTemplateSandboxResult( $valid ));
                $this->setCommonData();

                return false;
            }

            // otherwise, we can add it without problems
            $ts->addTemplate( $templateName );

            $this->_view = new AdminMessageView( $this->_blogInfo );
            $this->_view->setMessage( $this->_locale->pr("template_installed_ok", $templateName) );
            $this->setCommonData();

            return true;
        }

        function perform()
        {
            if( $this->_subOp == "uploadTemplate" )
                $result = $this->_performUploadTemplate();
            elseif( $this->_subOp == "addTemplateCode" )
                $result = $this->_performAddTemplateCode();
            else {
                throw( new Exception( "You shouldn't be seeing this!!! :)" ));
                die();
            }

            return $result;
        }
    }
?>
