<?php

	include_once( PLOG_CLASS_PATH."class/action/adminaction.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminmessageview.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminerrorview.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articlecategories.class.php" );

    /**
     * Deletes an article category from the database
     */
    class AdminDeleteArticleCategoryAction extends AdminAction {

    	var $_categoryId;
        var $_categoryIds;

    	/**
         * Constructor. If nothing else, it also has to call the constructor of the parent
         * class, BlogAction with the same parameters
         */
        function AdminDeleteArticleCategoryAction( $actionInfo, $request )
        {
        	$this->AdminAction( $actionInfo, $request );
        }

        /**
         * checks that the data is valid.
         */
        function validate()
        {
        	// get the array that contains the categories we'd like to delete
            $this->_categoryIds = $this->_request->getValue( "deleteCategories" );
            if( count($this->_categoryIds) == 0 ) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("no_categories_selected"));
                $this->setCommonData();

                return false;
            }

            return true;
        }

        /**
         * Carries out the specified action
         */
        function perform()
        {
        	// check if there is at least one article, because if so, we can't delete
            // the category (would create problems with the database!) It's a shame that
            // using MySQL we can't put this kind of application logic in the database...
            // I think postgres can...

            $message = "";
            $categories = new ArticleCategories();
            foreach( $this->_categoryIds as $categoryId ) {
            	// get the category
                $category   = $categories->getCategory( $categoryId, $this->_blogInfo->getId());
                // get how many articles it has
        		$numArticles = $categories->getNumArticlesCategory( $categoryId );

                // if it has at least one we can't delete it because then it would break the
                // integrity of our data in the database...
            	if( $numArticles > 0 ) {
                	$message .= $this->_locale->pr( "error_category_has_articles", $category->getName())."<br/><br/>";
            	}
                else {
            		// if everything correct, we can proceed and delete it
            		if( !$categories->deleteCategory( $categoryId, $this->_blogInfo->getId()))
                		$message .= $this->_locale->pr("error_deleting_category")."<br/><br/>";
                    else
                    	$message .= $this->_locale->pr("category_deleted_ok", $category->getName())."<br/><br/>";
                }
        	}

            // everything successfult
            $this->_view = new AdminMessageView( $this->_blogInfo );
            $this->_view->setMessage( $message );
            $this->setCommonData();

            return true;
        }
    }
?>
