<?php

	include_once( PLOG_CLASS_PATH."class/action/adminaction.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminmessageview.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminerrorview.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articlecomments.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/integervalidator.class.php" );

    /**
     * Action that shows a list of all the comments for a given post
     */
    class AdminDeleteCommentAction extends AdminAction {

    	var $_articleId;
        var $_commentIds;

    	/**
         * Constructor. If nothing else, it also has to call the constructor of the parent
         * class, BlogAction with the same parameters
         */
        function AdminDeleteCommentAction( $actionInfo, $request )
        {
        	$this->AdminAction( $actionInfo, $request );
        }

        /**
         * Checks that the post id is correct.
         */
        function validate()
        {
        	$this->_articleId = $this->_request->getValue( "postId" );

            // check that we have a valid post identifier
            $validator = new IntegerValidator( $this->_articleId );
            if( !$validator->validate()) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("incorrect_article_id"));
                $this->setCommonData();
                return false;
            }

            // and some comments to delete...
            $this->_commentIds = $this->_request->getValue( "deleteComments" );
            if( count($this->_commentIds) == 0 ) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_no_comments_selected"));
                $this->setCommonData();
                return false;
            }

            return true;
        }

        /**
         * Carries out the specified action
         */
        function perform()
        {
            $comments = new ArticleComments();
            $message = "";
            foreach( $this->_commentIds as $commentId ) {
            	// fetch the comment
                $comment = $comments->getPostComment( $this->_articleId, $commentId );
            	if( !$comments->deletePostComment( $this->_articleId, $commentId ))
                    $message .= $this->_locale->tr("error_deleting_comment")." \"".$comment->getTopic()."\"<br/><br/>";
                else
                	$message .= $this->_locale->pr("comment_deleted_ok", $comment->getTopic())."<br/><br/>";
            }

            $this->_view = new AdminMessageView( $this->_blogInfo );
            $this->_view->setMessage( $message );
            $this->setCommonData();

            // better to return true if everything fine
            return true;
        }
    }
?>
