<?php

	include_once( PLOG_CLASS_PATH."class/action/adminaction.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminmessageview.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminerrorview.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articles.class.php" );

    /**
     * Deletes a post from the database
     */
    class AdminDeletePostAction extends AdminAction {

    	var $_postId;
        var $_postIds;
        var $_blogId;

    	/**
         * Constructor. If nothing else, it also has to call the constructor of the parent
         * class, BlogAction with the same parameters
         */
        function AdminDeletePostAction( $actionInfo, $request )
        {
        	$this->AdminAction( $actionInfo, $request );
        }

        /**
         * Validates that the information coming from the user is valid
         */
        function validate()
        {
        	$this->_postId = $this->_request->getValue( "postId" );
            $this->_postIds = $this->_request->getValue( "deletePost" );
            $this->_blogId = $this->_blogInfo->getId();

            // check if we're trying to delete somebody else's post
            if( $this->_blogId != $this->_blogInfo->getId()) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setValue( "message", $this->_locale->tr("post_cannot_be_deleted"));
                $this->setCommonData();

                return false;
           }

            if( count($this->_postIds) == 0 ) {
                $this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setValue( "message", $this->_locale->tr("no_posts_selected"));
                $this->setCommonData();

                return false;
            }

            return true;
        }

        /**
         * Carries out the specified action
         */
        function perform()
        {
        	// delete the post (it is not physically deleted but rather, we set
            // the status field to 'DELETED'
            $articles = new Articles();
            $message = "";
            foreach( $this->_postIds as $postId ) {
            	// get the post
                $post = $articles->getBlogArticle( $postId, $this->_blogInfo->getId());
				
				//
				// the next if-else branch allows a site administrator or the blog owner to remove
				// anybody's articles. If not, then users can only remove their own articles
				//
				if( $this->_userInfo->isSiteAdmin() || $this->_blogInfo->getOwner() == $this->_userInfo->getId()) 
					$result = $articles->deleteArticle( $postId, $post->getUser(), $this->_blogInfo->getId(), false );
				else
					$result = $articles->deleteArticle( $postId, $this->_userInfo->getId(), $this->_blogInfo->getId(), false );
					
                if( !$result )
                	$message .= $this->_locale->pr("error_deleting_post", $post->getTopic(), $postId )."<br/>";
                else
                	$message .= $this->_locale->pr("post_deleted_ok", $postId, $post->getTopic())."<br/>";
            }


            $this->_view = new AdminMessageView( $this->_blogInfo );
			$message .= "<br/><a href=\"admin.php?op=editPosts\">".$this->_locale->tr("back")."</a>";
            $this->_view->setValue( "message", $message );
            $this->setCommonData();

            return true;

        }
    }
?>
