<?php

	include_once( PLOG_CLASS_PATH."class/action/adminaction.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminerrorview.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminmessageview.class.php" );
    include_once( PLOG_CLASS_PATH."class/gallery/dao/galleryresource.class.php" );
    include_once( PLOG_CLASS_PATH."class/gallery/dao/galleryalbums.class.php" );

    /**
     * Deletes a resource from the blog
     */
    class AdminDeleteResourceAlbumAction extends AdminAction
    {

    	var $_albumIds;

    	/**
         * Constructor. If nothing else, it also has to call the constructor of the parent
         * class, BlogAction with the same parameters
         */
        function AdminDeleteResourceAlbumAction( $actionInfo, $request )
        {
        	$this->AdminAction( $actionInfo, $request );
        }

        function validate()
        {
        	$this->_albumIds = $this->_request->getValue( "deleteAlbum" );

            if( count($this->_albumIds) == 0 ) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_no_albums_selected"));
                $this->setCommonData();

            	return false;
            }

            return true;
        }

        /**
         * Carries out the specified action
         */
        function perform()
        {
        	// load the resource
        	$albums = new GalleryAlbums();

            // loop through the array of resource ids and
            // remove them all
            foreach( $this->_albumIds as $albumId => $value ) {
            	// fetch the resource first, to get some info about it
                $album = $albums->getAlbum( $albumId, $this->_blogInfo->getId(), false );

                //
                // this album cannot be deleted if either:
                //   1) we have resources under it
                //   2) we have child albums under it
                //
                if( $album->getNumChildren() > 0 || $album->getNumResources() > 0 ) {
                	$message .= $this->_locale->pr("error_album_has_children", $album->getName())."<br/><br/>";
                }
                else {
                	// otherwise, we can go ahead and remove it
            		$res = $albums->deleteAlbum( $albumId, $this->_blogInfo->getId())."<br/><br/>";
                	if( $res )
                		$message .= $this->_locale->pr("album_deleted_ok", $album->getName())."<br/><br/>";
                	else
                		$message .= $this->_locale->pr("error_deleting_album", $album->getName())."<br/><br/>";
                }
            }

            // return the view
            $this->_view = new AdminMessageView( $this->_blogInfo );
            $this->_view->setMessage( $message );
            $this->setCommonData();

            // better to return true if everything fine
            return true;
        }
    }
?>
