<?php

	include_once( PLOG_CLASS_PATH."class/action/adminaction.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/admineditpostview.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminerrorview.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articles.class.php" );

    /**
     * Action that shows a form to update a post
     */
    class AdminEditPostAction extends AdminAction {

    	var $_postId;

    	/**
         * Constructor. If nothing else, it also has to call the constructor of the parent
         * class, BlogAction with the same parameters
         */
        function AdminEditPostAction( $actionInfo, $request )
        {
        	$this->AdminAction( $actionInfo, $request );
        }

        /**
         * Checks if the post identifier is valid
         */
        function validate()
        {
        	$this->_postId = $this->_request->getValue( "postId" );

            if( $this->_postId == "" || $this->_postId <= 0 ) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_incorrect_article_id"));
                $this->setCommonData();

                return false;
            }

            return true;
        }

        /**
         * Carries out the specified action
         */
        function perform()
        {
        	// fetch the post from the database
            $posts = new Articles();
            $post  = $posts->getBlogArticle( $this->_postId, $this->_blogInfo->getId());
            if( !$post ) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_fetching_article" ));
                $this->setCommonData();

                return false;
            }

            // fetch the categories since we're going to need them
            $categories = new ArticleCategories();
            $blogCategories = $categories->getBlogCategories( $this->_blogInfo->getId());

            // check if there is a notification for the article set up by this user
            $notifications = new ArticleNotifications();
            $userNotification = $notifications->getUserArticleNotification( $this->_postId, $this->_blogInfo->getId(), $this->_userInfo->getId());

            // put the post in the template context so that its information can be displayed
            $this->_view = new AdminEditPostView( $this->_blogInfo );
            $this->_view->setValue( "post", $post );
            $this->_view->setValue( "categories", $blogCategories );
            $this->_view->setValue( "previewMode", false );

            $months = $this->_locale->getMonthNames();
            $years = range( 1990, 2030 );
            $minutes = Array( "00", "01", "02", "03", "04", "05", "06", "07", "08", "09",
                              "10", "11", "12", "13", "14", "15", "16", "17", "18", "19",
                              "20", "21", "22", "23", "24", "25", "26", "27", "28", "29",
                              "30", "31", "32", "33", "34", "35", "36", "37", "38", "39",
                              "40", "41", "42", "43", "44", "45", "46", "47", "48", "49",
                              "50", "51", "52", "53", "54", "55", "56", "57", "58", "59" );

            //$hours   = range( 0, 23 );
            $hours = Array( "00", "01", "02", "03", "04", "05", "06", "07", "08",
                            "09", "10", "11", "12", "13", "14", "15", "16", "17",
                            "18", "19", "20", "21", "22", "23" );

            $days = $this->_locale->getDayNamesShort();

            // and put everything in the template
            $this->_view->setValue( "months", $months );
            $this->_view->setValue( "days", $days );
            $this->_view->setValue( "years", $years );
            $this->_view->setValue( "hours", $hours );
            $this->_view->setValue( "minutes", $minutes );

            $postDate = $post->getDateObject();
            $this->_view->setValue( "postYear", $postDate->getYear());
            $this->_view->setValue( "postMonth", $postDate->getMonth());
            $this->_view->setValue( "postDay", $postDate->getDay());
            $this->_view->setValue( "postHour", $postDate->getHour());
            $this->_view->setValue( "postMinutes", $postDate->getMinutes());
            $this->_view->setValue( "commentsEnabled", $post->getCommentsEnabled());
            $this->_view->setValue( "disableJavascriptCalendar", $this->_config->getValue( "disable_javascript_calendar" ));            

            // decide wether or not we should notify the user based on what we've just
            // fetched from the database
            if( $userNotification )
            	$this->_view->setValue( "notification", true );
            else
            	$this->_view->setValue( "notification", false );
            $this->setCommonData();

            return true;
        }
    }
?>
