<?php

	include_once( PLOG_CLASS_PATH."class/action/action.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminloginview.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminsimpleerrorview.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/users.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/blogs.class.php" );
    include_once( PLOG_CLASS_PATH."class/net/sessioninfo.class.php" );
	include_once( PLOG_CLASS_PATH."class/view/adminmainview.class.php" );
	include_once( PLOG_CLASS_PATH."class/config/config.class.php" );
	include_once( PLOG_CLASS_PATH."class/misc/version.class.php" );
	include_once( PLOG_CLASS_PATH."class/locale/locales.class.php" );

    /**
     * When the user fills in the login form, we jump to this action which will show
     * another form when the user will choose to which of the blog he or she wants to
     * carry out administrative tasks.
     */
    class AdminLoginAction extends Action {

    	var $_userName;
        var $_userPassword;
        var $_op;
        var $_locale;

    	/**
         * Constructor. If nothing else, it also has to call the constructor of the parent
         * class, BlogAction with the same parameters
         */
        function AdminLoginAction( $actionInfo, $request )
        {
        	$this->Action( $actionInfo, $request );

            $config =& Config::getConfig();
            $this->_locale = Locales::getLocale( $config->getValue( "default_locale" ));
        }

        /**
         * Validate that we have all the information and that the fields are
         * correct.
         */
        function validate()
        {
        	// get the parameters
            $this->_userName     = $this->_request->getValue( "userName" );
            $this->_userPassword = $this->_request->getValue( "userPassword" );
            $this->_op           = $this->_request->getValue( "op" );

            if( $this->_userName == "" || $this->_userPassword == "" ||
                $this->_op != "Login" ) {
                $this->_view = new AdminSimpleErrorView();
                $this->_view->setValue( "message", $this->_locale->tr("error_no_username_or_password"));

                return false;
            }

            return true;
        }

        /**
         * Carries out the specified action
         */
        function perform()
        {
        	// try to authenticate the user
            $users = new Users();
            if( !$users->authenticateUser( $this->_userName, $this->_userPassword )) {
            	$this->_view = new AdminSimpleErrorView();
                $this->_view->setValue( "message", $this->_locale->tr("error_incorrect_username_or_password"));

                return false;
            }
			
            // if the user is correct, get and put his or her information in the session
            $userInfo = $users->getUserInfo( $this->_userName, $this->_userPassword );
            //$sessionInfo = $_SESSION["SessionInfo"];
            $session = HttpVars::getSession();
            $sessionInfo = $session["SessionInfo"];

            $sessionInfo->setValue( "userInfo", $userInfo );
            $session["SessionInfo"] = $sessionInfo;
            HttpVars::setSession( $session );

            // get the list of blogs to which the user belongs
            $userBlogs = $users->getUsersBlogs( $userInfo->getId());

            // but if he or she does not belong to any yet, we quit
            if( empty($userBlogs)) {
            	$this->_view = new AdminSimpleErrorView();
                $this->_view->setValue( "message", $this->_locale->tr("error_dont_belong_to_any_blog"));

                return false;
            }
			
			// if the user belongs to only one blog			
			if( count($userBlogs) == 1 ) {
				// get the only blog
				$this->_blogInfo = array_pop( $userBlogs );
				// and place it in the session, since we're going to need it there
				$session = HttpVars::getSession();
				$session["SessionInfo"]->setValue( "blogInfo", $this->_blogInfo );
				HttpVars::setSession( $session );
				// prepare the view
				$this->_view = new AdminMainView( $this->_blogInfo );
				// and set the common data (if this action class inherited from AdminAction, then
				// we would only need to call setCommonData(), but since we don't have it, we have
				// to do it manually here
				$config =& Config::getConfig();
				$this->_view->setValue( "user", $userInfo );
				$this->_view->setValue( "blog", $this->_blogInfo );
				$this->_view->setValue( "blogsettings", $this->_blogInfo->getSettings());
				$this->_view->setValue( "baseurl", $config->getValue( "base_url"));
				$this->_view->setValue( "op", "login" );
				$this->_view->setValue( "version", Version::getVersion());
				$this->_view->setValue( "uploads_enabled", $config->getValue( "uploads_enabled" ));
			}
			else {
				// or else, show the page where we let the user choose one of the blogs
				$this->_view = new AdminLoginView();
				$this->_view->setValue( "blogs", $userBlogs );
			}

            // better to return true if everything's fine
            return true;
        }
    }
?>
