<?php

	include_once( PLOG_CLASS_PATH."class/action/adminaction.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articlecomments.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/integervalidator.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articles.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminmessageview.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminerrorview.class.php" );
    include_once( PLOG_CLASS_PATH."class/bayesian/bayesianfiltercore.class.php" );

    /**
     * sets the spam status for a post
     */
    class AdminMarkCommentAction extends AdminAction
    {

    	var $_commentId;
        var $_articleId;
        var $_mode;

    	/**
         * Constructor. If nothing else, it also has to call the constructor of the parent
         * class, BlogAction with the same parameters
         */
        function AdminMarkCommentAction( $actionInfo, $request )
        {
        	$this->AdminAction( $actionInfo, $request );
        }

        /**
         * Checks that the post id is correct.
         */
        function validate()
        {
        	$this->_commentId = $this->_request->getValue( "commentId" );
            $this->_articleId = $this->_request->getValue( "articleId" );
            $this->_mode = $this->_request->getValue( "mode" );

            // check comment id
            $validator = new IntegerValidator( $this->_commentId );
            if( !$validator->validate()) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_incorrect_comment_id"));
                $this->setCommonData();
                return false;
            }

            // check article id
            $val2 = new IntegerValidator( $this->_articleId );
            if( !$val2->validate()) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_incorrect_article_id"));
                $this->setCommonData();
                return false;
            }

            // check mode flag
            if( $this->_mode != 0 && $this->_mode != 1 ) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_incorrect_mode_parameter"));
                $this->setCommonData();
                return false;
            }

            return true;
        }

        /**
         * @private
         * Returns true wether the comment whose status we're trying to change
         * really belongs to this blog, just in case somebody's trying to mess
         * around with that...
         */
        function _checkComment( $commentId, $articleId, $blogId )
        {
        	$articleComments = new ArticleComments();
            $articles = new Articles();

            // fetch the comment
            $comment = $articleComments->getPostComment( $articleId, $commentId );
            if( !$comment )
            	return false;

            // fetch the article
            $article = $articles->getBlogArticle( $comment->getArticleId(), $blogId );
            if( !$article )
            	return false;

            return true;
        }

        /**
         * @private
         */
        function _markCommentAsSpam()
        {
        	//_debug("marking comment=".$this->_commentId." article=".$this->_articleId." as SPAM!<br/><br/>");
        	$comments = new ArticleComments();
            if( !$comments->updateCommentStatus( $this->_commentId, COMMENT_STATUS_SPAM )) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $message = $this->_locale->tr("error_marking_comment_as_spam" );
                $message .= "<br/><br/><a href=\"admin.php?op=editComments&amp;postId=".$this->_articleId."\">".$this->_locale->tr("back")."</a>";
                $this->_view->setMessage( $message );
                $this->setCommonData();
                $res = false;
            }
            else {
            	$this->_view = new AdminMessageView( $this->_blogInfo );
                $message = $this->_locale->tr("comment_marked_as_spam_ok" );
                $message .= "<br/><br/><a href=\"admin.php?op=editComments&amp;postId=".$this->_articleId."\">".$this->_locale->tr("back")."</a>";
                $this->_view->setMessage( $message );
                $this->setCommonData();
                $res = true;

                // before exiting, we should get the comment and train the filter
                // to recognize this as spam...
                //_debug("retraining filter...<br/>");
                $comment = $comments->getPostComment( $this->_articleId, $this->_commentId );
                $bayesian = new BayesianFilterCore();
                
				//_debug("UNTRAINING!!<br/>");
                $bayesian->untrain( $this->_blogInfo->getId(),
	                                $comment->getTopic(),
	                                $comment->getText(),
	                                $comment->getUserName(),
	                                $comment->getUserEmail(),
	                                $comment->getUserUrl(),
	                                false );
                                  
								  //_debug("training!!!!!<br/>");
                $bayesian->train( $this->_blogInfo->getId(),
                                  $comment->getTopic(),
                                  $comment->getText(),
                                  $comment->getUserName(),
                                  $comment->getUserEmail(),
                                  $comment->getUserUrl(),
                                  true );
            }

            return $res;
        }

        /**
         * @private
         */
        function _markCommentAsNonSpam()
        {
        	//_debug("marking comment=".$this->_commentId." article=".$this->_articleId." as NON SPAM!<br/><br/>");
        	$comments = new ArticleComments();
            if( !$comments->updateCommentStatus( $this->_commentId, COMMENT_STATUS_NONSPAM )) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $message = $this->_locale->tr("error_marking_comment_as_nonspam" );
                $message .= "<br/><br/><a href=\"admin.php?op=editComments&amp;postId=".$this->_articleId."\">".$this->_locale->tr("back")."</a>";
                $this->_view->setMessage( $message );
                $this->setCommonData();
                $res = false;
            }
            else {
            	$this->_view = new AdminMessageView( $this->_blogInfo );
                $message = $this->_locale->tr("comment_marked_as_nonspam_ok" );
                $message .= "<br/><br/><a href=\"admin.php?op=editComments&amp;postId=".$this->_articleId."\">".$this->_locale->tr("back")."</a>";
                $this->_view->setMessage( $message );
                $this->setCommonData();
                $res = true;

                // before exiting, we should get the comment and train the filter
                // to recognize this as spam...
                //_debug("retraining filter...<br/>");
                $comment = $comments->getPostComment( $this->_articleId, $this->_commentId );
                $bayesian = new BayesianFilterCore();
                
                $bayesian->untrain( $this->_blogInfo->getId(),
	                                $comment->getTopic(),
	                                $comment->getText(),
	                                $comment->getUserName(),
	                                $comment->getUserEmail(),
	                                $comment->getUserUrl(),
	                                true );
                                  
                $bayesian->train( $this->_blogInfo->getId(),
                                  $comment->getTopic(),
                                  $comment->getText(),
                                  $comment->getUserName(),
                                  $comment->getUserEmail(),
                                  $comment->getUserUrl(),
                                  false );
            }

            return $res;
        }

        /**
         * Carries out the specified action
         */
        function perform()
        {
        	// first, let's make sure that the user is trying to edit the right
            // comment...
            if( !$this->_checkComment( $this->_commentId, $this->_articleId, $this->_blogInfo->getId())) {
            	// if things don't match... (like trying to set the status of an article
                // from another blog, then quit...)
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_incorrect_comment_id"));
                $this->setCommonData();
                return false;
            }

        	// depending on the mode, we have to do one thing or another
            if( $this->_mode == 0 )
            	$result = $this->_markCommentAsNonSpam();
            else
            	$result = $this->_markCommentAsSpam();

            // better to return true if everything fine
            return $result;
        }
    }
?>
