<?php

	include_once( PLOG_CLASS_PATH."class/action/adminaction.class.php" );
	include_once( PLOG_CLASS_PATH."class/view/adminerrorview.class.php" );
	include_once( PLOG_CLASS_PATH."class/view/adminmessageview.class.php" );
	include_once( PLOG_CLASS_PATH."class/data/integervalidator.class.php" );
	include_once( PLOG_CLASS_PATH."class/gallery/resizers/galleryresizer.class.php" );
	include_once( PLOG_CLASS_PATH."class/gallery/dao/galleryresources.class.php" );
	include_once( PLOG_CLASS_PATH."class/gallery/dao/galleryresourcestorage.class.php" );
	
	class AdminRegeneratePreviewAction extends AdminAction
	{
		
		var $_resourceId;
		
		function AdminRegeneratePreviewAction( $actionInfo, $request )
        {
        	$this->AdminAction( $actionInfo, $request );
        }
		
		function validate()
		{
			$this->_resourceId = $this->_request->getValue( "resourceId" );
			
			$val = new IntegerValidator( $this->_resourceId );
			if( !$val->validate()) {
				$this->_view = new AdminErrorView( $this->_blogInfo );
				$this->_view->setMessage( $this->_locale->tr("error_loading_resource" ));
				$this->setCommonData();
				
				return false;
			}
			
			return true;
		}
		
		function perform()
		{
			// first of all, fetch the resource
			$resources = new GalleryResources();
			$resource = $resources->getResource( $this->_resourceId, $this->_blogInfo->getId());
			
			// check if it was loaded ok
			if( !$resource ) {
				$this->_view = new AdminErrorView( $this->_blogInfo );
				$this->_view->setMessage( $this->_locale->tr("error_loading_resource"));
				$this->setCommonData();
				
				return false;
			}
			
			// if so, continue... first by checking if the resource is an image or not
			// because if not, then there is no point in generating a thumbnail of it!
			if( !$resource->isImage()) {
				$this->_view = new AdminErrorView( $this->_blogInfo );
				$this->_view->setMessage( $this->_locale->tr("error_resource_is_not_an_image" ));
			}
			
			
			$previewHeight = $this->_config->getValue( "thumbnail_height" );
			$previewWidth  = $this->_config->getValue( "thumbnail_width" );
			$previewKeepAspectRatio = $this->_config->getValue( "thumbnails_keep_aspect_ratio" );

			// build the filename
			$fileNameParts = explode( ".", $resource->getFileName());
			$fileExt = strtolower($fileNameParts[count($fileNameParts)-1]);
			$fileName = $resource->getOwnerId()."-".$resource->getId().".".$fileExt;

            // and start the resizing process
			$resourceStorage = new GalleryResourceStorage();
            $resizer = new GalleryResizer( $resourceStorage->getResourcePath( $resource ));
            $resourceStorage->checkPreviewsStorageFolder( $resource->getOwnerId());
            $outFile = $resourceStorage->getPreviewsFolder( $resource->getOwnerId()).$fileName;
			
            // and finally, we can generate the preview!
            $result = $resizer->generate( $outFile, $previewHeight, $previewWidth, $previewKeepAspectRatio );
			
			$previewFormat = $resizer->getThumbnailFormat();
			$resource->setThumbnailFormat( $previewFormat );
			$resources->updateResource( $resource );
			
			if( !$result ) {
				$this->_view = new AdminErrorView( $this->_blogInfo );
				$message = $this->_locale->tr("error_generating_resource_preview");
				$message .= "<br/><br/><a href=\"javascript:history.go(-1);\">".$this->_locale->tr("back")."</a>";
			}
			else {
				$this->_view = new AdminMessageView( $this->_blogInfo );
				$message = $this->_locale->tr("resource_preview_generated_ok");
				$message .= "<br/><br/><a href=\"admin.php?op=resources&amp;albumId=".$resource->getAlbumId()."\">".$this->_locale->tr("view")."</a>";			
			}
			
			$this->_view->setMessage( $message );
			$this->setCommonData();
			
			return true;
		}
	}
?>