<?php

	include_once( PLOG_CLASS_PATH."class/action/adminaction.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminmessageview.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminerrorview.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/integervalidator.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/trackbackclient.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articles.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/article.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminsendtrackbacksview.class.php" );

    /**
     * Action that shows a form to change the settings of the current blog.
     */
    class AdminSendTrackbacksAction extends AdminAction {

    	var $_postLink;
        var $_postId;
        var $_locale;

    	/**
         * Constructor. If nothing else, it also has to call the constructor of the parent
         * class, BlogAction with the same parameters
         */
        function AdminSendTrackbacksAction( $actionInfo, $request )
        {
        	$this->AdminAction( $actionInfo, $request );
        }

        function validate()
        {
        	$this->_trackbacks = $this->_request->getValue( "postLink" );

            // if there are no trackbacks, there is nothing to do here :)
            if( count($this->_trackbacks) == 0 ) {
            	$this->_view = new AdminMessageView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("trackbacks_no_urls_selected"));
                $this->setCommonData();

                return false;
            }

            // get the post identifier and check that it is correct
            $this->_postId = $this->_request->getValue( "postId" );
            $validator = new IntegerValidator( $this->_postId );
            if( !$validator->validate()) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_incorrect_article_id"));
                $this->setCommonData();

                return false;
            }

            return true;
        }

        /**
         * Carries out the specified action
         */
        function perform()
        {
            // we need to have the post
            $articles = new Articles();
            $post     = $articles->getBlogArticle( $this->_postId, $this->_blogInfo->getId());
            // and now start sending the trackback pings
            $tbClient = new TrackbackClient();
            $results = $tbClient->sendTrackbacks( $this->_trackbacks, $post, $this->_blogInfo);
            // now check the results and give the user the possiblity to retry again with the
            // ones that had some problem.
            $errors = false;
            //print("results:");print_r($results);
            $trackbackLinks = Array();
            foreach( $results as $result ) {
            	if( $result["status"] == TRACKBACK_FAILED) {
                	// add them again to the list of hosts
                    $errors = true;
                    array_push( $trackbackLinks, $result["url"] );
                }
                else if( $result["status"] == TRACKBACK_SUCCESS ) {
                	if( $message == "" ) $message = $this->_locale->tr("trackbacks_received_ok")."<br/><br/>";
                    $message .= "<a href=\"".$result["url"]."\">".$result["url"]."</a><br/>";
                }
                else if( $result["status"] == TRACKBACK_UNAVAILABLE ) {
                    $message .= $this->_locale->tr("trackbacks_no_trackback")."<a href=\"".$result["url"]."\">".$result["url"]."</a><br/>";
                }
            }

            // if there were errors, we let the user try again
            if( $errors ) {
            	if( $message != "" )
                	$message .= "<br/>";
            	$message .= $this->_locale->tr("trackbacks_problems_sending");
                $this->_view = new AdminSendTrackbacksView( $this->_blogInfo );
                $this->_view->setValue( "message", $message );
                $this->_view->setValue( "post", $post);
                $this->_view->setValue( "postlinks", $trackbackLinks );
                $this->setCommonData();
            }
            else {
            	if( $message != "" )
                	$message .= "<br/>";
            	$message .= $this->_locale->tr("trackbacks_received_all_hosts");
                $this->_view = new AdminMessageView( $this->_blogInfo );
                $this->_view->setMessage( $message );
                $this->setCommonData();
            }
        }
    }
?>
