<?php

	include_once( PLOG_CLASS_PATH."class/action/blogowneradminaction.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminblogsettingsview.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/blogs.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/stringvalidator.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/integervalidator.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminmessageview.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminerrorview.class.php" );
    include_once( PLOG_CLASS_PATH."class/locale/locales.class.php" );

    /**
     * Action that shows a form to change the settings of the current blog.
     */
    class AdminUpdateBlogSettingsAction extends BlogOwnerAdminAction {

    	var $_blogLocale;
        var $_blogMaxRecentItems;
        var $_blogMaxMainPageItems;
        var $_blogTemplate;
        var $_blogAbout;
        var $_blogName;
        var $_blogShowMoreEnabled;
        var $_blogShowMoreThreshold;
        var $_blogCommentsEnabled;

    	/**
         * Constructor. If nothing else, it also has to call the constructor of the parent
         * class, BlogAction with the same parameters
         */
        function AdminUpdateBlogSettingsAction( $actionInfo, $request )
        {
        	$this->BlogOwnerAdminAction( $actionInfo, $request );
        }

        /**
         * Validates that the information we've just received from the blog settings
         * form is valid... We have to be really sure about this one!!!
         */
        function validate()
        {
        	// fetch the values from the form
        	$this->_blogLocale = $this->_request->getValue( "blogLocale" );
            $this->_blogMaxRecentItems = $this->_request->getValue( "blogMaxRecentItems" );
            $this->_blogMaxMainPageItems = $this->_request->getValue( "blogMaxMainPageItems" );
            $this->_blogTemplate = $this->_request->getValue( "blogTemplate" );
            $this->_blogAbout = $this->_request->getValue( "blogAbout" );
            $this->_blogName  = $this->_request->getValue( "blogName" );
            $this->_blogShowMoreEnabled = $this->_request->getValue( "blogShowMoreEnabled" );
            $this->_blogEnableHtmlarea = $this->_request->getValue( "blogEnableHtmlarea" );
            $this->_blogCommentsEnabled = $this->_request->getValue( "blogCommentsEnabled" );
            $this->_blogCommentsOrder = $this->_request->getValue( "blogCommentsOrder" );
            $this->_blogTimeOffset = $this->_request->getValue( "blogTimeOffset" );

            // and check them
            $val1 = new StringValidator( $this->_blogLocale );
            if( !$val1->validate()) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setValue( "message", $this->_locale->tr("error_invalid_locale"));
                $this->setCommonData();
                return false;
            }

            $val2 = new IntegerValidator( $this->_blogMaxRecentItems );
            if( !$val2->validate()) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setValue( "message", $this->_locale->tr("error_incorrect_max_recent_items"));
                $this->setCommonData();

                return false;
            }

            $val3 = new IntegerValidator( $this->_blogMaxMainPageItems );
            if( !$val3->validate()) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setValue( "message", $this->_locale->tr("error_incorrect_number_of_posts"));
                $this->setCommonData();

                return false;
            }

            $val4 = new StringValidator( $this->_blogTemplate );
            if( !$val4->validate()) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setValue( "message", $this->_locale->tr("error_incorrect_template"));
                $this->setCommonData();

                return false;
            }

            $val5 = new StringValidator( $this->_blogName );
            if( !$val5->validate()) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setValue( "message", $this->_locale->tr("error_no_blog_name"));
                $this->setCommonData();

                return false;
            }

            // if all correct, we can proceed
            return true;
        }

        /**
         * Carries out the specified action
         */
        function perform()
        {
        	// fetch the settings from the db
            $blogs = new Blogs();
            $blogSettings = $blogs->getBlogSettings( $this->_blogInfo->getId());

            $blogSettings->setValue( "locale", $this->_blogLocale );
            $blogSettings->setValue( "recent_posts_max", $this->_blogMaxRecentItems );
            $blogSettings->setValue( "show_posts_max", $this->_blogMaxMainPageItems );
            $blogSettings->setValue( "template", $this->_blogTemplate );
            $blogSettings->setValue( "time_offset", $this->_blogTimeOffset );

            // check about the show more feature
            if( $this->_blogShowMoreEnabled == "on" )
            	$blogSettings->setValue( "show_more_enabled", true );
            else
            	$blogSettings->setValue( "show_more_enabled", false );

            // check if we'd like to use htmlarea to edit posts
            if( $this->_blogEnableHtmlarea == "on" )
            	$blogSettings->setValue( "htmlarea_enabled", true );
            else
            	$blogSettings->setValue( "htmlarea_enabled", false );

            // check wether to enable or disable comments
            if( $this->_blogCommentsEnabled == "on" )
            	$blogSettings->setValue( "comments_enabled", true );
            else
            	$blogSettings->setValue( "comments_enabled", false );
	 
	    // sets the order in which comments are shown (newest first or olders first)
	    $blogSettings->setValue( "comments_order", $this->_blogCommentsOrder );

            $this->_blogInfo->setAbout( $this->_blogAbout );
            $this->_blogInfo->setBlog( $this->_blogName );
            $this->_blogInfo->setSettings( $blogSettings );
            //$this->_session->setValue( "blogInfo", $this->_blogInfo );
            //$this->saveSession();

            // and now update the settings in the database
            $blogs = new Blogs();
            if( !$blogs->updateBlog( $this->_blogInfo->getId(), $this->_blogInfo )) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setValue( "message", $this->_locale->tr("error_updating_settings"));
               	$this->setCommonData();

                return false;
            }

            // do it again, baby :)))
            $this->_blogInfo->setAbout( stripslashes($this->_blogAbout ));
            $this->_blogInfo->setBlog( stripslashes($this->_blogName ));
            $this->_blogInfo->setSettings( $blogSettings );
            $this->_session->setValue( "blogInfo", $this->_blogInfo );
            $this->saveSession();

            $this->_view = new AdminMessageView( $this->_blogInfo );
            $this->_locale =& Locales::getLocale( $blogSettings->getValue( "locale" ));
            $this->_view->setValue( "message", $this->_locale->pr("blog_settings_updated_ok", $this->_blogInfo->getBlog())."<br/><br/><a href=\"admin.php?op=blogSettings\">".$this->_locale->tr("back")."</a>");
            $this->setCommonData();

            // better to return true if everything fine
            return true;
        }
    }
?>
