<?php

	include_once( PLOG_CLASS_PATH."class/action/siteadminaction.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminblogsettingsview.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminmessageview.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminerrorview.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/blogs.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/users.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/stringvalidator.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/integervalidator.class.php" );

    /**
     * Action that shows a form to change the settings of the current blog.
     */
    class AdminUpdateEditBlogAction extends SiteAdminAction {

    	var $_blogLocale;
        var $_editBlogId;
        var $_blogOwner;
        var $_blogMaxRecentItems;
        var $_blogMaxMainPageItems;
        var $_blogTemplate;
        var $_blogAbout;
        var $_blogName;
        var $_blogShowMoreEnabled;
        var $_blogShowMoreThreshold;
        var $_blogCommentsEnabled;
		var $_blogTimeOffset;

    	/**
         * Constructor. If nothing else, it also has to call the constructor of the parent
         * class, BlogAction with the same parameters
         */
        function AdminUpdateEditBlogAction( $actionInfo, $request )
        {
        	$this->SiteAdminAction( $actionInfo, $request );
        }

        /**
         * Validates that the information we've just received from the blog settings
         * form is valid... We have to be really sure about this one!!!
         */
        function validate()
        {
        	// fetch the values from the form
        	$this->_blogLocale = $this->_request->getValue( "blogLocale" );
            $this->_blogMaxRecentItems = $this->_request->getValue( "blogMaxRecentItems" );
            $this->_blogMaxMainPageItems = $this->_request->getValue( "blogMaxMainPageItems" );
            $this->_blogTemplate = $this->_request->getValue( "blogTemplate" );
            $this->_blogAbout = $this->_request->getValue( "blogAbout" );
            $this->_blogName  = $this->_request->getValue( "blogName" );
            $this->_blogShowMoreEnabled = $this->_request->getValue( "blogShowMoreEnabled" );
            $this->_blogEnableHtmlarea = $this->_request->getValue( "blogEnableHtmlarea" );
            $this->_blogCommentsEnabled = $this->_request->getValue( "blogCommentsEnabled" );
            $this->_blogOwner = $this->_request->getValue( "blogOwner" );
            $this->_editBlogId = $this->_request->getValue( "editBlogId" );
            $this->_blogTimeOffset = $this->_request->getValue( "blogTimeOffset" );			

            // and check them
            $val1 = new StringValidator( $this->_blogLocale );
            if( !$val1->validate()) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setValue( "message", $this->_locale->tr("error_invalid_locale"));
                $this->setCommonData();
                return false;
            }

            $val2 = new IntegerValidator( $this->_blogMaxRecentItems );
            if( !$val2->validate()) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setValue( "message", $this->_locale->tr("error_incorrect_max_recent_items"));
                $this->setCommonData();

                return false;
            }

            $val3 = new IntegerValidator( $this->_blogMaxMainPageItems );
            if( !$val3->validate()) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setValue( "message", $this->_locale->tr("error_incorrect_number_of_posts"));
                $this->setCommonData();

                return false;
            }

            $val4 = new StringValidator( $this->_blogTemplate );
            if( !$val4->validate()) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setValue( "message", $this->_locale->tr("error_incorrect_template"));
                $this->setCommonData();

                return false;
            }

            $val5 = new StringValidator( $this->_blogName );
            if( !$val5->validate()) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setValue( "message", $this->_locale->tr("error_no_blog_name"));
                $this->setCommonData();

                return false;
            }

            // validate the blog identifier
            $val7 = new IntegerValidator( $this->_editBlogId );
            if( !$val7->validate()) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setValue( "message", $this->_locale->tr("error_incorrect_blog_id"));
                $this->setCommonData();

                return false;
            }

            // and now validate the user
            $val8 = new IntegerValidator( $this->_blogOwner );
            if( !$val8->validate()) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_incorrect_user_id"));
                $this->setCommonData();

                return false;
            }

            // if all correct, we can proceed
            return true;
        }

        /**
         * Carries out the specified action
         */
        function perform()
        {
        	// get the blog we're trying to update
            $blogs = new Blogs();
            $blogInfo = $blogs->getBlogInfo( $this->_editBlogId );
            if( !$blogInfo ) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_updating_settings"));
                $this->setCommonData();

                return false;
            }

            // make sure that the user we'd like to set as owner exists
            $users = new Users();
            $userInfo = $users->getUserInfoFromId( $this->_blogOwner );
            if( !$userInfo ) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_incorrect_blog_owner"));
                $this->setCommonData();

                return false;
            }

            $blogSettings = $blogInfo->getSettings();
            $blogSettings->setValue( "locale", $this->_blogLocale );
            $blogSettings->setValue( "recent_posts_max", $this->_blogMaxRecentItems );
            $blogSettings->setValue( "show_posts_max", $this->_blogMaxMainPageItems );
            $blogSettings->setValue( "template", $this->_blogTemplate );

            // check about the show more feature
            if( $this->_blogShowMoreEnabled == "on" )
            	$blogSettings->setValue( "show_more_enabled", true );
            else
            	$blogSettings->setValue( "show_more_enabled", false );

            // check if we'd like to use htmlarea to edit posts
            if( $this->_blogEnableHtmlarea == "on" )
            	$blogSettings->setValue( "htmlarea_enabled", true );
            else
            	$blogSettings->setValue( "htmlarea_enabled", false );

            // check wether to enable or disable comments
            if( $this->_blogCommentsEnabled == "on" )
            	$blogSettings->setValue( "comments_enabled", true );
            else
            	$blogSettings->setValue( "comments_enabled", false );

            $blogSettings->setValue( "time_offset", $this->_blogTimeOffset );

            $blogInfo->setAbout( $this->_blogAbout );
            $blogInfo->setBlog( $this->_blogName );
            $blogInfo->setSettings( $blogSettings );
            $blogInfo->setOwner( $this->_blogOwner );
            //$this->_session->setValue( "blogInfo", $this->_blogInfo );
            //$this->saveSession();

            // and now update the settings in the database
            if( !$blogs->updateBlog( $blogInfo->getId(), $blogInfo )) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setValue( "message", $this->_locale->pr( "error_updating_blog_settings", $blogInfo->getBlog())."<br/><br/>");
               	$this->setCommonData();

                return false;
            }

            // do it again, baby :)))
            if( $this->_blogInfo->getId() == $blogInfo->getId()) {
            	$this->_blogInfo->setAbout( stripslashes($this->_blogAbout ));
            	$this->_blogInfo->setBlog( stripslashes($this->_blogName ));
            	$this->_blogInfo->setSettings( $blogSettings );
            	$this->_session->setValue( "blogInfo", $this->_blogInfo );
            	$this->saveSession();
            }

            // if everything went fine, we can show a nice message
            $this->_view = new AdminMessageView( $this->_blogInfo );
            $this->_locale =& Locales::getLocale( $blogSettings->getValue( "locale" ));
			$message = $this->_locale->pr( "blog_settings_updated_ok", $blogInfo->getBlog())."<br/><br/>";
			$message .= "<a href=\"admin.php?op=editSiteBlogs\">".$this->_locale->tr("back")."</a>";
            $this->_view->setValue( "message", $message );
            $this->setCommonData();

            // better to return true if everything fine
            return true;
        }
    }
?>
