<?php

	include_once( PLOG_CLASS_PATH."class/action/blogowneradminaction.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminerrorview.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminmessageview.class.php" );
    include_once( PLOG_CLASS_PATH."class/plugin/pluginmanager.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/stringvalidator.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/admintemplatedview.class.php" );

    class AdminUpdatePluginConfigurationAction extends BlogOwnerAdminAction {

    	var $_pluginId;

        function AdminUpdatePluginConfigurationAction( $actionInfo, $request )
        {
        	$this->BlogOwnerAdminAction( $actionInfo, $request );
        }

        function validate()
        {
        	$this->_pluginId = $this->_request->getValue( "pluginId" );

            $val = new StringValidator( $this->_pluginId );
            if( !$val->validate()) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_incorrect_plugin_id") );
                $this->setCommonData();

                return false;
            }

            return true;
        }

        /**
         * If we don't reload the settings they won't become visible until the
         * user starts another session, since they are stored in the session.
         */
        function updateCurrentBlogSettings()
        {
        	$blogs = new Blogs();

            $blogInfo = $blogs->getBlogInfo( $this->_blogInfo->getId());
            $blogSettings = $blogInfo->getSettings();

            $this->_blogInfo->setSettings( $blogSettings );
            $this->_session->setValue( "blogInfo", $this->_blogInfo );
            $this->saveSession();
        }

        /**
         * Carries out the specified action
         */
        function perform()
        {
        	// initialize the plugin manager and load the plugins
            $pm = new PluginManager( $this->_blogInfo, $this->_userInfo );

            // check if the plugin manager is enabled or not, since if it's not
            // there is no point in this feature...
            if( !$pm->isEnabled()) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_plugins_disabled"));
                $this->setCommonData();

                return false;
            }

            // get the configuration message of the plugin
            $res = $pm->configurePlugin( $this->_pluginId, $this->_request );

            // check if the message is valid and if it is, continue or show an
            // error otherwise
            if( !$res ) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $message .= $this->_locale->tr("error_configuring_plugin")."<br/><br/>";
                $message .= $pm->getPluginConfigurationErrorMessage( $this->_pluginId );
                $this->_view->setMessage( $message );
                $this->setCommonData();
            }
            else {
            	// create a view and put the plugin objects in the template
                $pluginConfigMessage = $pm->getPluginConfigurationMessage( $this->_pluginId );
                $this->_view = new AdminTemplatedView( $this->_blogInfo, "pluginconfig" );
            	$this->_view->setValue( "pluginconfiguration", $pluginConfigMessage );
            	$this->setCommonData();
                // besides, we have to reload this blog settings and config just
                // so that they become automatically available
                $this->updateCurrentBlogSettings();
            }

            return true;
        }
    }
?>
