<?php

	include_once( PLOG_CLASS_PATH."class/action/siteadminaction.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/admintemplatedview.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminerrorview.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/users.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/blogs.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/userpermissions.class.php" );
    include_once( PLOG_CLASS_PATH."class/action/admineditsiteblogusersaction.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminmessageview.class.php" );

    class AdminUpdateSiteBlogUsersAction extends SiteAdminAction {

    	var $_editBlogId;
        var $_subOp;
        var $_siteSelectedUsers;
        var $_blogSelectedUsers;

    	function AdminUpdateSiteBlogUsersAction( $actionInfo, $request )
        {
        	$this->SiteAdminAction( $actionInfo, $request );
        }

        function validate()
        {
        	// fetch the id of the blog we're trying to update
        	$this->_editBlogId = $this->_request->getValue( "editBlogId" );
            // and the sub operation
            $this->_subOp = $this->_request->getValue( "subOp" );

            // list of users in the site that have been selected
            $this->_siteSelectedUsers = $this->_request->getValue( "siteUsers" );

            // list of users in the blog that have been selected
            $this->_blogSelectedUsers = $this->_request->getValue( "blogUsers" );

            return true;
        }

        /**
         * Adds the selected users from the list of available users to the blog
         */
        function _performAddOperation()
        {
        	// check if the user selected any user from the list
        	if( count($this->_siteSelectedUsers) == 0 ) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_no_users_selected_to_add") );

                return false;
            }

            // go through the list of users and add them to the database
            $userPermissions = new UserPermissions();
            $users = new Users();
            $message = "";
            foreach( $this->_siteSelectedUsers as $userId ) {
            	$userInfo = $users->getUserInfoFromId( $userId );
            	$result = $userPermissions->grantPermission( $userId, $this->_editBlogId, PERMISSION_BLOG_USER );
                if( $result )

                    $message .= $this->_locale->pr( "user_added_to_blog_ok", $userInfo->getUsername());
                else
                    $message .= $this->_locale->pr( "error_adding_user_to_blog", $userInfo->getUsername());

                $message .= "<br/><br/>";
            }

            //$this->_view = new AdminMessageView( $this->_blogInfo );
            //$this->_view->setMessage( $message );

            $this->_loadView();

        	return true;
        }

        function _loadView()
        {
            $this->_view = new AdminTemplatedView( $this->_blogInfo, "siteblogusers" );

            // get all the users in the site
            $users = new Users();
            $siteUsers = $users->getAllUsers();
            $this->_view->setValue( "siteusers", $siteUsers );

            // get info about the blog we'd like to edit
            $blogs = new Blogs();
            $editBlogInfo = $blogs->getBlogInfo( $this->_editBlogId );
            $this->_view->setValue( "editblog", $editBlogInfo );

            // and the users who belong to the given blog
            $blogUsers = $users->getBlogUsers( $this->_editBlogId, false );
            $this->_view->setValue( "blogusers", $blogUsers );

            $this->_view->setValue( "availableusers", AdminEditSiteBlogUsersAction::filterAvailableUsers( $siteUsers, $blogUsers ));
            $this->setCommonData();

            return true;
        }

        /**
         * Removes the selected users from the list of users of the blog
         */
        function _performRemoveOperation()
        {
        	// check if the user selected any user from the list
        	if( count($this->_blogSelectedUsers) == 0 ) {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_no_users_selected_to_remove"));

                return false;
            }

            // go through the list of users and revoke the permission to be
            // part of the blog
            $userPermissions = new UserPermissions();
            $users = new Users();
            $message = "";
            foreach( $this->_blogSelectedUsers as $userId ) {
            	$userInfo = $users->getUserInfoFromId( $userId );
            	$result = $userPermissions->revokePermission( $userId, $this->_editBlogId, PERMISSION_BLOG_USER );
               if( $result )
                    $message .= $this->_locale->pr( "user_removed_from_blog_ok", $userInfo->getUsername());
                else
                    $message .= $this->_locale->pr( "error_removing_user_from_blog", $userInfo->getUsername());

                $message .= "<br/><br/>";
            }

            //$this->_view = new AdminMessageView( $this->_blogInfo );
            //$this->_view->setMessage( $message );

            $this->_loadView();

        	return true;
        }

        function perform()
        {
            //$this->_view = new AdminTemplatedView( $this->_blogInfo, "siteblogusers" );
            $this->_view = new AdminMessageView( $this->_blogInfo );

            if( $this->_subOp == "Add -->" )
            	$this->_performAddOperation();
            else if( $this->_subOp == "<-- Remove" )
            	$this->_performRemoveOperation();
            else {
            	$this->_view = new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( "Incorrect operation." );
                $this->setCommonData();

                return false;
            }

            $this->setCommonData();

            return true;
        }
    }
?>
