<?php

	include_once( PLOG_CLASS_PATH."class/action/adminaction.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/stringvalidator.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/emailvalidator.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/userinfo.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/users.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminerrorview.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/adminmessageview.class.php" );

    /**
     * Action that shows a form to change the settings of the current user.
     */
    class AdminUpdateUserSettingsAction extends AdminAction {

        var $_userAbout;
        var $_userEmail;
        var $_userPassword;
        var $_userConfirmPassword;

    	/**
         * Constructor. If nothing else, it also has to call the constructor of the parent
         * class, BlogAction with the same parameters
         */
        function AdminUpdateUserSettingsAction( $actionInfo, $request )
        {
        	$this->AdminAction( $actionInfo, $request );
        }

        /**
         * Validates that the information we've just received from the blog settings
         * form is valid... We have to be really sure about this one!!!
         */
        function validate()
        {
            // if all correct, we can proceed
            $this->_userEmail = trim($this->_request->getValue( "userEmail" ));
            $this->_userAbout = trim($this->_request->getValue( "userAbout" ));
            $this->_userPassword = trim($this->_request->getValue( "userPassword" ));
            $this->_userConfirmPassword = trim($this->_request->getValue( "confirmPassword" ));

            // validate the email address
            $emailVal = new EmailValidator( $this->_userEmail );
            if( !$emailVal->validate()) {
            	$this->_view =  new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_incorrect_email_address"));
                $this->setCommonData();

                return false;
            }

            // check that the password is correct and confirm it
            if( $this->_userPassword != "" ) {
            	$passwordVal = new StringValidator( $this->_userPassword );
            	if( !$passwordVal->validate()) {
            		$this->_view =  new AdminErrorView( $this->_blogInfo );
                	$this->_view->setMessage( $this->_locale->tr("error_incorrect_password"));
                	$this->setCommonData();

                	return false;
            	}
            	if( $this->_userPassword != $this->_userConfirmPassword ) {
            		$this->_view =  new AdminErrorView( $this->_blogInfo );
                	$this->_view->setMessage( $this->_locale->tr("error_password_dont_match"));
                	$this->setCommonData();

                	return false;
            	}
            }

            return true;
        }

        /**
         * Carries out the specified action
         */
        function perform()
        {
        	// update the user information
            $this->_userInfo->setEmail( $this->_userEmail );
            if( $this->_userPassword != "" )
            	$this->_userInfo->setPassword( $this->_userPassword );
            $this->_userInfo->setAboutMyself( $this->_userAbout );
            $this->_session->setValue( "userInfo", $this->_userInfo );
            $this->saveSession();

            $users = new Users();
            // update the user information
            if( !$users->updateUser( $this->_userInfo )) {
            	$this->_view =  new AdminErrorView( $this->_blogInfo );
                $this->_view->setMessage( $this->_locale->tr("error_updating_user_settings"));
                $this->setCommonData();

                return false;
            }

            // if everything fine, also say so...
            $this->_view = new AdminMessageView( $this->_blogInfo );
            $this->_view->setMessage( $this->_locale->pr("user_settings_updated_ok", $this->_userInfo->getUsername())."<br/><br/><a href=\"admin.php?op=userSettings\">".$this->_locale->tr("back")."</a>");
            $this->setCommonData();

            return true;
        }
    }
?>
