<?php

	include_once( PLOG_CLASS_PATH."class/action/action.class.php" );
    include_once( PLOG_CLASS_PATH."class/config/config.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/blogs.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articles.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/timestamp.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/mylinks.class.php" );
    include_once( PLOG_CLASS_PATH."class/xml/rssparser.class.php" );
    include_once( PLOG_CLASS_PATH."class/plugin/pluginmanager.class.php" );
    include_once( PLOG_CLASS_PATH."class/net/httpvars.class.php" );
    include_once( PLOG_CLASS_PATH."class/misc/version.class.php" );
    include_once( PLOG_CLASS_PATH."class/security/pipeline.class.php" );
    include_once( PLOG_CLASS_PATH."class/security/pipelineresult.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/mylinkscategories.class.php" );
	include_once( PLOG_CLASS_PATH."class/dao/archivelink.class.php" );
	include_once( PLOG_CLASS_PATH."class/locale/locales.class.php" );

	/**
     * Extends the BlogAction class so that some common operations for all the actions
     * can be done in one common place, for example to fetch the SessionInfo object
     * from the HTTP session.
     * It is recommended that all the classes implementing actions extend this one unless
     * it is really necessary.
     *
     * <b>IMPORTANT (28-06-2003)</b>
     * Using the design of Views and Templates, now I have a problem... Whenever the header has
     * to be displayed (the header includes a calendar, the most recent posts, the links feature
     * and the archives) all that information has to be fetched from the database. Since _any_
     * post can do an {include file="header.template"}, that information has to be _always_ ready
     * for whenever we need it. That is the reason why I have to do here common things such as
     * _getRecentPosts or _getMonthPosts. Ideally, that should only be calculated _only_ when
     * the header is called... I'll try to think about something else in the meantime :(
     */
	class BlogAction extends Action {

    	var $_session;
        var $_config;
        var $_blogInfo;
        var $_recentPosts;
        var $_blogLinks;
        var $_blogLinksCategories;
        var $_monthPosts;
        var $_getMonthPosts;
        var $_archiveStats;
        var $_archives;
        var $_articleCategories;

        var $_pluginManager;

        /**
         * Constructor. Additionally, it fetches the SessionInfo object from
         * the session information
         */
		function BlogAction( $actionInfo, $request )
        {
        	$this->Action( $actionInfo, $request );

            // we use the HttpVars package since then we can access the session object
            // independently wether we're using php ver. < 4.1.0 or not
            $session = HttpVars::getSession();
        	$this->_session = $session["SessionInfo"];

            $this->_config  =& Config::getConfig();

            $this->_getBlogInfo();

            // save the blogid in the session
            $this->_session->setValue( "blogId", $this->_blogInfo->getId());

            $this->checkDateParameter();

            $this->_getRecentPosts();
            $this->_getMonthPosts();
            $this->_getBlogLinks();

            $this->_getArchives();

            $this->_getArticleCategories();

            // initialize the plugin manager
            $this->_pluginManager = new PluginManager( $this->_blogInfo, $this->_blogInfo );

            //
            // security stuff
            //
            $pipeline = new Pipeline( $request, $this->_blogInfo );
            $result = $pipeline->process();
            //
            // if the pipeline blocked the request, then we have
            // to let the user know
            if( !$result->isValid()) {
                $blogSettings = $this->_blogInfo->getSettings();
                $localeCode = $blogSettings->getValue( "locale" );
                $locale =& Locales::getLocale( $localeCode );                        
				$message = $locale->tr("error_you_have_been_blocked")."<br/><br/>";
				$message .= $result->getErrorMessage();
            	$this->_view = new ErrorView( $this->_blogInfo, $message );
                $this->setCommonData();
                $this->_view->render();

                die();
            }
        }

        /**
         * Saves the information from the session
         */
        function saveSession()
        {
        	//$_SESSION["SessionInfo"] = $this->_session;
            $session = HttpVars::getSession();
            $session["SessionInfo"] = $this->_session;
            HttpVars::setSession( $session );
        }

        /**
         * Sets some common information (dirty dirty...)
         */
        function setCommonData()
        {
            $this->_view->setValue( "recentposts", $this->_recentPosts );
            $this->_view->setValue( "mylinks", $this->_blogLinks );
            $this->_view->setValue( "mylinkscategories", $this->_blogLinksCategories );
            $this->_view->setValue( "archives", $this->_archives );
            $this->_view->setValue( "monthposts", $this->_monthPosts );
            $this->_view->setValue( "articlecategories", $this->_articleCategories );
            $this->_view->setValue( "rss", new RSSParser());
            $this->_view->setValue( "version", Version::getVersion());
			$this->_view->setValue( "now", new Timestamp());
            $this->_view->generateCalendar( $this->_session->getValue( "Year"), $this->_session->getValue( "Month" ));

            // put the plugins in the context
            $plugins = $this->_pluginManager->getPlugins();

            foreach( $plugins as $name => $plugin ) {
            	$this->_view->setValue( $name, $plugin );
            }

        }

        /**
         * Fetches the article categories for the given blog
         */
        function _getArticleCategories()
        {
        	$categories = new ArticleCategories();
            $this->_articleCategories = $categories->getBlogCategories( $this->_blogInfo->getId());
        }
        /**
         * Fetches the stats for the archives
         */
        function _getArchives()
        {
        	$articles = new Articles();
            $this->_archiveStats = $articles->getNumberPostsPerMonth( $this->_blogInfo->getId());

            if( $this->_archiveStats == "" )
            	return false;

            $links = Array();
            $blogSettings = $this->_blogInfo->getSettings();
            $localeCode = $blogSettings->getValue( "locale" );
            $locale =& Locales::getLocale( $localeCode );
            $urls = new RequestGenerator( $this->_blogInfo );
            foreach( $this->_archiveStats as $yearName => $year) {
            	foreach( $year as $monthName => $month ) {
                	// we can use the Timestamp class to help us with this...
                	$t = new Timestamp( "" );
                    $t->setYear( $yearName );
                    $t->setMonth( $monthName );
                    $archiveUrl = $urls->getArchiveLink( $t->getYear().$t->getMonth());
                    // we can get the string representing the month since Timestamp is a Locale-aware
                    // class
                	//$link = new MyLink( $t->getMonthString()." $yearName", "", $archiveUrl, $this->_blogInfo->getId(), 0);
                    //print($locale->toString);
                    $linkName = $locale->formatDate( $t, "%B %Y" );
                	$link = new ArchiveLink( $linkName, "", $archiveUrl, $this->_blogInfo->getId(), 0, $month, 0);
                    array_push( $links, $link );
                }
            }

            $this->_archives = $links;
        }

        /**
         * Fetches the information for this blog from the database since we are going to need it
         * almost everywhere.
         */
        function _getBlogInfo()
        {
        	$blogId = $this->_config->getValue("default_blog_id");
			
            // if there is a "blogId" parameter, it takes precedence over the
            // "user" parameter.
            if( $this->_request->getValue("blogId"))
            	$blogId = $this->_request->getValue("blogId");
            else {
            	// check if there was a user parameter
                $userParam = $this->_request->getValue( "user" );
                if( $userParam ) {
                	// if so, check to which blogs the user belongs
                	$users = new Users();
                	$userInfo = $users->getUserInfoFromUsername( $userParam );
                    // if the user exists and is valid...
                	if( $userInfo ) {
                    	$userBlogs = $users->getUsersBlogs( $userInfo->getId());
                        // check if he or she belogs to any blog. If he or she does, simply
                        // get the first one (any better rule for this?)
                    	if( !empty($userBlogs)) {
	                		$blogId = $userBlogs[0]->getId();
                        }
                    }
                }
                else {
                    // if there is no user parameter, we take the blogId from the session
                    if( $this->_session->getValue("blogId") != "" ) {
                    	$blogId = $this->_session->getValue("blogId");
                    }
                }
            }

            // fetch the BlogInfo object
            $blogs = new Blogs();
            $this->_blogInfo = $blogs->getBlogInfo( $blogId );

            // security checks...
            if( $this->_blogInfo == false ) {
            	print( "ERROR: The blog does not exist." );
                die;
            }
        }

        /**
         * Retrieves the most recent posts
         */
        function _getRecentPosts()
        {
        	$articles = new Articles();
            $blogSettings = $this->_blogInfo->getSettings();

            $t = new Timestamp();
            $todayTimestamp = $t->getTimestamp();
            
        	$this->_recentPosts = $articles->getBlogArticles( $this->_blogInfo->getId(), -1, $blogSettings->getValue( "recent_posts_max" ), 0, 1, 0, $todayTimestamp);
        }

        /**
         * Checks if there is at least a year and a month in the request
         */
        function checkDateParameter()
        {
        	$date = $this->_request->getValue( "Date" );
        	if( $date ) {
            	$year = substr( $date, 0, 4);
                $month = substr( $date, 4,2 );
                $day = substr( $date, 6, 2);
                //print("year: ".$year." - month: ".$month." - day: ".$day."<br/>" );
            }
            else {
            	// is it in the session?
                /*$year = $this->_session->getValue( "Year" );
                $month = $this->_session->getValue( "Month" );
                $day   = $this->_session->getValue( "Day" );
                if( $year == "" || $month == "" ) {*/
            		$t = new Timestamp();
                	$year = $t->getYear();
                	$month = $t->getMonth();
                	$day = $t->getDay();
                //}
            }

            $this->_session->setValue( "Year", $year );
            $this->_session->setValue( "Month", $month );
            $this->_session->setValue( "Day", $day );
        }

        /**
         * Retrieves all the posts for the current month
         */
        function _getMonthPosts()
        {
        	$t = new Timestamp();
        	$articles = new Articles();
            $blogSettings = $this->_blogInfo->getSettings();
            // the values for the month and the year have been included in the session
            $month = $this->_session->getValue( "Month" );
            $year = $this->_session->getValue( "Year" );

            $this->_monthPosts = $articles->getNumberPostsPerDay( $this->_blogInfo->getId(), $year, $month );
        }

        /**
         * Retrieves the links
         */
        function _getBlogLinks()
        {
        	$myLinks = new MyLinks();
            $myLinksCategories = new MyLinksCategories();
            $this->_blogLinks = $myLinks->getLinks( $this->_blogInfo->getId());
            $this->_blogLinksCategories = $myLinksCategories->getMyLinksCategories( $this->_blogInfo->getId());
        }
    }
?>
