<?php

	include_once( PLOG_CLASS_PATH."class/action/blogaction.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articles.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/rssview.class.php" );
    include_once( PLOG_CLASS_PATH."class/locale/locale.class.php" );
    include_once( PLOG_CLASS_PATH."class/locale/locales.class.php" );

    //
    // default profile used if none specified
    //
    define( "DEFAULT_PROFILE", "rss090" );

    /**
     * This class is used by the controller that takes care of handling the requests for the
     * RSS feed.
     */
	class RssAction extends BlogAction {

    	/**
         * Constructor.
         */
    	function RssAction( $blogInfo, $request )
        {
        	$this->BlogAction( $blogInfo, $request );
        }

        /**
         * Performs the action.
         */
        function perform()
        {
        	// fetch the articles for the given blog
        	$articles = new Articles();
            $blogSettings = $this->_blogInfo->getSettings();
            $localeCode = $blogSettings->getValue( "locale" );

            // fetch the default profile as chosen by the administrator
            $defaultProfile = $this->_config->getValue( "default_rss_profile" );
            if( $defaultProfile == "" || $defaultProfile == null )
            	$defaultProfile = DEFAULT_PROFILE;

            // fetch the profile
            // if the profile specified by the user is not valid, then we will
            // use the default profile as configured
            $profile = $this->_request->getValue( "profile" );

            // fetch the category, or set it to '0' otherwise, which will mean
            // fetch all the most recent posts from any category
            $categoryId = $this->_request->getValue( "categoryId" );
            if( !is_numeric($categoryId))
            	$categoryId = 0;

            // create an instance of a locale object
            $locale = Locales::getLocale( $localeCode );
            
            // get current date
            $t = new Timestamp();
            $today = $t->getTimestamp();

            // fetch the posts
            $amount = $blogSettings->getValue( "recent_posts_max", 15 );
            $blogArticles = $articles->getBlogArticles( $this->_blogInfo->getId(), -1, $amount, 
                                                        $categoryId, POST_STATUS_PUBLISHED, 0, $today );
            
            // check if the template is available
            $this->_view = new RssView( $this->_blogInfo, $profile, $defaultProfile );
            $this->_view->setValue( "locale", $locale );
            $this->_view->setValue( "posts", $blogArticles );
            $this->setCommonData();

            return true;
        }

    }

?>
