<?php

	include_once( PLOG_CLASS_PATH."class/action/blogaction.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/viewarticleview.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/blogs.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articles.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articlecategories.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articlecomments.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/users.class.php" );
    include_once( PLOG_CLASS_PATH."class/config/config.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/trackbacks.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/integervalidator.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/referers.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/textfilter.class.php" );
    include_once( PLOG_CLASS_PATH."class/view/errorview.class.php" );

    /**
     * This class represents the defaut view in our application
     */
	class ViewArticleAction extends BlogAction
    {

    	var $_config;
        var $_articleId;

		function ViewArticleAction( $actionInfo, $request )
        {
			$this->BlogAction( $actionInfo, $request );

            $this->_config =& Config::getConfig();
        }

        // checks that the articleId is valid
        function validate()
        {
        	$this->_articleId = $this->_request->getValue( "articleId" );

            $validator = new IntegerValidator( $this->_articleId );
            // if the information is not correct, we better show a message and quit
            if( !$validator->validate()) {
            	$this->_view = new ErrorView( $this->_blogInfo, "error_incorrect_article_id" );
                $this->setCommonData();

                return false;
            }

            return true;
        }

        function perform()
        {
            // fetch the article
            $articles = new Articles();
            $article  = $articles->getBlogArticle( $this->_articleId, $this->_blogInfo->getId());
            $nextArticle = $articles->getBlogNextArticle( $this->_articleId, $this->_blogInfo->getId());
            $prevArticle = $articles->getBlogPrevArticle( $this->_articleId, $this->_blogInfo->getId());
            $blogSettings = $this->_blogInfo->getSettings();

        	$this->_view = new ViewArticleView( $this->_blogInfo );

            // if the article id doesn't exist, cancel the whole thing...
            if( $article == false) {
				$this->_view = new ErrorView( $this->_blogInfo );
                $this->_view->setValue( "message", "error_fetching_article" );
                $this->setCommonData();

                return false;
            }

            // if the article has been deleted, we shouldn't show it either!!
            if( $article->isDeleted()) {
				$this->_view = new ErrorView( $this->_blogInfo );
                $this->_view->setValue( "message", "error_fetching_article" );
                $this->setCommonData();

                return false;
            }

            // check if we have to update how many times an article has been read
            if( $this->_config->getValue( "update_article_reads" )) {
            	$curReads = $article->getNumReads();
            	$article->setNumReads( $curReads + 1 );
                //if( !$articles->updateArticle( $article ))
                //	print("error!!!!");
                $articles->updateArticleNumReads( $article->getId(), $curReads + 1 );
            }

            // fetch the comments
            $comments = new ArticleComments();
            $blogSettings = $this->_blogInfo->getSettings();
            $articleComments = $comments->getPostComments( $article->getId(), $blogSettings->getValue( "comments_order" ), COMMENT_STATUS_NONSPAM );
            $this->_view->setValue( "comments", $articleComments );

            // fetch the trackback items related to this article, in case we want to show them.
            $trackbacks = new TrackBacks();
            $postTrackbacks = $trackbacks->getArticleTrackbacks( $article->getId());

            // update the referer tracking
            $referers = new Referers();
            $referers->addReferer( $_SERVER["HTTP_REFERER"], $article->getId(), $this->_blogInfo->getId());
            $articleReferers = $referers->getArticleReferers( $article->getId());

           //**** sorry, but this has to go into every action that will include the header.template *****//
            //**** dirtiest hack I've ever seen, though :( ******//
            $this->setCommonData();

            // if everything's fine, we set up the article object for the view
            $this->_view->setValue( "category", $article->getCategory());
            $this->_view->setValue( "post", $article );
            $this->_view->setValue( "nextpost", $nextArticle );
            $this->_view->setValue( "prevpost", $prevArticle );
            $this->_view->setValue( "user", $article->getUser());
            $this->_view->setValue( "trackbacks", $postTrackbacks );
            $this->_view->setValue( "referers", $articleReferers );

            // and return everything normal
            return true;
        }
    }
?>