<?php

	include_once( PLOG_CLASS_PATH."class/object/object.class.php" );
    include_once( PLOG_CLASS_PATH."class/bayesian/bayesiantokenizer.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/bayesianfilterinfos.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/bayesiantokens.class.php" );

    /**
     * provides the 'train' and 'untrain' method that are crucial to get our
     * filter to differentiate between spam and non-spam...
     */
	class BayesianFilterCore extends Object
    {

    	function BayesianFilterCore()
        {
        	$this->Object();
        }

        /**
         * Trains the filter to recognize comments like this as spam
         *
         * @param blogId
         * @param topic
         * @param text
         * @param userName
         * @param userEmail
         * @param userUrl
         * @param spam Wether we should set this message as spam or not
         */
        function train($blogId, $topic, $text, $userName, $userEmail, $userUrl, $spam = false)
        {
            $tokenizer = new BayesianTokenizer();
            
            $tokensTopic = $tokenizer->addContextMark($tokenizer->tokenize($topic), TOKEN_TOPIC_MARK);
            $tokensText = $tokenizer->tokenize($text);
            $tokensUserName = $tokenizer->addContextMark($tokenizer->tokenize($userName), TOKEN_USER_NAME_MARK);
            $tokensUserEmail = $tokenizer->addContextMark($tokenizer->tokenize($userEmail), TOKEN_USER_EMAIL_MARK);
            $tokensUserUrl = $tokenizer->addContextMark($tokenizer->tokenize($userUrl), TOKEN_USER_URL_MARK);
            
            $tokens = array_merge($tokensTopic, $tokensText, $tokensUserName, $tokensUserEmail, $tokensUserUrl);
            
            $bayesianFilterInfos = new BayesianFilterInfos();
            $bayesianFilterInfo  = $bayesianFilterInfos->getBlogBayesianFilterInfo($blogId);
            
            $totalSpam = $bayesianFilterInfo->getTotalSpam();
            $totalNonSpam = $bayesianFilterInfo->getTotalNonSpam();
            
            $bayesianTokens = new BayesianTokens();
            
            if ($spam)
            {
                $totalSpam++;
                $bayesianFilterInfos->incTotalSpam($bayesianFilterInfo->getId());
                $bayesianTokens->incSpamOccurrencesFromTokensArray($blogId, $tokens, $totalSpam, $totalNonSpam);
            }
            else
            {
                $totalNonSpam++;
                $bayesianFilterInfos->incTotalNonSpam($bayesianFilterInfo->getId());
                $bayesianTokens->incNonSpamOccurrencesFromTokensArray($blogId, $tokens, $totalSpam, $totalNonSpam);
            }                        
        }


        /**
        * -- Add function info here --
        */
        function untrain($blogId, $topic, $text, $userName, $userEmail, $userUrl, $spam = false)
        {
            $tokenizer = new BayesianTokenizer();
            
            $tokensTopic = $tokenizer->addContextMark($tokenizer->tokenize($topic), TOKEN_TOPIC_MARK);
            $tokensText = $tokenizer->tokenize($text);
            $tokensUserName = $tokenizer->addContextMark($tokenizer->tokenize($userName), TOKEN_USER_NAME_MARK);
            $tokensUserEmail = $tokenizer->addContextMark($tokenizer->tokenize($userEmail), TOKEN_USER_EMAIL_MARK);
            $tokensUserUrl = $tokenizer->addContextMark($tokenizer->tokenize($userUrl), TOKEN_USER_URL_MARK);
            
            $tokens = array_merge($tokensTopic, $tokensText, $tokensUserName, $tokensUserEmail, $tokensUserUrl);
            
            $bayesianFilterInfos = new BayesianFilterInfos();
            $bayesianFilterInfo  = $bayesianFilterInfos->getBlogBayesianFilterInfo($blogId);
            
            $totalSpam = $bayesianFilterInfo->getTotalSpam();
            $totalNonSpam = $bayesianFilterInfo->getTotalNonSpam();
            
            $bayesianTokens = new BayesianTokens();
            
            if ($spam)
            {
				//_debug("untraining previous spam!");
                $bayesianTokens->decSpamOccurrencesFromTokensArray($blogId, $tokens, $totalSpam, $totalNonSpam);
                $bayesianFilterInfos->decTotalSpam($bayesianFilterInfo->getId());
            }
            else
            {
				//_debug("untraining previous non-spam!!!");
                $bayesianTokens->decNonSpamOccurrencesFromTokensArray($blogId, $tokens, $totalSpam, $totalNonSpam);
                $bayesianFilterInfos->decTotalNonSpam($bayesianFilterInfo->getId());
            }
        }
   }
?>
