<?php

	include_once( PLOG_CLASS_PATH."class/dao/model.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articlecategory.class.php" );

	/**
     * Model for the article categories
     */
	class ArticleCategories extends Model
    {

    	/**
         * Constructor, does nothing special.
         */
    	function ArticleCategories()
        {
        	$this->Model();

            $this->totalCategories = Array();
            $this->cache = Array();
        }

        /**
         * Returns an article category from the database, given its id
         *
         * @param categoryId Identifier of the category
         * @param blogId The blog identifier to which the category belongs
         * @return an ArticleCategory object representing a category from the database
         */
        function getCategory( $categoryId, $blogId = -1 )
        {
        	// check if we already loaded that category
            if( isset($this->cache[$categoryId])) {
            	return $this->cache[$categoryId];
            }

            // if not, do the normal process...
            $query = "SELECT * FROM ".$this->getPrefix()."articles_categories WHERE id = ".$categoryId;
            if( $blogId != -1 )
            	$query .= " AND blog_id = ".$blogId;
            $query .= ";";

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;

            if( $result->RowCount() == 0 )
            	return false;

            $row = $result->FetchRow($result);
            $category = $this->_fillCategoryInformation( $row );

            // ...and store the result
            $this->cache[$categoryId] = $category;

            return $category;
        }

        /**
         * Returns all the categories in the database
         *
         * @return An array containing <b>all</b> the categories from the database.
         */
        function getAllCategories()
        {
			$query = "SELECT * FROM ".$this->getPrefix()."articles_categories ORDER BY id ASC;";

			$result = $this->_db->Execute( $query );

			if( $this->emptySet($result))
				return false;

			$categories = array();
			while( $row = $result->FetchRow( $result )) {
            	$category = $this->_fillCategoryInformation( $row );
				//array_push( $categories, $category );
                $categories[$category->getId()] = $category;
            }

			return $categories;
        }

        /**
         * Returns the categories that belong to a given blog
         *
         * @param blogId The identifer of the blog from which we want to fetch the
         * categories.
         * @param onlyInMainPage Returns only the categories that have been configured
         * to be shown in the main page (attribute in_main_page is '1') By default
         * it is set to 'false', so that we always return all the categories of the given
         * blog.
         * @return An array containing all the categories from a given blog.
         */
        function getBlogCategories( $blogId, $onlyInMainPage = false )
        {
        	$query = "SELECT * FROM ".$this->getPrefix()."articles_categories
                      WHERE blog_id = $blogId";
            if( $onlyInMainPage )
                      $query .= " AND in_main_page = 1";

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;

            $categories = Array();
            while( $row = $result->FetchRow( $result )) {
            	$category = $this->_fillCategoryInformation( $row );
                $categories[$category->getId()] = $category;

                // we can cache this information for later...
                $this->cache[$category->getId()] = $category;
            }

            return $categories;
        }

        /**
         * Private function that takes an associative array from the database and returns
         * an object of the type ArticleCategory properly filled
         *
         * @param query_result Resulting resource of executing a query
         * @return Returns an ArticleCategory object
         */
        function _fillCategoryInformation( $query_result )
        {
        	$category = new ArticleCategory( $query_result["name"],
                                             $query_result["url"],
                                             $query_result["blog_id"],
                                             $query_result["in_main_page"],
                                             $this->getNumArticlesCategory( $query_result["id"]),
                                             $query_result["id"]);

            return $category;
        }

        /**
         * Adds an article category to the database.
         *
         * @param category A Category object with all the information needed to add it
         * to the database
         * @return False if error or the article category identifier if successful.
         */
        function addArticleCategory( $articleCategory )
        {
        	$query = "INSERT INTO ".$this->getPrefix()."articles_categories
                      (name,url,blog_id,in_main_page) VALUES ('".Db::qstr($articleCategory->getName()).
                      "','".$articleCategory->getURL().
                      "',".$articleCategory->getBlogId().
                      ",".$articleCategory->isInMainPage().");";

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;

            $query = "SELECT id FROM ".$this->getPrefix()."articles_categories WHERE
                      name = '".Db::qstr($articleCategory->getName())."' AND
                      url = '".$articleCategory->getUrl()."' AND
                      blog_id = ".$articleCategory->getBlogId()." AND
                      in_main_page = ".$articleCategory->isInMainPage().";";

            $result = $this->_db->Execute( $query );
            if( !$result )
            	return false;

            $row = $result->FetchRow();
            return $row["id"];
        }

        /**
         * Returns how many articles have been classified under this category
         *
         * @param categoryId The identifier of that category.
         * @param status
         * @param maxDate
         * @return The number of articles, or 0 if none.
         */
        function getNumArticlesCategory( $categoryId, $status = "PUBLISHED", $maxDate = null )
        {
        	if( empty($this->totalArticles)) {
        		$query1 = "SELECT category_id, COUNT(*) AS 'count' FROM ".$this->getPrefix()."articles WHERE status = '$status' GROUP BY (category_id)";
            	$this->totalArticles = $this->_db->GetAssoc( $query1 );
            }

            $totalArticles = $this->totalArticles[$categoryId];
            if( $totalArticles == "" ) $totalArticles = 0;

            return $totalArticles;
        }

        /**
         * Removes a category from the database. We should check <b>beforehand</b> that such category
         * does not have any article classified under it, since then we would have problems
         * with the database.
         *
         * @param $categoryId The identifier of the category
         * @param $blogId The blog identifier to which the article belongs
         * @return True if success or false otherwise.
         */
        function deleteCategory( $categoryId, $blogId )
        {
        	$query = "DELETE FROM ".$this->getPrefix()."articles_categories WHERE id = ".$categoryId." AND blog_id = ".$blogId;

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;
            else
            	return true;
        }

        /**
         * Removes all the posts from the given blog
         *
         * @param blogId The blog identifier
         */
        function deleteBlogCategories( $blogId )
        {
        	$query = "DELETE FROM ".$this->getPrefix()."articles_categories WHERE blog_id = $blogId";

            $result = $this->_db->Execute( $query );

            return $result;
        }

        /**
         * Updates a category.
         *
         * @param category An ArticleCategory object with the information about the category we're
         * going to update.
         * @return True if successful or false otherwise.
         */
        function updateCategory( $category )
        {
        	$query = "UPDATE ".$this->getPrefix()."articles_categories
                      SET name = '".Db::qstr($category->getName())."',
                      url = '".$category->getUrl()."',
                      in_main_page = ".$category->isInMainPage()."
                      WHERE id = ".$category->getId().";";

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;
            else
            	return true;
        }
    }
?>
