<?php
	// Category.php
	// Encapsulates a category from the db

	include_once( PLOG_CLASS_PATH."class/object/object.class.php" );

    /**
     * Represents a category. Every post belongs to exactly one category, but categories
     * are also passed to the template as an array, so that they can be displayed
     * in a column... or something :)
     */
	class ArticleCategory extends Object {

		var $_id;
		var $_name;
		var $_url;
        var $_blogId;
        var $_numArticles;
        var $_inMainPage;

		function ArticleCategory( $name, $url, $blogId, $inMainPage, $numArticles = 0, $id = -1 )
		{
			$this->Object();
			$this->_name = $name;
            $this->_url = $url;
			$this->_id = $id;
            $this->_blogId = $blogId;
            $this->_inMainPage = $inMainPage;
            $this->_numArticles = $numArticles;
		}

        /**
         * @private
         */
		function setId( $newId )
		{
			$this->_id = $newId;
		}

        /**
         * @private
         */
		function setName( $newName )
		{
			$this->_name = $newName;
		}

        /**
         * @private
         */
		function setURL( $newURL )
		{
			$this->_url = $newURL;
		}

        /**
         * @private
         */
        function setBlogId( $blogId )
        {
        	$this->_blogId = $blogId;
        }

        /**
         * Returns the identifier assigned to this category.
         *
         * @return An integer value with the category number.
         */
		function getId()
		{
			return $this->_id;
		}

        /**
         * Returns the name assigned to the category.
         *
         * @return A string value with the name assigned to the category.
         */
		function getName()
		{
			return $this->_name;
		}

        /**
         * @private
         */
		function getURL()
		{
			return $this->_url;
		}

        /**
         * Returns the identifier of the blog to which this category belongs.
         *
         * @return An integer value containing the identifier to which this category belongs.
         */
        function getBlogId()
        {
        	return $this->_blogId;
		}

        /**
         * Returns how many articles have been categorized under this category.
         *
         * @return An integer value
         */
        function getNumArticles()
        {
        	return $this->_numArticles;
        }

        /**
         * Returns true if the category has been configured to be shown in the main
         * page or not.
         *
         * @return True wether the category is shown in main page, or false otherwise.
         */
        function isInMainPage()
        {
        	return $this->_inMainPage;
        }

        /**
         * If set to true, enables the category to be shown in the main page, or false
         * otherwise.
         *
         * @param show True or false, depending wether we'd like to show the category in
         * the main page or not.
         */
        function setInMainPage( $show )
        {
        	$this->_inMainPage = $show;
        }
	}
?>
