<?php

	include_once( PLOG_CLASS_PATH."class/dao/model.class.php" );
	include_once( PLOG_CLASS_PATH."class/dao/usercomment.class.php" );

	define( COMMENT_ORDER_OLDEST_FIRST, 1 );
	define( COMMENT_ORDER_NEWEST_FIRST, 2 );

    /**
     * different post statuses
     */
    define( COMMENT_STATUS_ALL, -1 );
    define( COMMENT_STATUS_NONSPAM, 0 );
    define( COMMENT_STATUS_SPAM, 1 );
    define( COMMENT_STATUS_DUBIOUS, 2 );

    /**
     * Model for the comments each article can have
     */
	class ArticleComments extends Model {

    	var $totalComments;
        var $_blogSettings;

    	function ArticleComments()
        {
        	$this->Model();

            $this->totalComments = Array();

            $this->_blogSettings = Array();
        }

        /**
         * Adds a comment to an article
         */
        function addComment( $topic, $text, $articleId, $parentId = 0, $userName = "", $userEmail = "", $userUrl = "", $clientIp = "0.0.0.0", $status = COMMENT_STATUS_NONSPAM )
        {
        	$query = "INSERT INTO ".$this->getPrefix()."articles_comments (article_id,topic,text,user_name,user_email,user_url,parent_id,client_ip,status) VALUES (".$articleId.",'".Db::qstr($topic)."','".Db::qstr($text)."','".$userName."','".$userEmail."','".$userUrl."',".$parentId.", '".$clientIp."', $status);";

            $result = $this->_db->Execute( $query );

            if( $result == false )
            	return false;
            else
            	return true;
        }

		/**
		 * Retrieves all the comments for a post
		 */
		function getPostComments( $artid, $order = COMMENT_ORDER_NEWEST_FIRST, $status = COMMENT_STATUS_ALL )
		{

			$query = "SELECT * FROM ".$this->getPrefix()."articles_comments WHERE article_id = ".$artid;
            if( $status != COMMENT_STATUS_ALL )
            	$query .= " AND status = $status";

			// check in which order we should display those comments
			if( $order == COMMENT_ORDER_NEWEST_FIRST )
				$query .= " ORDER BY date DESC";
			else
				$query .= " ORDER BY date ASC";


            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;

            $comments = Array();
            while ($row = $result->FetchRow()) {
                $comment = $this->_fillCommentInformation( $row );
                array_push( $comments, $comment );
			}

			return $comments;
		}

        /**
         * Returns the total number of comments for a post
         */
        function getNumberOfComments( $artId )
        {
        	if( empty($this->totalComments)) {
        		$query1 = "SELECT article_id, COUNT(*) AS 'count' FROM ".$this->getPrefix()."articles_comments GROUP BY (article_id)";
            	$this->totalComments = $this->_db->GetAssoc( $query1 );
            }

            $totalPosts = $this->totalComments[$artId];
            if( $totalPosts == "" ) $totalPosts = 0;

            return $totalPosts;
        }

        /**
         * Retrieves only one post from a given article
         */
        function getPostComment( $artid, $commentid )
		{
			$query = "SELECT * FROM ".$this->getPrefix()."articles_comments WHERE id = ".$commentid." AND article_id = ".$artid.";";

			$result = $this->_db->Execute( $query );

			if( !$result )
				return false;

            $row = $result->FetchRow();
            $comment = $this->_fillCommentInformation( $row );

			return $comment;
		}

        /**
         * Returns true if there already is a comment in the database with the same
         * article_id field, topic, text, replying to the same comment, username and so on
         * Useful in case we want to check if the user sent the same comment by mistake
         * by pressing the 'reload' button twice or something...
         *
         * @return Returns true if there already is such comment or false otherwise.
         */
        function getIdenticalComment( $topic, $text, $articleId, $parentId = 0, $userName = "", $userEmail = "", $userUrl = "", $clientIp = "0.0.0.0" )
        {
        	$query = "SELECT COUNT(*) AS total FROM ".$this->getPrefix()."articles_comments
                       WHERE topic = '".Db::qstr($topic)."' AND text = '".Db::qstr($text)."'
                       AND article_id = $articleId AND parent_id = $parentId AND
                       client_ip = '$clientIp' AND user_name = '$userName' AND
                       user_email = '$userEmail' AND user_url = '$userUrl'
                       AND date <= NOW()";

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;

            $row = $result->FetchRow();

            if( $row["total"] >= 1 )
            	return true;
            else
            	return false;
        }

        /**
         * code factored out and made to be reused through several functions here...
         */
        function _fillCommentInformation( $row )
        {
        	// ---
            // this stuff is another disgusting hack to make comments
            // be aware of time differences that may have been set by
            // users in their blogs... actually, it will make the whole
            // thing run a little bit slower because for every article
            // that we fetch, we need to know the time difference
            // and that means, additional accesses to the database :(
            // But hey, users demand these things and we are not in the position
            // to deny anything, are we? :P
            // ---
            // at the same time, we're going to brake some of our "nicely"
            // implemented bussiness logic with the query below, bypassing all
            // our objects and directly fetching info from the database, just for
            // the sake of being able to do things with a single query rather
            // than needing at least two to do the same. We would have to first
            // find to which blog the given article belongs and then use another
            // query to find its settings... If we do it this way, we can still
            // do it in one query and get a nice BlogSettings object with the
            // information that we need. Dirty but works :S
            // ---
            $prefix = $this->getPrefix();
            $date = $row["date"];
            $articleId = $row["article_id"];

            // let's try and keep some kind of cache for this...
            if( empty($this->_blogSettings[$articleId])) {
            	$query = "SELECT DISTINCT b.settings AS settings
                          FROM {$prefix}blogs b, {$prefix}articles a,
                          {$prefix}articles_comments c
                          WHERE c.article_id = a.id AND
                          a.blog_id = b.id
                          AND a.id = $articleId";
                $result = $this->_db->Execute( $query );
                $tmpRow = $result->FetchRow();
                $blogSettings = Blogs::getBlogSettingsFromField( $tmpRow["settings"] );
            }
            else {
            	$blogSettings = $this->_blogSettings[$articleId];
            }

            $timeDiff = $blogSettings->getValue( "time_offset" );
            // now that we've got the time difference, we can
            // calculate what would the "real" date...
            $date = Timestamp::getDateWithOffset( $date, $timeDiff );

        	$comment = new UserComment( $row["article_id"],
            							$row["parent_id"],
                                        $row["topic"],
                                        $row["text"],
                                        $date,
                                        $row["user_name"],
                                        $row["user_email"],
                                        $row["user_url"],
                                        $row["client_ip"],
                                        $row["spam_rate"],
                                        $row["status"],
                                        $row["id"] );

            return $comment;
        }

		/**
		 * Removes a comment from a post. It also updates all the other posts that
         * have this one as the parent and makes them look as if they were 'top level'
         * comments with no parent.
         *
         * @param artid The article identifier.
         * @param commentid The comment identifier.
		 */
		function deletePostComment( $artid, $commentid)
		{
			$query = "DELETE FROM ".$this->getPrefix()."articles_comments WHERE id = ".$commentid." AND article_id = ".$artid.";";

			$result = $this->_db->Execute( $query );

			if( $this->_db->Affected_Rows( $result ) == 0 )
				return;

            // update all the other posts
            $query = "UPDATE ".$this->getPrefix()."articles_comments SET parent_id = 0 WHERE parent_id = ".$commentid." AND article_id = ".$artid.";";
            $result = $this->_db->Execute( $query );

			return $result;
		}

        /**
         * Removes all the comments from a post.
         *
         * @param artId The article identifier
         */
        function deletePostComments( $artId )
        {
        	$query = "DELETE FROM ".$this->getPrefix()."articles_comments WHERE article_id = $artId";

			$result = $this->_db->Execute( $query );

            return $result;
        }

        /**
         * updates the status of a comment, regarding its spam status...
         *
         * @param commentId
         * @param status
         * @return true or false, depending...
         */
        function updateCommentStatus( $commentId, $status )
        {
        	$query = "UPDATE ".$this->getPrefix()."articles_comments SET
                      status = $status, date = date
                      WHERE id = $commentId";

            $result = $this->_db->Execute( $query );

            return $result;
        }

        /**
         * removes all comments marked as spam from the database
         */
        function purgeSpamComments()
        {
        	$query = "DELETE FROM ".$this->getPrefix()."articles_comments WHERE status = ".COMMENT_STATUS_SPAM;

            return $this->_db->Execute( $query );
        }
	}
?>
