<?php

	include_once( PLOG_CLASS_PATH."class/dao/model.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/article.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articlecategories.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articlenotifications.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articlecomments.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/users.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/trackbacks.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/timestamp.class.php" );
	include_once( PLOG_CLASS_PATH."class/config/config.class.php" );

    /**
     * different statuses that a post can have
     */
    define( "POST_STATUS_PUBLISHED", 1 );
    define( "POST_STATUS_DRAFT", 2 );
    define( "POST_STATUS_DELETED", 3 );

	/**
     * Model for the Articles
     */
	class Articles extends Model {

    	// DAO objects that we keep for later, so that we don't have
        // to create them eeeeevery time!
    	var $categories;
        var $comments;
        var $trackbacks;
        var $users;
        var $blogs;

		function Articles()
        {
        	$this->Model();

            //
            $this->categories = new ArticleCategories();
            $this->comments = new ArticleComments();
            $this->trackbacks = new Trackbacks();
            $this->users      = new Users();
            $this->blogs      = new Blogs();
        }

		/**
		 * Gets an article from the database, given its id
         *
         * @param artId Identifier of the article we want to fetch
         * @param userId If set, the article must belong to user identified by
         ** such id.
         * @return Returns an Article object or 'false' otherwise.
		 */
		function getUserArticle( $artId, $userId = -1 )
		{
			$query = "SELECT * FROM ".$this->getPrefix()."articles WHERE id = ".$artId;
            if( $userId != -1 )
            	$query .= " AND user_id = ".$userId;
            $query .= ";";

			// we send the query and then fetch the first array with the result
			$result = $this->_db->Execute( $query );

			if ( $result->RecordCount() == 0)
				return false;

			$row = $result->FetchRow( $result );

			$article = $this->_fillArticleInformation( $row );

			return $article;
		}

        /**
		 * Gets an article from the database, given its id
         *
         * @param artId Identifier of the article we want to fetch
         * @param blogId If set, the article must belong to the given blog
         * @return Returns an Article object or 'false' otherwise.
		 */
		function getBlogArticle( $artId, $blogId = -1 )
		{
			$query = "SELECT * FROM ".$this->getPrefix()."articles WHERE id = ".$artId;
            if( $blogId != -1 )
            	$query .= " AND blog_id = ".$blogId;
            $query .= ";";

			// we send the query and then fetch the first array with the result
			$result = $this->_db->Execute( $query );

            if( $result == false )
            	return false;

			if ( $result->RecordCount() == 0)
				return false;

			$row = $result->FetchRow( $result );

			$article = $this->_fillArticleInformation( $row );

			return $article;
		}

        /**
         * Returns the article that goes after the one we give as the parameter.
         *
         * @param articleId The Article identifier.
         * @param blogId The blog to which the article belongs.
         * @return An Article object with the next article or false if there was no article.
         */
        function getBlogNextArticle( $articleId, $blogId )
        {
        	// fetch the article
            $article = $this->getBlogArticle( $articleId, $blogId );
            if( !$article )
            	return false;

            // gets the article that is just next in time
        	$query = "SELECT * FROM ".$this->getPrefix()."articles WHERE date > ".$article->getDate()." AND status = 'PUBLISHED' AND blog_id = ".$blogId." ORDER BY date ASC LIMIT 1;";
            $result = $this->_db->Execute( $query );
            if( !$result )
            	return false;
            if( $result->RecordCount() == 0 )
            	return false;

            // get the information from the article
            $row = $result->FetchRow();

            return $this->_fillArticleInformation( $row );
        }

        /**
         * Returns the article that goes before the one we give as the parameter.
         *
         * @param articleId The Article identifier.
         * @param blogId The blog to which the article belongs.
         * @return An Article object with the next article or false if there was no article.
         */
        function getBlogPrevArticle( $articleId, $blogId )
        {
        	// fetch the article
            $article = $this->getBlogArticle( $articleId, $blogId );
            if( !$article )
            	return false;

            // gets the article that is just next in time
        	$query = "SELECT * FROM ".$this->getPrefix()."articles WHERE date < ".$article->getDate()." AND status = 'PUBLISHED' AND blog_id = ".$blogId." ORDER BY date DESC LIMIT 1;";
            $result = $this->_db->Execute( $query );
            if( !$result )
            	return false;
            if( $result->RecordCount() == 0 )
            	return false;

            // get the information from the article
            $row = $result->FetchRow();

            return  $this->_fillArticleInformation( $row );
        }

        /**
         * Returns all the articles for a given blog, according to the conditions specified in the call. If
		 * this function is too cumbersome to use (I reckon it might be, too many parameters that have been
		 * added over time due to new requirements) you may want to have a look at getBlogArticlesByQuery below.
		 * It's some kind of straight-through method that allows free-form SQL queries to be passed.
         *
         * @param blogId Identifier of the blog from where we want to fetch the articles
		 * @param date date in MySQL TIMESTAMP(14) format
		 * @param amount The maximum amount of posts that we would like to be returned.
		 * @param categoryId A category identifier. If specified, only the posts of the given category will be returned
		 * @param status If specified, only the posts with given status will be returned.
		 * @param userId If specified, only the posts that belong to the specified user will be returned
		 * @param maxDate a date in MySQL TIMESTAMP(14)
         * @return Returns an array with all the articles from the given blog
         */
        function getBlogArticles( $blogid, $date = -1, $amount = -1, $categoryId = 0, $status = 0, $userId = 0, $maxDate = 0 )
        {
        	// build the query

            // the query gets quite complicated to build because we have to take care of plenty
            // of conditions, such as the maximum date, the amount, the category,
            // wether the category has to be shown in the main page or not, etc...
            if( $status == POST_STATUS_PUBLISHED )
            	$postStatus = "PUBLISHED";
            elseif( $status == POST_STATUS_DRAFT )
            	$postStatus = "DRAFT";
            elseif( $status == POST_STATUS_DELETED )
            	$postStatus = "DELETED";

			$query = "SELECT a.id,a.category_id,a.topic,a.text,a.date,
                             a.user_id,a.blog_id,a.status,a.properties,
                             a.num_reads, c.id AS c_id FROM ".$this->getPrefix()."articles a, ".$this->getPrefix()."articles_categories c
                      WHERE a.blog_id = $blogid";
            if( $date != -1 )
            	$query .= " AND a.date+0 LIKE '$date%'";

            // the common part "c.id = a.category_id" is needed so that
            // we don't get one article row as many times as the amount of categories
            // we have... due to the sql 'join' operation we're carrying out
            if( $categoryId == -1 )
            	$query .=" AND c.id = a.category_id";
            else {
            	if( $categoryId > 0 )
                	$query .= " AND a.category_id = $categoryId AND c.id = a.category_id";
                else
            		$query .= " AND c.id = a.category_id AND c.in_main_page = 1";
            }

            if( $status > 0 )
            	$query .= " AND a.status = '$postStatus'";
            if( $userId > 0 )
            	$query .= " AND a.user_id = $userId";
            if( $maxDate > 0 )
            	$query .= " AND a.date <= '$maxDate'";
            $query .= " ORDER BY a.date DESC";
            // we don't need limits if we're getting the posts for a given day
            if( ($amount > 0) && ($date == -1))
            	$query .= " LIMIT $amount;";

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return Array();

            $articles = Array();
            while( $row = $result->FetchRow()) {
				$article = $this->_fillArticleInformation( $row );
                array_push( $articles, $article );
            }

            return $articles;
        }
		
        /**
         * Returns all the articles for a given blog (takes a query string as an argument).
		 *
		 * @param query A valid SQL query that will be used to generate a list of articles to fetch.
		 * @return Returns an Array filled with Article objects or an empty array if none could be found or
		 * the query was invalid.
         */
        function getBlogArticlesByQuery( $query )
        {
            $result = $this->_db->Execute( $query );
            if( !$result ) return Array();
            $articles = Array();
            while( $row = $result->FetchRow()) {
				$article = $this->_fillArticleInformation( $row );
				array_push( $articles, $article );
            }
            return $articles;
		}			

        /**
         * Gets the number of posts per month per year from the database
         *
         * @param blogId The numeric identifier of the blog from which we'd like to calculate this values
         * @return A 2-dimensional associative array where the first index is the year and the second
         * index is the number of the month: result[2003][11] = _posts for november 2003_
         *
         * @param blogId
         * @param showAll Whether to include posts in the future and months that had no posts too
         */
        function getNumberPostsPerMonth( $blogId )
        {
        	// query to get the earliest post
            // this must be my longest and most complex SQL query ever :)
			$config =& Config::getConfig();
			if( $config->getValue( "show_future_posts_in_calendar" )) {
				$numPostsPerMonthQuery = "SELECT COUNT(*) AS 'count', YEAR(date) AS 'year',
				                          MONTH(date) AS 'month', DAYOFMONTH(date) AS 'daymonth' 
				                          FROM ".$this->getPrefix()."articles 
				                          WHERE status = 'PUBLISHED' AND blog_id = $blogId 
				                          GROUP BY YEAR(date), MONTH(date) 
				                          ORDER BY YEAR(date) DESC,MONTH(date) DESC;";
			}	
			else {
				$numPostsPerMonthQuery = "SELECT COUNT(*) AS 'count', YEAR(date) AS 'year',
				                          MONTH(date) AS 'month', DAYOFMONTH(date) AS 'daymonth' 
				                          FROM ".$this->getPrefix()."articles 
				                          WHERE status = 'PUBLISHED' AND blog_id = $blogId 
				                          AND date <= NOW() GROUP BY YEAR(date), MONTH(date) 
				                          ORDER BY YEAR(date) DESC,MONTH(date) DESC;";
			}	

            $result = $this->_db->Execute( $numPostsPerMonthQuery);
            if( $result == false )
            	return false;

            $archives = Array();
            while( $row = $result->FetchRow()) {
            	$archives[$row["year"]][$row["month"]] = $row["count"];
            }

            return $archives;
        }
        
        /**
         * like the one above but with a few changes, such as always showing posts in the future
         * and returning all the months in the array, even if the total amount was '0'
         * Only used in the "editPosts" screen of the admin interface
         */
        function getNumberPostsPerMonthAdmin( $blogId )
        {
        	// query to get the earliest post
            // this must be my longest and most complex SQL query ever :)
			$numPostsPerMonthQuery = "SELECT COUNT(*) AS 'count', YEAR(date) AS 'year',
			                          MONTH(date) AS 'month', DAYOFMONTH(date) AS 'daymonth' 
			                          FROM ".$this->getPrefix()."articles 
			                          WHERE blog_id = $blogId 
			                          GROUP BY YEAR(date), MONTH(date) 
			                          ORDER BY YEAR(date) DESC,MONTH(date) DESC;";

            $result = $this->_db->Execute( $numPostsPerMonthQuery);
            if( $result == false )
            	return false;
            	
            while( $row = $result->FetchRow()) {
            	$archives[$row["year"]][$row["month"]] = $row["count"];
            }

            return $archives;
        }
        

        /**
         * The same as the one above but for just one month
         *
         * @param blogId The identifier of the blog from which we'd like to calculate this
         * @param year Year
         * @param month Month from which we'd like to calculate this
         * @return An associative array where the index is the day of the month and the value
         * is the number of posts made that day.
         */
        function getNumberPostsPerDay( $blogId, $year = null , $month = null )
        {
        	// if month and/or year are empty, get the current ones
			if( $year == null ) {
            	$t = new Timestamp();
                $year = $t->getYear();
            }
            if( $month == null ) {
            	$t = new Timestamp();
                $month = $t->getMonth();
            }
            
        	// another long sql query :) the id and date fields are there just in case we need to debug
            // but they're not included in the resulting array
			$config =& Config::getConfig();
			// check whether we're supposed to show posts that happen in the future or not
			if( $config->getValue( "show_future_posts_in_calendar" ))
				$numPostsPerDayQuery = "SELECT COUNT(*) AS 'count', DAYOFMONTH(date) AS 'day', id AS 'id', date AS 'date' FROM ".$this->getPrefix()."articles WHERE status = 'PUBLISHED' AND blog_id = ".$blogId." AND MONTH(date) = ".$month." AND YEAR(date) = ".$year." GROUP BY DAYOFMONTH(date);";
			else
				$numPostsPerDayQuery = "SELECT COUNT(*) AS 'count', DAYOFMONTH(date) AS 'day', id AS 'id', date AS 'date' FROM ".$this->getPrefix()."articles WHERE status = 'PUBLISHED' AND blog_id = ".$blogId." AND date <= NOW() AND MONTH(date) = ".$month." AND YEAR(date) = ".$year." GROUP BY DAYOFMONTH(date);";
			

            $result = $this->_db->Execute( $numPostsPerDayQuery );

            if( $result == false )
            	return 0;

            $postPerDay = Array();
            while( $row = $result->FetchRow()) {
            	$postsPerDay[$row["day"]] = $row["count"];
            }

            return $postsPerDay;
        }

        /**
		 * Adds a new article to the database
		 *
		 * @param newArticle An Article object with all the necessary information.
		 * @return Returns true if article was added successfully or false otherwise.
		 */
		function addArticle( $newArticle )
		{
			// first, we build up the query
            $query = "INSERT INTO ".$this->getPrefix()."articles( category_id,topic,text,user_id,blog_id,status,date,properties) VALUES (".$newArticle->getCategoryId().",'".Db::qstr($newArticle->getTopic())."','".Db::qstr($newArticle->getText(false))."',".$newArticle->getUser().",".$newArticle->getBlog().",'".$newArticle->getStatus()."', '".$newArticle->getDate()."','".serialize($newArticle->getProperties())."')";
			// and then we send it to the db
            //$this->_db->debug=true;
			$result = $this->_db->Execute( $query );

			if( !$result )
				return false;

			// get id of the post from the database
			$postId = $this->_db->Insert_ID();
			
			return $postId;
		}

        /**
		 * Updates an article in the database
		 *
		 * @param article The Article object that we'd like to update in the database.
		 * @return Returns true if update was successful or false otherwise.
		 */
		function updateArticle( $article )
		{
			// we have to build up the query, which will be pretty long...
			$articleDate = Timestamp::getDateWithOffset( $article->getDate(), -($article->getTimeOffset()));
			
			$query = "UPDATE ".$this->getPrefix()."articles SET id = ".$article->getId().
			         ", topic = '".Db::qstr($article->getTopic())."'".
			         ", text = '".Db::qstr($article->getText(false))."'".
			         ", user_id = ".$article->getUser().
			         ", date = ".$articleDate.
			         ", category_id = ".$article->getCategoryId().
                     ", blog_id = ".$article->getBlog().
                     ", status = '".$article->getStatus()."'".
                     ", num_reads = ".$article->getNumReads().
                     ", properties = '".serialize($article->getProperties())."'".
			         " WHERE id = ".$article->getId().";";
					 
			$result = $this->_db->Execute( $query );

			return $result;
		}

        /**
		 * Updates the number of times a post has been read. This method does not just increase the num_read 
		 * counter by one but it can set it to whatever we want... Usually the value we pass in '$numReads' will
		 * be the old value + 1, but it could be whatever.
		 *
		 * @param articleId A valid article identifier.
		 * @param numReads A value, meaning how many times the post has been read.
		 * @return Returns true if successful or false otherwise.
		 */
		function updateArticleNumReads( $articleId, $numReads )
		{
			// we have to build up the query, which will be pretty long...
			$query = "UPDATE ".$this->getPrefix()."articles SET ".
                     " num_reads = $numReads ".
                     ", date = date ".
                     " WHERE id = $articleId;";

			$result = $this->_db->Execute( $query );

			return $result;
		}

        /**
		 * Removes an article from the database
         *
         * If forever == true, the article is physically removed from the database.
         * Otherwise, the 'status' field is set to 'deleted'
         *
         * Problem is, that MySQL will automatically update the 'date' field because he feels
         * like it... even if we explicitely say date = old_date... grrreat :P
		 *
		 * Valid article identifier, blog identifier and user identifier are required to remove an
		 * article. It was done for security reasons and to make perfectly clear that we are removing
		 * an article (so that we wouldn't be deleting the wrong one if there was any bug!)
		 *
		 * @param artId A valid article identifier
		 * @param userid A valid user identifier
		 * @param blogId A valid blog identifier
		 * @param forever A boolean meaning whether the post should be removed forever or simply its status
		 * should be set to 'deleted'
		 * @return Returns true if successful or false otherwise.
		 */
		function deleteArticle( $artId, $userId, $blogId, $forever = false )
		{
			if( $forever ) {
				$query = "DELETE FROM ".$this->getPrefix()."articles WHERE id = ".$artId." AND user_id = ".$userId." AND blog_id = ".$blogId.";";
                // we also have to remove its comments and trackbacks if the article is being deleted forever
                // -- comments --
                $comments = new ArticleComments();
                $comments->deletePostComments( $artId );
                // -- trackbacks --
                $trackbacks = new Trackbacks();
                $trackbacks->deletePostTrackbacks( $artId );
            }
            else {
				$query = "UPDATE ".$this->getPrefix()."articles SET date = date, status = 'DELETED' WHERE id = ".$artId." AND user_id = ".$userId." AND blog_id = ".$blogId.";";
            }

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;
				
			if( $this->_db->Affected_Rows() == 0 )
				return false;

			return true;
		}

        /**
		 * returns true if the user posted this article/post, or false
		 * if he or she doesn't own it
		 *
		 * @param userInfo A UserInfo object with information about the user
		 * @param artId A valid article identifier
		 * @return Returns true if the article is owned by the user or false otherwise.
		 */
		function doesUserOwnArticle( $userinfo, $artid )
		{
			$article = $this->getArticle( $artid );
			$articleOwnerInfo = $article->getUserInfo();

			if( $articleOwnerInfo->getId() == $userinfo->getId())
				return true;
			else
				return false;
		}

        /**
         * Gets all the posts for given blog in a given day
         *
         * @deprecated
         * <b>NOTE:</b> Deprecated!
         */
        function getPostsPerDay( $blogId, $daystamp )
		{
			$query = "SELECT * FROM ".$this->getPrefix()."articles WHERE blog_id = ".$blogId." AND date+0 LIKE '".$daystamp."%' ORDER BY date DESC;";

			$result = $this->_db->Execute( $query );

			$articles = array();
			while( $row = $result->FetchRow( $result ))
				array_push( $articles, $this->_fillArticleInformation( $row ));

			return $articles;
		}

        /**
         * Returns all the posts in the database or the 'maxPosts' recent ones that have been
         * published (status=='published')
         *
         * @param maxPosts The amount of post we want.
         * @return Returns an array of Article objects containing the information about the posts.
         */
        function getAllArticles( $maxPosts = 0, $date = 0)
        {
        	$query = "SELECT * FROM ".$this->getPrefix()."articles WHERE status = 'PUBLISHED'";
            if( $date > 0 )
            	$query .= " AND date < '$date'";

            $query .= " ORDER BY date DESC";

            if( $maxPosts > 0 )
            	$query .= " LIMIT 0,".$maxPosts;

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;

            $posts = Array();
            while( $row = $result->FetchRow()) {
            	array_push( $posts, $this->_fillArticleInformation($row));
            }

            return $posts;
        }

        /**
         * Removes all the posts from the given blog
         *
         * @param blogId The blog identifier
         */
        function deleteBlogPosts( $blogId )
        {
            $blogArticles = $this->getBlogArticles( $blogId );

            foreach( $blogArticles as $article ) {
            	// the deleteArticle method will also take care of removing comments and
                // trackbacks
            	$this->deleteArticle( $article->getId(), $article->getUser(), $article->getBlog(), true );
            }
			
			return true;
        }

        /**
         * removes all posts that have 'deleted' status
		 *
		 * @return Returns true if all posts were deleted or false otherwise.
         */
        function purgePosts()
        {
        	$query = "SELECT * FROM ".$this->getPrefix()."articles WHERE status = 'deleted'";

            $result = $this->_db->Execute( $query );
            if( !$result )
            	return false;

            while( $row = $result->FetchRow()) {
            	$this->deleteArticle( $row["id"], $row["user_id"], $row["blog_id"], true );
            }

            return true;
        }

		/**
		 * Does exactly the same as _fillUserInformation, so that it can be reused
		 * by the different functions that work on articles from the database. This method is
		 * private and shouldn't be used outside this class.
		 *
		 * @private
		 */
		function _fillArticleInformation( $query_result )
		{
        	// this is a little dirty trick or otherwise the old
            // that don't have the 'properties' field will not work
            // as they will appear to have comments disabled
        	if( $query_result["properties"] == "" ) {
                $tmpArray = Array( "comments_enabled" => true );
            	$query_result["properties"] = serialize($tmpArray);
            }

            // ---
            // this, i do not like... but I couldn't find a more
            // "elegant" way to arrange it! This makes this method
            // totally dependant on the blog configuration so it basically
            // means an additional query every time we fetch an article
            // (just in case we didn't have enough!)
            // ---

            //
            // if there's a time difference applied to all dates, then we'd better
            // calculate it here!!
            //
            $blogId = $query_result["blog_id"];
            $blogInfo = $this->blogs->getBlogInfo( $blogId );
            $blogSettings = $blogInfo->getSettings();
            $timeDiff = $blogSettings->getValue( "time_offset" );

            // we can use this auxiliary function to help us...
            $date = Timestamp::getDateWithOffset( $query_result["date"], $timeDiff );

			// create a dummy article
			$article = new Article( $query_result["topic"],
									$query_result["text"],
									$query_result["category_id"],
									$query_result["user_id"],
									$query_result["blog_id"],
									$query_result["status"],
                                    $query_result["num_reads"],
                                    unserialize($query_result["properties"]),
                                    $query_result["id"] );
			// and fill in all the fields with the information we just got from the db
			$article->setDate( $date );
			$article->setTimeOffset( $timeDiff );

            // get information about the category of the article
            $article->setCategory( $this->categories->getCategory( $article->getCategoryId()));
            $article->setUserInfo( $this->users->getUserInfoFromId( $query_result["user_id"] ));
            $article->setNumTrackbacks( $this->trackbacks->getNumTrackbacksArticle($article->getId()));
            $article->setTotalComments( $this->comments->getNumberOfComments($article->getId()));
            $article->setBlogInfo( $blogInfo );

			return $article;
		}

        /**
         * sets a time difference that will be added to each
         * date and time
         *
         * @param timeDiff The time difference. IT should be between +12 and -12
		 * @return Always returns true.
         */
        function setTimeDiff( $timeDiff )
        {
        	$this->_timeDiff = $timeDiff;
			
			return true;
        }
        
        function getMostCommentedArticles( $maxPosts = 0, $date = 0)
        {
			$query = " SELECT COUNT(*) as total_comments, a.* FROM ".$this->getPrefix()."articles_comments AS c "
			       . " INNER JOIN ".$this->getPrefix()."articles AS a ON c.article_id=a.id WHERE a.status = 'PUBLISHED' ";
				   
            if( $date > 0 )
            	$query .= " AND c.date > '$date'";

            $query .= "GROUP BY c.article_id ORDER BY total_comments DESC ";

            if( $maxPosts > 0 )
            	$query .= " LIMIT 0,".$maxPosts;

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;

            $posts = Array();
            while( $row = $result->FetchRow()) {
            	array_push( $posts, $this->_fillArticleInformation($row));
            }

            return $posts;
        }

        function getMostReadArticles( $maxPosts = 0, $date = 0)
        {
			$query = " SELECT * FROM ".$this->getPrefix()."articles WHERE status = 'PUBLISHED' ";
				   
            if( $date > 0 )
            	$query .= " AND date > '$date'";

            $query .= " ORDER BY num_reads DESC ";

            if( $maxPosts > 0 )
            	$query .= " LIMIT 0,".$maxPosts;

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;

            $posts = Array();
            while( $row = $result->FetchRow()) {
            	array_push( $posts, $this->_fillArticleInformation($row));
            }

            return $posts;
        }        
    }
?>