<?php
	include_once( PLOG_CLASS_PATH."class/object/object.class.php" );
	include_once( PLOG_CLASS_PATH."class/dao/blogsettings.class.php" );

	/**
	 * This is the representation of a blog. It contains all the information we need to know,
     * such as the name of the blog, the owner, description, etc.
	 */
	class BlogInfo extends Object 
	{

		var $_blog;
		var $_owner;
		var $_about;
		var $_settings;
		var $_id;
		
        // More optional information for each blog. Only used when we load this data
        var $_createDate;
        var $_updateDate;
        var $_totalPosts;
        var $_viewedTotal;
        var $_ownerInfo;
        var $_usersInfo;        
        var $_createTimestamp;        
        var $_updateTimestamp;        		

		function BlogInfo( $blog, $owner, $about, $settings, $id = -1 )
		{
			$this->_blog = $blog;
			$this->_owner = $owner;
			$this->_about = $about;
			$this->_settings = $settings;
			$this->_id = $id;
			
			// settings for the extended information
            $this->_createDate = "";
            $this->_updateDate = "";
            $this->_totalPosts = 0;
            $this->_viewedTotal = 0;
            $this->_ownerInfo = "";
            $this->_usersInfo = array();
            $this->_createTimestamp = "";
            $this->_updateTimestamp = "";			
		}

        /**
         * Returns the short name of the blog.
         *
         * @return A string with the short name of the blog.
         */
		function getBlog()
		{
			return $this->_blog;
		}


        /**
         * Returns the identifier of the user who owns this journal.
         *
         * @return An integer value representing the identifier of the user who owns this blog.
         */
		function getOwner()
		{
			return $this->_owner;
		}

        /**
         * Returns a longer and descriptive text about this blog. It can also be empty since it is
         * configurable from within the "Blog Settings" in the administration interface.
         *
         * @return A string containing the more descriptive text about the journal.
         */
		function getAbout()
		{
			return $this->_about;
		}

        /**
         * Returns a BlogSettings object with information about the settings of the journal.
         * @private
         */
		function getSettings()
		{
			return $this->_settings;
		}

         /**
         */
		function getCreateDateObject( )
		{
			return $this->_createTimestamp;
		}

        /**
         */
		function getUpdateDateObject( )
		{
			return $this->_updateTimestamp;
		}

        /**
         */
		function getTotalPosts( )
		{
			return $this->_totalPosts;
		}

        /**
         */
		function getViewedTotal( )
		{
			return $this->_viewedTotal;
		}

        /**
         */
		function getOwnerInfo( )
		{
			return $this->_ownerInfo;
		}

        /**
         * @private
         */
		function getUsersInfo( )
		{
			return $this->_usersInfo;
		}

        /**
         * Returns the identifier of this blog.
         *
         * @return An integer value with the identifier of this blog.
         */
		function getId()
		{
			return $this->_id;
		}

        /**
         * @private
         */
		function setBlog( $blog )
		{
			$this->_blog = $blog;
		}

        /**
         * @private
         */
		function setOwner( $owner )
		{
			$this->_owner = $owner;
		}

        /**
         * @private
         */
		function setAbout( $about )
		{
			$this->_about = $about;
		}

        /**
         * @private
         */
		function setSettings( $settings )
		{
			$this->_settings = $settings;
		}

        /**
         * @private
         */
		function setCreateDate( $newCreateDate )
		{
			$this->_createDate = $newCreateDate;

			$this->_createTimestamp = new Timestamp( $newCreateDate );
		}

        /**
         * @private
         */
		function setUpdateDate( $newUpdateDate )
		{
			$this->_updateDate = $newUpdateDate;

			$this->_updateTimestamp = new Timestamp( $newUpdateDate );
		}

        /**
         * @private
         */
		function setTotalPosts( $newTotalPosts )
		{
			$this->_totalPosts = $newTotalPosts;
		}

        /**
         * @private
         */
		function setViewedTotal( $newViewedTotal )
		{
			$this->_viewedTotal = $newViewedTotal;
		}

        /**
         * @private
         */
		function setOwnerInfo( $newOwnerInfo )
		{
			$this->_ownerInfo = $newOwnerInfo;
		}

        /**
         * @private
         */
		function setUsersInfo( $newUsersInfo )
		{
			$this->_usersInfo = $newUsersInfo;
		}

        /**
         * @private
         */
		function setId( $id )
		{
			$this->_id = $id;
		}
    }
?>
