<?php

    include_once( PLOG_CLASS_PATH."class/dao/model.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/bloginfo.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/blogsettings.class.php" );
    include_once( PLOG_CLASS_PATH."class/template/templateutils.class.php" );
    include_once( PLOG_CLASS_PATH."class/config/config.class.php" );
    include_once( PLOG_CLASS_PATH."class/net/xmlrpcclient.class.php" );
	include_once( PLOG_CLASS_PATH."class/dao/articles.class.php" );
	include_once( PLOG_CLASS_PATH."class/dao/mylinks.class.php" );
	include_once( PLOG_CLASS_PATH."class/dao/mylinkscategories.class.php" );
	include_once( PLOG_CLASS_PATH."class/dao/bayesianfilterinfos.class.php" );
	include_once( PLOG_CLASS_PATH."class/dao/users.class.php" );
	include_once( PLOG_CLASS_PATH."class/dao/userpermissions.class.php" );

    /**
     * Model for the Blogs
     */
	class Blogs extends Model 
	{

		/**
		 * Returns information about a blog.
         *
         * @param blogId Identifier of the blog we want to get information
         * @return Returns a BlogInfo object containing information about the blog
		 */
		function getBlogInfo( $blogId )
		{
			$query = "SELECT * FROM ".$this->getPrefix()."blogs WHERE id = ".$blogId.";";

			$result = $this->_db->Execute( $query );

			if( !$result )
				return false;

            if( $result->RecordCount() == 0 )
            	return false;

			$row = $result->FetchRow( $result );
			$blogInfo = $this->_fillBlogInformation( $row );
            $blogSettings = $this->getBlogSettingsFromField( $row["settings"] );
            $blogInfo->setSettings( $blogSettings );

			return $blogInfo;
		}

        /**
         * Private function.
         *
         * @private.
         */
        function _fillBlogInformation( $query_result, $extended = false )
        {
        	$blogInfo = new BlogInfo( stripslashes($query_result["blog"]),
                                      $query_result["owner_id"],
                                      stripslashes($query_result["about"]),
                                      unserialize($query_result["settings"]),
                                      $query_result["id"] );
             
            // this is used for example in the summary page...                          
            if ($extended)
            {
                $users = new Users();
                $userpermissions = new UserPermissions();
                
                include_once( PLOG_CLASS_PATH."class/dao/summarystats.class.php" );
                $stats = new SummaryStats();
                $blogInfo->setCreateDate( $stats->getBlogCreateDate( $blogInfo->getId() ) );
                $blogInfo->setUpdateDate( $stats->getBlogUpdateDate( $blogInfo->getId() ) );
                $blogInfo->setTotalPosts( $stats->getBlogTotalPosts( $blogInfo->getId() ) );
                $blogInfo->setViewedTotal( $stats->getBlogViewedTotal( $blogInfo->getId() ) );
                $blogInfo->setOwnerInfo( $users->getUserInfoFromId( $query_result["owner_id"] ) );
                $blogInfo->setUsersInfo( $userpermissions->getBlogUsersPermissions( $blogInfo->getId() ) );
            }                                      

            return $blogInfo;
        }
        
        /**
         * Retrieves the blog settings
         *
         * @param settingsField The contents of the blog_settings field from the database
         * @return Returns a BlogSettings object with the settings for the blog
         */
        function getBlogSettingsFromField( $settingsField )
        {
        	if( $settingsField == "" || $settingsField == null ) {
            	$settings = new BlogSettings();
            }
            else {
				$settings = unserialize( $settingsField );
                if( !is_object($settings)) {
                	// if the BlogSettings object is not valid, return a valid one
                    // with some of the default settings, so that at least
                    // the blog will keep working!
                	$settings = new BlogSettings();
                }
            }

            return $settings;
        }

        /**
         * Retrieves the blog settings given its id
         *
         * @param blogId The identifier of the blog from which we'd like to fetch the settings
         * @return A BlogSettings object containing the settings, or 'false' if error.
         */
        function getBlogSettings( $blogId )
        {
			$query = "SELECT settings FROM ".$this->getPrefix()."blogs WHERE id = ".$blogId;

            $result = $this->_db->Execute( $query );

            $row = $result->FetchRow();

            return $this->getBlogSettingsFromField( $row["settings"] );
        }

        /**
         * Updates the configuration of a blog
         *
         * @param blogId Blog identifier
         * @param blogInfo A BlogInfo object containing all the information of the blog
         * @param return Returns true if everything ok or false otherwise
         */
        function updateBlog( $blogId, $blogInfo )
        {
        	$query = "UPDATE ".$this->getPrefix()."blogs SET blog = '".Db::qstr($blogInfo->getBlog())."', about = '".Db::qstr($blogInfo->getAbout())."', settings = '".serialize($blogInfo->getSettings())."', owner_id = ".$blogInfo->getOwner()." WHERE id = ".$blogId.";";

            $result = $this->_db->Execute( $query );

            return $result;
        }

        /**
         * Updates the settings of a blog
         *
         * @param blogId The blog we want to update
         * @param blogSettings the BlogSettings object that we would like to update
         * @return 'true' if all correct. 'false' otherwise.
         */
         function updateBlogSettings( $blogId, $blogSettings )
         {
         	$query = "UPDATE ".$this->getPrefix()."blogs SET settings = '".serialize($blogSettings)."' WHERE id = ".$blogId;

            $result = $this->_db->Execute( $query );

            return $result;
         }

         /**
          * Sends a weblogsUpdate.ping xmlrpc call to notifiy of changes in this blog
          *
          * @param blogInfo The BlogInfo object containing information about the blog
          * @return Returns true if successful or false otherwise.
          */
         function updateNotify( $blogInfo )
         {
         	// if this feature is not enabled, we quit
            if( !$this->_config->getValue( "xmlrpc_ping_enabled" ))
            	return;

         	// get the array which contains the hosts
            $hosts = $this->_config->getValue( "xmlrpc_ping_hosts" );

            // if it is not an array, quit
            if( !is_array($hosts))
            	return;

            // if no hosts, we can quit too
            if( empty($hosts))
            	return;

            // otherwise, we continue
            $xmlrpcPingResult = Array();
            $rg = new RequestGenerator( $blogInfo );
            $blogLink = $rg->blogLink();
            foreach( $hosts as $host ) {
                //print("sending xmlrpc ping: host = ".$host." weblog = ".$blogInfo->getBlog()." link = ".TemplateUtils::blogLink( $blogInfo ));
            	$client = new XmlRpcClient( $host );
                $result = $client->ping( $blogInfo->getBlog(), $blogLink);
                //print("result = ".$result. "is Error = ".$client->isError()." message: ".$client->getErrorMessage()."<br/>");
                //$xmlrpcPingResult[$result["host"]
                $xmlrpcPingResult=array_merge($xmlrpcPingResult, $result);
            }

            return $xmlrpcPingResult;
         }

         /**
          * Adds a new blog to the database.
          *
          * @param name Short descriptive name of the blog.
          * @param ownerId Identifier of the user who owns the blog.
          * @param about Longer text describing the blog.
          * @return False if unsuccessful or true otherwise.
          */
         function addBlog( $name, $ownerId, $about ="" )
         {
         	$blogSettings = new BlogSettings();
         	$query = "INSERT INTO ".$this->getPrefix()."blogs (blog,owner_id,about,settings) ".
                     "VALUES ('".Db::qstr($name)."',$ownerId,'".Db::qstr($about)."', '".serialize($blogSettings)."');";

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;

            $query = "SELECT id FROM ".$this->getPrefix()."blogs WHERE blog = '".Db::qstr($name)."' AND owner_id = ".$ownerId.";";
            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;
				
            $row = $result->FetchRow();
            $blogId = $row["id"];

			// create the row for the bayesian filter info
			$bayesianFilterInfo = new BayesianFilterInfos();
			$bayesianFilterInfo->insert( $blogId );
			
			// and return the blog identifier
			return $blogId;
         }

         /**
          * Returns all the blogs defined for the site in an array, sorted by its
          * blog identifier.
          *
          * @return Returns an array with all the blogs defined for this site. The array
          * is sorted by the blog identifier, so that $blogs[$blogId] will give us the information
          * of the blog with $blogId as its identifier.
          */
         function getAllBlogs( $extended = false )
         {
         	$query = "SELECT * FROM ".$this->getPrefix()."blogs";

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;

            $blogs = Array();
            while( $row = $result->FetchRow()) {
            	$blog = $this->_fillBlogInformation( $row, $extended );
                $blogs[$blog->getId()] = $blog;
            }

            return $blogs;
        }

        /**
         * Removes a blog from the database. It also removes all its posts, its posts categories
         * its links, its links categories, its trackbacks and its comments
         *
         * @param blogId the id of the blog we'd like to delete
         */
        function deleteBlog( $blogId )
        {
        	// first of all, delete the posts
            $articles = new Articles();
            $articles->deleteBlogPosts( $blogId );

            // next is to remove the article categories
            $articleCategories = new ArticleCategories();
            $articleCategories->deleteBlogCategories( $blogId );

            // next, all the links and links categories
            $myLinks = new MyLinks();
            $myLinks->deleteBlogMyLinks( $blogId );
            $myLinksCategories = new MyLinksCategories();
            $myLinksCategories->deleteBlogMyLinksCategories( $blogId );

            // the permissions for the blog
            $perms = new UserPermissions();
            $perms->revokeBlogPermissions( $blogId );

            // and finally, delete the blog
        	$query = "DELETE FROM ".$this->getPrefix()."blogs WHERE id = $blogId";

            $result = $this->_db->Execute( $query );

            return $result;
        }        
     }
?>