<?php

	include_once( PLOG_CLASS_PATH."class/dao/model.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/mylinkscategory.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/mylinks.class.php" );

    /**
     * Model for retrieving MyLinksCategory objects from the database
     */
	class MyLinksCategories extends Model {

        /**
         * Returns the categories for my_links for a given blog
         *
         * @param blogId Identifier of the blog.
         */
        function getMyLinksCategories( $blogId )
        {
			$query = "SELECT * FROM ".$this->getPrefix()."mylinks_categories WHERE blog_id = ".$blogId;

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;

            $myLinksCategories = Array();
            while( $row = $result->fetchRow($result)) {
				$myLinksCategory = $this->_fillMyLinksCategoryFromRow( $row );
                $myLinksCategories[$myLinksCategory->getId()] = $myLinksCategory;
            }

            return $myLinksCategories;
        }

        /**
         * Adds a category to the database of link categories.
         *
         * @param blogId Identifier of the blog.
         * @param linkCategory A MyLinkCategory object with the information we need.
         * @return Returns true if successful or false otherwise.
         */
        function addMyLinksCategory( $myLinksCategory, $blogId )
        {
        	$query = "INSERT INTO ".$this->getPrefix()."mylinks_categories (name, blog_id) VALUES ('".Db::qstr($myLinksCategory->getName())."', ".$blogId.");";

            $result = $this->_db->Execute( $query );

            return $result;
        }

        /**
         * Removes a link category from the database.
         *
         * @param categoryId Category identifier.
         * @param blogId Blog identifier.
         * @return True if successful or false otherwise.
         */
        function deleteMyLinksCategory( $categoryId, $blogId )
        {
        	$query = "DELETE FROM ".$this->getPrefix()."mylinks_categories WHERE id = ".$categoryId." AND blog_id = ".$blogId.";";

            $result = $this->_db->Execute( $query );

            return $result;
        }

        /**
         * Removes all the links categories from the given blog
         *
         * @param blogId The blog identifier
         */
        function deleteBlogMyLinksCategories( $blogId )
        {
        	$query = "DELETE FROM ".$this->getPrefix()."mylinks_categories WHERE blog_id = $blogId";

			$result = $this->_db->Execute( $query );

            return $result;
        }        

        /**
         * Retrieves a single link category from the database
         *
         * @param categoryId Category identifier.
         * @param blogId Blog identifier.
         * @return The MyLinksCategory object containing the information.
         */
        function getMyLinksCategory( $categoryId, $blogId = 0 )
        {
        	$query = "SELECT * FROM ".$this->getPrefix()."mylinks_categories WHERE id = ".$categoryId;
            if( $blogId > 0 )
            	$query .= " AND blog_id = ".$blogId;
            $query .= ";";

            $result = $this->_db->Execute( $query );

            if( !$result || $result->RecordCount() == 0 )
            	return false;

            $row = $result->FetchRow();
            $myLinksCategory = $this->_fillMyLinksCategoryFromRow( $row );

            return $myLinksCategory;
        }

        /**
         * Updates a category.
         *
         * @param category The MyLinkCategory object we're trying to update.
         * @retun True if successful or false otherwise.
         */
        function updateMyLinksCategory( $category )
        {
        	$query = "UPDATE ".$this->getPrefix()."mylinks_categories SET
                      name = '".Db::qstr($category->getName())."'
                      WHERE id = ".$category->getId()." AND blog_id = ".$category->getBlogId().";";

            $result = $this->_db->Execute( $query );

            return $result;
        }

        /**
         * Returns how many links are under the given category.
         *
         * @param categoryId the category identifier.
         * @return The number of links under this category or 0 if none.
         */
        function getNumMyLinksCategory( $categoryId )
        {
        	$query = "SELECT COUNT(*) AS total FROM ".$this->getPrefix()."mylinks WHERE category_id = ".$categoryId.";";

            $result = $this->_db->Execute( $query );
            if( !$result )
            	return 0;

            $row = $result->FetchRow();
            return $row["total"];
        }

        function _fillMyLinksCategoryFromRow( $row )
        {
        	$myLinksCategory = new MyLinksCategory( $row["name"],
                                                    $row["blog_id"],
                                                    $this->getNumMyLinksCategory($row["id"]),
                                                    $row["id"] );

            $myLinks = new MyLinks();
            $categoryLinks = $myLinks->getLinks( $row["blog_id"], $row["id"] );
            $myLinksCategory->setLinks( $categoryLinks );

            return $myLinksCategory;
        }
    }
?>