<?php

	include_once( PLOG_CLASS_PATH."class/dao/model.class.php" );
    include_once( PLOG_CLASS_PATH."class/config/config.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/referer.class.php" );
    include_once( PLOG_CLASS_PATH."class/net/url.class.php" );

    /**
     * Data access class (or model) to handle all the referers.
     */
    class Referers extends Model {

    	var $_enabled;

    	/**
         * Constructor.
         */
    	function Referers()
        {
        	$this->Model();

            $config =& Config::getConfig();
            $this->_enabled = $config->getValue( "referer_tracker_enabled" );
        }

        /**
         * Adds a referer to the database.
         *
         * @param refererHeader The referer header as was received by PHP.
         * @param articleId The article being hit by this referer.
         * @return Returns true if successful or false otherwise.
         */
        function addReferer( $refererHeader, $articleId, $blogId )
        {
        	// quit inmediately if this is not enabled...
        	if( !$this->_enabled )
            	return;

        	// we only add a new referer if we come from somewhere else than our own server
            $ourHost = $_SERVER["HTTP_HOST"];

            $refererUrl = new Url( $refererHeader );

            // if they're the same, we quit
            if( $refererUrl->getHost() == $ourHost || $refererUrl->getHost() == "" ) {
            	return;
            }

            // we have to check if a referer with that information exists
            // in the database
            $query = "UPDATE ".$this->getPrefix()."referers SET hits = hits + 1 WHERE url = '".$refererHeader."' AND article_id = ".$articleId." AND blog_id = ".$blogId.";";
            $result = $this->_db->Execute( $query );
            if( !$result )
            	return false;

            // check how many rows were updated this time.
            if( $this->_db->Affected_Rows() == 0 ) {
            	// we have to insert the row manually
                $query2 = "INSERT INTO ".$this->getPrefix()."referers (url,article_id,blog_id) VALUES ('".$refererHeader."',".$articleId.",".$blogId.");";
                $result2 = $this->_db->Execute( $query2 );
            }

            return true;
        }

        /**
         * Reads all the referers for a given article.
         *
         * @param articleId The identifier of the article from which we want to know
         * the referers.
         * @return An array of Referer objects with the information about all the referers
         * for this article, or false if the article does not have any referer.
         */
        function getArticleReferers( $articleId )
        {
        	$query = "SELECT * FROM ".$this->getPrefix()."referers WHERE article_id = ".$articleId.";";

            $result = $this->_db->Execute( $query );
            if( !$result )
            	return false;

            $referers = Array();
            while( $row = $result->FetchRow()) {
            	array_push( $referers, $this->_fillRefererInformation( $row ));
            }

            return $referers;
        }

        /**
         * Retrieves all the referers from the database.
         *
         * @return An array of Referer objects containing all the referers from the
         * database, or false if there are no referers.
         */
        function getAllReferers()
        {
        	$query = "SELECT * FROM ".$this->getPrefix()."referers;";

            $result = $this->_db->Execute( $query );
            if( !$result )
            	return false;

            $referers = Array();
            while( $row = $result->FetchRow()) {
            	array_push( $referers, $this->_fillRefererInformation( $row ));
            }

            return $referers;
        }

        /**
         * Returns a list with all the referers for a blog.
         *
         * @param blogId Blog identifier. If not specified, retrieves all
         * the referers.
         * @return An array with all the referers for a blog.
         * article.
         */
        function getBlogReferers( $blogId = 0 )
        {
        	$query = "SELECT * FROM ".$this->getPrefix()."referers WHERE blog_id = ".$blogId." ORDER BY last_date";

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;

            $referers = Array();
            while( $row = $result->FetchRow())
            	array_push( $referers, $this->_fillRefererInformation( $row ));

            return $referers;
        }

        /**
         * Private function.
         */
        function _fillRefererInformation( $row )
        {
        	$referer = new Referer( $row["url"], $row["article_id"], $row["blog_id"], $row["last_date"], $row["hits"], $row["id"] );

            return $referer;
        }
    }
?>
