<?php

    include_once( PLOG_CLASS_PATH."class/dao/model.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articles.class.php" );    
    include_once( PLOG_CLASS_PATH."class/dao/blogs.class.php" );    
    
    define( "SUMMARY_MAX_CACHE_SECS", 60 * 60 );
    
    /**
     * implements some general methods that return some statistical
     * data that is used in the summary.php script, and that is not
     * very useful for anybody else
     */
    class SummaryStats extends Model 
    {
    
        // whether we're using cache or not
        var $_useCache;
    
        function SummaryStats()
        {
            // initialize ADOdb        
            $this->Model();
            
            // whether we're really using cache or not
            $config =& Config::getConfig();
            $this->_useCache = $config->getValue( "summary_use_cache" );
        }

        /**
         * Returns the most commented articles so far
         *
         * @param maxPosts The maximum number of posts to return
         * @return An array of Article objects with the most commented articles
         */
        function getMostCommentedArticles( $maxPosts = 0, $date = 0)
        {
			$query = " SELECT COUNT(*) as total_comments, a.* FROM ".$this->getPrefix()."articles_comments AS c "
			       . " INNER JOIN ".$this->getPrefix()."articles AS a ON c.article_id=a.id WHERE a.status = 'PUBLISHED' ";

            if( $date > 0 )
                	$query .= " AND date_format(c.date,'%Y%m%d') > $date ";

            $query .= "GROUP BY c.article_id ORDER BY total_comments DESC ";

            if( $maxPosts > 0 )
            	$query .= " LIMIT 0,".$maxPosts;
            	
            if( $this->_useCache )
                $result = $this->_db->CacheExecute( SUMMARY_MAX_CACHE_SECS, $query );
            else
                $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;

            $posts = Array();
            $articles = new Articles();            
            while( $row = $result->FetchRow()) {
            	array_push( $posts, $articles->_fillArticleInformation($row));
            }

            return $posts;
        }

        /**
         * Returns an array with the most read articles
         *
         * @param maxPosts The maximum number of posts to return
         * @return an array of Article objects with information about the posts
         */
        function getMostReadArticles( $maxPosts = 0, $date = 0)
        {
			$query = " SELECT * FROM ".$this->getPrefix()."articles WHERE status = 'PUBLISHED' ";
				   
            if( $date > 0 )
                	$query .= " AND date_format(date,'%Y%m%d') > $date";

            $query .= " ORDER BY num_reads DESC ";

            if( $maxPosts > 0 )
            	$query .= " LIMIT 0,".$maxPosts;

            if( $this->_useCache ) 
                $result = $this->_db->CacheExecute( SUMMARY_MAX_CACHE_SECS, $query );
            else
                $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;

            $posts = Array();
            $articles = new Articles();
            while( $row = $result->FetchRow()) {
            	array_push( $posts, $articles->_fillArticleInformation($row));
            }

            return $posts;
        }
        
        /**
         * Retrieves the first article date timestamp
         *
         * @param blogId The identifier of the blog from which we'd like to fetch the settings
         * @return Returns an Timestamp with the first article date
         */
        function getBlogCreateDate( $blogId )
        {
			$query  = "SELECT date FROM ".$this->getPrefix()."articles WHERE blog_id = ".$blogId. " ORDER BY date ASC LIMIT 0,1" ;

            if( $this->_useCache ) 
                $result = $this->_db->CacheExecute( $query, SUMMARY_MAX_CACHE_SECS );
            else
                $result = $this->_db->Execute( $query );

            if (!$result)
                return false;

            $row = $result->FetchRow();

            if (!isset($row["date"]))
                return false;

            return $row["date"];
        }

        /**
         * Retrieves the last article date timestamp
         *
         * @param blogId The identifier of the blog from which we'd like to fetch the settings
         * @return Returns an Timestamp with the last article date
         */
        function getBlogUpdateDate( $blogId )
        {
			$query  = "SELECT date FROM ".$this->getPrefix()."articles WHERE blog_id = ".$blogId. " ORDER BY date DESC LIMIT 0,1" ;
            $result = $this->_db->Execute( $query );

            if (!$result)
                return false;

            $row = $result->FetchRow();

            if (!isset($row["date"]))
                return false;

            return $row["date"];
        }

        /**
         * Retrieves the total number of reads of all articles in that blog
         *
         * @param blogId The identifier of the blog from which we'd like to fetch the settings
         * @return Returns an Timestamp with the last article date
         */
        function getBlogViewedTotal( $blogId )
        {
			$query  = "SELECT SUM(num_reads) as total FROM ".$this->getPrefix()."articles WHERE blog_id = ".$blogId;
			
			if( $this->_useCache )
                $result = $this->_db->CacheExecute( SUMMARY_MAX_CACHE_SECS, $query );
            else
                $result = $this->_db->Execute( $query );

            if (!$result)
                return false;

            $row = $result->FetchRow();

            if (!isset($row["total"]))
                return false;

            return $row["total"];
        }

        /**
         * Retrieves the total number of posts in a blog
         *
         * @param blogId The identifier of the blog from which we'd like to fetch the settings
         * @return Returns an integer with the number of posts
         */
        function getBlogTotalPosts( $blogId )
        {
			$query  = "SELECT COUNT(*) as total FROM ".$this->getPrefix()."articles WHERE blog_id = $blogId AND status = 'published'";
            $result = $this->_db->Execute( $query );

            if (!$result)
                return false;

            $row = $result->FetchRow();

            if (!isset($row["total"]))
                return false;

            return intval($row["total"]);
        }
        
        /**
         *returns  list with the most recently created blogs
         *
         * @param maxBlogs The maximum number of blogs to return, or '0' to get all of them
         * @return An array of BlogInfo objects
         * @see BlogInfo
         */
         function getRecentBlogs($maxBlogs = 0)
         {
         	$query = "SELECT * FROM ".$this->getPrefix()."blogs ORDER BY id DESC";

            if( $maxBlogs > 0 )
            	$query .= " LIMIT 0,".$maxBlogs;

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;

            $blogs = Array();
            $blogdao = new Blogs();
            while( $row = $result->FetchRow()) {
            	$blog = $blogdao->_fillBlogInformation( $row );
                $blogs[$blog->getId()] = $blog;
            }

            return $blogs;
        }

        /**
         * returns an array with the most active blogs
         *
         * @param maxBlogs How many blogs to return
         * @param date
         * @return An array of BlogInfo objects
         * @see BlogInfo
         */
         function getMostActiveBlogs($maxBlogs = 0, $date = 0)
         {
			$query = " SELECT COUNT(*) as posts, b.id, b.blog, b.owner_id FROM ".$this->getPrefix()."articles AS a "
				   . " INNER JOIN ".$this->getPrefix()."blogs AS b ON a.blog_id=b.id ";
				   
            if( $date > 0 )
                $query .= " AND date_format(a.date,'%Y%m%d') > $date";	
				   
			$query .= " GROUP BY a.blog_id ORDER BY posts DESC ";
				   
            if( $maxBlogs > 0 )
            	$query .= " LIMIT 0,".$maxBlogs;
            	
            if( $this->_useCache ) 
                $result = $this->_db->CacheExecute( SUMMARY_MAX_CACHE_SECS, $query );
            else
                $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;

            $blogs = Array();
            $blogdao = new Blogs();
            while( $row = $result->FetchRow()) {
            	$blog = $blogdao->_fillBlogInformation( $row );
                $blogs[$blog->getId()] = $blog;
            }

            return $blogs;
        }
        
        /**
         * returns a list with the most recent articles, but only one per blog so that
         * one blog cannot clog the whole main page because they've posted 100 posts today
         *
         * @param maxPosts The maximum number of posts to return
         * @return An array of Article objects with the most recent articles.
         */
        function getRecentArticles( $maxPosts )
        {		
            $t      = new Timestamp();
			$date   = $t->getTimestamp();
        	$query  = " SELECT * FROM ".$this->getPrefix()."articles WHERE status = 'PUBLISHED'
        	            AND date <= NOW()
        	            ORDER BY date DESC LIMIT 0, $maxPosts";

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;

			$blogs = Array();
            $posts = Array();
			$i     = 0;
			$articles = new Articles();
            while( ($row = $result->FetchRow()) && ($i < $maxPosts) ) {
				if (!in_array($row["blog_id"], $blogs))
				{
					$blogs[] = $row["blog_id"];
					array_push( $posts, $articles->_fillArticleInformation($row) );
					$i++;
				}
            }

            return $posts;
        }                    
    }
?>