<?php

	include_once( PLOG_CLASS_PATH."class/dao/model.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/trackback.class.php" );

	/**
     * Implementation of a trackback feature.
     * The technical specifications of the trackback protocol can be found
     * here: http://www.movabletype.org/docs/mttrackback.html.
     */
    class Trackbacks extends Model {

    	var $_blogSettings;

    	/**
         * Initializes the connection to the database
         */
    	function Trackbacks()
        {
        	$this->Model();

            // we can keep a little cache of already-fetched BlogSettings
            // objects, so that we don't have to fetch them again when the only
            // thing that we need to know is the time difference once again
            $this->_blogSettings = Array();
        }

        /**
         * Adds a trackback to the database.
         *
         * @param artId The article to which we are adding the trackback.
         * @param title The title of the entry pinging us.
         * @param url Permalink for the entry.
         * @param blog The name of the blog where the entry is posted.
         * @param excerpt An excerpt of the entry.
         * @return Returns true if the entry was added successfully to the database.
         */
        function addTrackBack( $artId, $url, $title = "", $blog = "", $excerpt = "")
        {
        	if( $title == "" )
            	$title = $url;

        	$query = "INSERT INTO ".$this->getPrefix()."trackbacks (url,title,article_id,excerpt,blog_name) VALUES ('".$url."','".Db::qstr($title)."',".$artId.",'".Db::qstr($excerpt)."','".Db::qstr($blog)."');";

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;
            else
            	return true;
        }

        /**
         * Returns the trackback items for a given article.
         *
         * @param artId The article identifier.
         * @return An array of TrackbackItem objects with the information, or false otherwise.
         */
        function getArticleTrackBacks( $artId )
        {
        	$query = "SELECT * FROM ".$this->getPrefix()."trackbacks WHERE article_id = ".$artId.";";

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;

            $trackbacks = Array();
            while( $row = $result->FetchRow()) {
            	$trackback = $this->_fillTrackbackInformation( $row );
                array_push( $trackbacks, $trackback );
            }

            return $trackbacks;
        }

        /**
         * function factored out from the above
         *
         * @private
         * @param row The row with the information
         */
        function _fillTrackbackInformation( $row )
        {
        	// ---
            // there we go again doing dirty things to the poor trackbacks...
            // ---
            $prefix = $this->getPrefix();
            $date = $row["date"];
            $articleId = $row["article_id"];

            // let's try and keep some kind of cache for this...
            if( empty($this->_blogSettings[$articleId])) {
            	$query = "SELECT DISTINCT b.settings AS settings
                          FROM {$prefix}blogs b, {$prefix}articles a,
                          {$prefix}trackbacks t
                          WHERE t.article_id = a.id AND
                          a.blog_id = b.id
                          AND a.id = $articleId";
                $result = $this->_db->Execute( $query );
                $tmpRow = $result->FetchRow();
                $blogSettings = Blogs::getBlogSettingsFromField( $tmpRow["settings"] );
            }
            else {
            	$blogSettings = $this->_blogSettings[$articleId];
            }

            $timeDiff = $blogSettings->getValue( "time_offset" );
            // now that we've got the time difference, we can
            // calculate what would the "real" date...
            $date = Timestamp::getDateWithOffset( $date, $timeDiff );

        	return new TrackBack( $row["url"],
                                  $row["title"],
                                  $row["article_id"],
                                  $row["excerpt"],
                                  $row["blog_name"],
                                  $date,
                                  $row["id"] );
        }

        /**
         * Returns how many trackbacks have been received for the given article
         *
         * @param artId The article identifier.
         * @return The number of trackbacks received pointing to that article,
         */
        function getNumTrackbacksArticle( $artId )
        {
        	if( empty($this->totalTrackbacks)) {
        		$query1 = "SELECT article_id, COUNT(*) AS 'count' FROM ".$this->getPrefix()."trackbacks GROUP BY (article_id)";
            	$this->totalTrackbacks = $this->_db->GetAssoc( $query1 );
            }

            $totalTrackbacks = $this->totalTrackbacks[$artId];
            if( $totalTrackbacks == "" ) $totalTrackbacks = 0;

            return $totalTrackbacks;
        }

        /**
         * Removes all the trackbacks from a post
         *
         * @param artId The article identifier
         */
        function deletePostTrackbacks( $artId )
        {
        	$query = "DELETE FROM ".$this->getPrefix()."trackbacks WHERE article_id = $artId";

			$result = $this->_db->Execute( $query );

            return $result;
        }        
    }
?>
