<?php

	include_once( PLOG_CLASS_PATH."class/dao/model.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/userinfo.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/userpermissions.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/blogs.class.php" );

    /**
     * Model representing the users in our application. Provides the methods such as
     * authentication and querying for users.
     */
	class Users extends Model {

    	/**
         * Initializes the model
         */
		function Users()
        {
			$this->Model();

            $this->usercache = Array();

            $this->perms =  new UserPermissions();
        }

		/**
		 * Returns true if the user is in the database and the username
		 * and password match
         *
         * @param user Username of the user who we'd like to authenticate
         * @param pass Password of the user
         * @return true if user and password correct or false otherwise.
		 */
		function authenticateUser( $user, $pass )
		{
			$query = "SELECT * FROM ".$this->getPrefix()."users WHERE user = '".$user."' AND password = '".md5($pass)."'";

			$result = $this->_db->Execute( $query );

            if( $result == false )
				return false;

			if( $result->RecordCount() == 1 )
            	return true;
			else
				return false;
		}

		/**
		 * Returns all the information associated to the user given
         *
         * @param user Username of the user from who we'd like to get the information
         * @param pass Password of the user we'd like to get the information
         * @return Returns a UserInfo object with the requested information, or false otherwise.
		 */
		function getUserInfo( $user, $pass )
		{
			$query = "SELECT * FROM ".$this->getPrefix()."users WHERE user = '".$user."' AND password = '".md5($pass)."'";

			$userInfo = $this->_getUserInfoFromQuery( $query );

			return $userInfo;
		}

		/**
		 * Retrieves the user information but given only a username
         *
         * @param username The username of the user
         * @return Returns a UserInfo object with the requested information, or false otherwise.
		 */
		function getUserInfoFromUsername( $username )
		{
			$query = "SELECT * FROM ".$this->getPrefix()."users WHERE user = '".$username."';";

			$userInfo = $this->_getUserInfoFromQuery( $query );

			return $userInfo;
		}

		/**
		 * Retrieves the user infromation but given only a userid
         *
         * @param userId User ID of the user from whom we'd like to get the information
         * @return Returns a UserInfo object with the requested information, or false otherwise.
		 */
		function getUserInfoFromId( $userid )
		{
        	if( isset($this->usercache[$userid])) {
            	$userInfo = $this->usercache[$userid];
            }
            else {
				$query = "SELECT * FROM ".$this->getPrefix()."users WHERE id = ".$userid.";";

				$userInfo = $this->_getUserInfoFromQuery( $query );

                $this->usercache[$userid] = $userInfo;
            }

			return $userInfo;
		}

		/**
		 * More common code used by several functions
         *
         * Private function used to fill in all the fields of UserInfo objects given
         * a row of the database.
		 */
		function _getUserInfoFromQuery( $sql_query )
		{
			$result = $this->_db->Execute( $sql_query );
			//$result = $this->_db->CacheExecute( 60, $sql_query );
            if( !$result )
            	return false;

            if( $result->RowCount() == 0 )
            	return false;

			$info = $result->FetchRow( $result );

			$userInfo = $this->_fillUserInformation( $info );

			return $userInfo;
		}

		/**
		 * Given a result record from a Execute call, it will fill in the
		 * fields of the object, so that we don't have to repeat the same
		 * code too many times
		 */
		function _fillUserInformation( $query_result )
		{
			$userInfo = new UserInfo( $query_result["user"], $query_result["password"],
									  $query_result["email"],
									  $query_result["about"],
									  $query_result["id"]);
			//$userBlogs = $this->getUsersBlogs($userInfo->getId());
			//$userInfo->setBlogs($userBlogs);

            // set some permissions
            $userInfo->setSiteAdmin($this->perms->isSiteAdmin( $userInfo->getId()));

			return $userInfo;
		}

		/**
		 * Returns an array of BlogInfo objects with the information of all the blogs to which
		 * a user belongs
         *
         * @param userId Identifier of the user
         * @return An array of BlogInfo objects to whom the user belongs.
		 */
		function getUsersBlogs( $userid )
		{
			$usersBlogs = Array();
            $blogs = new Blogs();

        	// check if the user is the owner of any blog
            $owner = "SELECT * FROM ".$this->getPrefix()."blogs WHERE owner_id = ".$userid.";";
            $result = $this->_db->Execute( $owner );
            while( $row = $result->FetchRow($result)) {
            	$blogId = $row["id"];
                $blogInfo = $blogs->getBlogInfo( $blogId );
            	//array_push( $usersBlogs, $blogId );
                array_push( $usersBlogs, $blogInfo );
            }

            // and now check to which other blogs he or she belongs
			$otherBlogs = "SELECT * FROM ".$this->getPrefix()."users_permissions WHERE user_id = ".$userid.";";
			$result = $this->_db->Execute( $otherBlogs );
			// now we know to which he or she belongs, so we only have
			// to load the information about those blogs
			while( $row = $result->FetchRow($result)) {
				$blogId = $row["blog_id"];
                if( $blogId > 0 ) {
                	$blogInfo = $blogs->getBlogInfo( $blogId );
            		//array_push( $usersBlogs, $blogId );
                	array_push( $usersBlogs, $blogInfo );
                }
			}

			return $usersBlogs;
		}

		/**
		 * Returns an array with all the users available in the database
         *
         * @return An array containing all the users.
		 */
		function getAllUsers()
		{
			$query = "SELECT * FROM ".$this->getPrefix()."users ORDER BY id ASC";

			$result = $this->_db->Execute( $query );

			$users = Array();

			while ($info = $result->FetchRow( $result ))
				array_push( $users, $this->_fillUserInformation( $info ));

			return $users;
		}

		/**
		 * Updates the information related to a user
         *
         * @param userInfo An UserInfo object containing the <b>already udpated</b> information of the
         * user we would like to update.
         * @return Returns true if ok or false otherwise.
		 */
		function updateUser( $userInfo )
		{
			$query = "UPDATE ".$this->getPrefix()."users SET email = '".$userInfo->getEmail()."', about = '".Db::qstr($userInfo->getAboutMyself())."', password = '".$userInfo->getPassword()."' WHERE id = ".$userInfo->getId().";";

            // update the users table
			$result = $this->_db->Execute( $query );

            // and now update the permissions, if there has been any change
            $perms = new UserPermissions();
            $perms->updateSiteAdmin( $userInfo->getId(), $userInfo->isSiteAdmin());

			return $result;
		}

        /**
         * Adds a user to the database.
         *
         * @param userName Username for the new user.
         * @param password Password that the new user will have.
         * @param email Email address used by this user.
         * @param about Long, descriptive text about the user.
         * @return Returns the identifier assigned to the user, or false if there was any error.
         */
        function addUser( $userName, $password, $email, $about = "")
        {
        	$query = "INSERT INTO ".$this->getPrefix()."users(user,password,email,about) VALUES ('".Db::qstr($userName)."','".md5($password)."','".Db::qstr($email)."','".Db::qstr($about)."');";

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;

            // we return the identifier of the user we just created
            /*$query = "SELECT id FROM ".$this->getPrefix()."users WHERE user = '".$userName."' AND password = '".md5($password)."' AND email = '".$email."' AND about = '".$about."';";

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;

            $row = $result->FetchRow();
            return $row["id"];*/
            
            $userId = $this->_db->Insert_ID();
            
            return $userId;
        }

        /**
         * Returns an array with all the users that belong to the given
         * blog.
         *
         * @param blogId The blog identifier.
         * @param includeOwner Wether to include the owner of the blog or not.
         * @return An array with the information about the users who belong in
         * one way or another to that blog.
         */
        function getBlogUsers( $blogId, $includeOwner = true )
        {
            $users = Array();

            // get the information about the owner, if requested so
            if( $includeOwner ) {
                $query = "SELECT ".$this->getPrefix()."users.* FROM ".$this->getPrefix()."users, ".$this->getPrefix()."blogs WHERE ".$this->getPrefix()."users.id = ".$this->getPrefix()."blogs.owner_id AND ".$this->getPrefix()."blogs.id = ".$blogId.";";
                $result = $this->_db->Execute( $query );

                if( !$result )
                	return false;

                $row = $result->FetchRow();
                array_push( $users, $this->_fillUserInformation( $row ));
            }

            // now get the other users who have permission for that blog.
            $query2 = "SELECT ".$this->getPrefix()."users.* FROM ".$this->getPrefix()."users, ".$this->getPrefix()."users_permissions WHERE ".$this->getPrefix()."users.id = ".$this->getPrefix()."users_permissions.user_id AND ".$this->getPrefix()."users_permissions.blog_id = ".$blogId.";";
            $result2 = $this->_db->Execute( $query2 );
            if( !$result2 ) // if error, return what we have so far...
            	return $users;

            while( $row = $result2->FetchRow()) {
            	array_push( $users, $this->_fillUserInformation($row));
            }

            return $users;
        }

        /**
         * Removes users from the database
         *
         * @param userId The identifier of the user we are trying to remove
         */
        function deleteUser( $userId )
        {
        	// first, delete all of his/her permissions
            $perms = new UserPermissions();
            $perms->revokeUserPermissions( $userId );

        	$query = "DELETE FROM ".$this->getPrefix()."users WHERE id = $userId;";

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;

            if( $this->_db->Affected_Rows() == 0 )
            	return false;

            return true;
        }
    }
?>
