<?php

	include_once( PLOG_CLASS_PATH."class/object/object.class.php" );
	include_once( PLOG_CLASS_PATH."class/config/config.class.php" );
	include_once( PLOG_CLASS_PATH."class/data/kses.class.php" );

    /**
	 * Implements some text filters, for example removing javascript
	 * code from a string or removing any non-allowed character.
	 */
	class TextFilter extends Object 
	{

		var $htmlAllowedTags;
		var $removeJavaScript;
		var $_smileys;

        /**
         * Constructor.
         *
         * @param removeJavaScript By default, JavaScript code will also be removed
         * from texts
         */
		function TextFilter( $removeJavaScript = true )
		{
			$this->Object();

			$this->removeJavaScript = $removeJavaScript;

			$config = Config::getConfig();
			$this->htmlAllowedTags = $config->getValue( "html_allowed_tags_in_comments" );
			$this->_smileys = $config->getValue( "smileys" );
		}

		/**
		 * Removes the Javascript code from a string.
		 * Original function from http://www.zend.com/tips/tips.php?id=124&single=1
         *
         * @param text The text we want to filter
         * @return Returns the filtered text
		 */
        function filterJavaScript( $text )
        {
                // Strip all of the Javascript in script tags out...
                $text = preg_replace('/<SCRIPT.*?<\/SCRIPT>/ims',"",$text);

				/** copied from the original function **/
                /* The following matches any on* events, followed by any amount of space, a
                 *' or " some script and then the matching ' or " (the \\2 matches the
                 *single or double quote).  Note that this regex is
                 * in single quotes to alleviate the problem of double quoting special
                 * chars, otherwise the backreferenced 2 would be \\\\2
                 * -- which is just silly...
				 */
                $text = preg_replace('/on(Load|Click|DblClick|DragStart|KeyDown|KeyPress|KeyUp|MouseDown|MouseMove|MouseOut|MouseOver|SelectStart|Blur|Focus|Scroll|Select|Unload|Change|Submit)\s*=\s*(\'|").*?\\2/smi',"",$text);

                $text = preg_replace('/(\'|")Javascript:.*?\\1/smi','',$text);

                return $text;
        }

		/**
		 * Filters a string with html code removing all the non-allowed tags
		 * (see the 'html_allowed_tags_in_comments' from the configuration file
		 * config.properties)
		 * It also removes the Javascript code if any.
         *
         * @param string The text we would like to filter
         * @return Returns the filtered text.
		 */
        function filterHTML( $string )
        {
			$tmp = strip_tags( $string, $this->htmlAllowedTags );
			// y luego eliminamos el javascript
			$filteredString = $this->filterJavaScript( $tmp );

			return $filteredString;
        }

		/**
		 * Works like the one above but it simply removes *all* html code
		 * from a string
         *
         * @param string The string we would like to filter
         * @return The filtered text.
		 */
        function filterAllHTML( $string )
        {
			$tmp = strip_tags( $string );
			// y luego eliminamos el javascript
			$filteredString = $this->filterJavaScript( $tmp );

			return $filteredString;
        }

		/**
		 * Converts special characters to HTML entities
		 * Works as a wrapper to the htmlentities() function from the PHP API
         * @param string The string we would like to process
         * @return The same string but with the HTML/XML entities encoded to their representation
		 */
		function filterHTMLEntities( $string )
		{
			return htmlentities( $string );
		}

        /**
         * Alias for filterHTMLEntities
         */
		function filterXMLEntities( $string )
		{
			return $this->filterHTMLEntities($string);
		}

		/**
		 * Very dirty function that cuts the text after as many words
		 * as $maxWords, <b>respecting</b> HTML and XML tags. Tags that are not
         * cut are &lt;a&gt;, &lt;b&gt;, &lt;i&gt;
         *
         * @param text Text we would like to cut
         * @param maxWords This function counts based on words and <b>not</b> on bytes!!
         * @return A string of $maxWords words long, but respecting some html tags.
		 */
		function CutText( $text, $maxWords )
		{
			$parts = split( ' ', $text );

			//$not_split_tags = Array( "a", "b", "i" );
            $not_split_tags = Array();

			$inside = false;
			$i = 0;
			$total = 0;
			$total2 = 0;
			$result = "";

			while( ($i < count($parts)) && ($total < $maxWords)) {

				$part = $parts[$i];
				//print(htmlentities($part)."-");
				$last = substr( $part, -4 );

                //print("part: ".htmlentities($part)." - last: ".$last."<br/>");

				foreach( $not_split_tags as $tag ) {
					if($part == ("<".$tag)) {
						$inside=true;
						$total2=0;
						print("inside!!");
					}
				}

				foreach( $not_split_tags as $tag ) {
					$start = substr($part, 0,3 );
					if ($start == ("<".$tag.">")) {
						print("inside 2!!");
						$inside=true;
					}
				}

				foreach( $not_split_tags as $tag ) {
					if( $last == ("</".$tag.">" )) {
						$inside=false;
						print("outside! - part:".$part);
						$total += $total2;
						//$total--;
					}
				}

				if( !$inside)
					$total++;
				else
					$total2++;

				$result .= $part." ";

				$i++;
			}

			$result = rtrim($result);
            //print("result=".$result);
			return $result;
		}

        /**
         * Translates all the &gt;a href="..."&lt; tags of a text to include the
         * target="blank_" parameter so that they will open in a new window
         */
		function linksInNewWindow( $text )
		{
			$text = str_replace( "<a ", "<a target=\"blank_\" ", $text );

			return $text;
		}

		function translateSmileys( $text )
		{
			foreach( $this->_smileys as $symbol => $icon ) {
				$new_text = str_replace( $symbol, $icon, $text );
				$text = $new_text;
			}

			return $text;
		}

        /**
         * Texturize function borrowed from http://photomatt.net/tools/texturize
         *
         * Takes care of "beautifying" code typed by users.
         */
        function texturize($text)
        {
            $textarr = preg_split("/(<.*>)/U", $text, -1, PREG_SPLIT_DELIM_CAPTURE); // capture the tags as well as in between
            $stop = count($textarr); $next = true; // loop stuff
            for ($i = 0; $i < $stop; $i++) {
            	$curl = $textarr[$i];
                if (!strstr($_SERVER['HTTP_USER_AGENT'], 'Gecko')) {
                	$curl = str_replace('<q>', '&#8220;', $curl);
                    $curl = str_replace('</q>', '&#8221;', $curl);
                }
                if ('<' != $curl{0} && $next) { // If it's not a tag
                	$curl = str_replace('---', '&#8212;', $curl);
                    $curl = str_replace('--', '&#8211;', $curl);
                    $curl = str_replace("...", '&#8230;', $curl);
                    $curl = str_replace('``', '&#8220;', $curl);

                    $curl = preg_replace("/'s/", "&#8217;s", $curl);
                    $curl = preg_replace("/'(\d\d(?:&#8217;|')?s)/", "&#8217;$1", $curl);
                    $curl = preg_replace('/(\s|\A|")\'/', '$1&#8216;', $curl);
                    $curl = preg_replace("/(\d+)\"/", "$1&Prime;", $curl);
                    $curl = preg_replace("/(\d+)'/", "$1&prime;", $curl);
                    $curl = preg_replace("/(\S)'([^'\s])/", "$1&#8217;$2", $curl);
                    $curl = preg_replace('/"([\s.]|\Z)/', '&#8221;$1', $curl);
                    $curl = preg_replace('/(\s|\A)"/', '$1&#8220;', $curl);
                    $curl = preg_replace("/'([\s.]|\Z)/", '&#8217;$1', $curl);
                    $curl = preg_replace("/\(tm\)/i", '&#8482;', $curl);
                    $curl = preg_replace("/\(c\)/i", '&#169;', $curl);
                    $curl = preg_replace("/\(r\)/i", '&#174;', $curl);

                    $curl = str_replace("''", '&#8221;', $curl);
                    $curl = preg_replace('/&([^#])(?![a-z]{2,8};)/', '&#038;$1', $curl);

                    $curl = preg_replace('/(d+)x(\d+)/', "$1&#215;$2", $curl);
               } elseif (strstr($curl, '<code') || strstr($curl, '<pre') || strstr($curl, '<kbd' || strstr($curl, '<style') || strstr($curl, '<script'))) {
               		// strstr is fast
            		$next = false;
        	  } else {
            		$next = true;
        	  }
        	  $output .= $curl;
    	}

    	return $output;
	}

        /**
         * Texturize function borrowed from http://photomatt.net/tools/texturize
         *
         * Takes care of "beautifying" text, by adding <p> tags to blocks of texts without the users
         * needing to know any html.
         */
		function autoP($pee, $br=1)
        {
			$pee = preg_replace("/(\r\n|\n|\r)/", "\n", $pee); // cross-platform newline
            $pee = preg_replace("/\n\n+/", "\n\n", $pee); // take care of duplicates
        	$pee = preg_replace('/\n?(.+?)(\n\n|\z)/s', "<p>$1</p>\n", $pee); // make paragraphs, including one at the end
        	$pee = preg_replace('/<p>(<(?:table|[ou]l|pre|select|form|blockquote)>)/', "$1", $pee);
        	$pee = preg_replace('!(</?(?:table|[ou]l|pre|select|form|blockquote)>)</p>!', "$1", $pee);
        	if ($br) $pee = preg_replace('|(?<!</p>)\s*\n|', "<br />\n", $pee); // optionally make line breaks
        	$pee = preg_replace('!(</(?:table|[ou]l|pre|select|form|blockquote)>)<br />!', "$1", $pee);
        	$pee = str_replace('<p><p>', '<p>', $pee);
        	$pee = str_replace('</p></p>', '</p>', $pee);

        	return $pee;
		}
		
		/**
		 * improved version of the nl2br method from the PHP API
		 *
		 * @param text
		 * @return 
		 */
		function Nl2Br( $text, $wrap = 60 ) 
		{
            $string = nl2br($string);
            
            preg_match_all("/<pre[^>]*?>(.|\n)*?<\/pre>/", $string, $pre1);
            
            for ($x = 0; $x < count($pre1[0]); $x++) {
                $pre2[$x] = preg_replace("/<br[^>]*?>/", "", $pre1[0][$x]);
                $pre2[$x] = preg_replace("/([^\n]{".$wrap."})(?!<\/pre>)/", "$1\n", $pre2[$x]);
                $pre1[0][$x] = "/".preg_quote($pre1[0][$x], "/")."/";
            }
            
            return preg_replace($pre1[0], $pre2, $string);
		}
		
		/**
		 * improved version of the br2nl function from the PHP API
		 *
		 * @param text
		 * @return
		 */
		function Br2Nl( $text )
		{
		  return preg_replace( '!<br.*>!iU', "\n", $text );
		}
		
        /**
         * balanceTags
         *
         * Balances Tags of string using a modified stack.
         *
         * @param text      Text to be balanced
         * @return          Returns balanced text
         * @author          Leonard Lin (leonard@acm.org)
         * @version         v1.1
         * @date            November 4, 2001
         * @license         GPL v2.0
         * @notes           
         * @changelog       
         *            1.2  ***TODO*** Make better - change loop condition to $text
         *            1.1  Fixed handling of append/stack pop order of end text
         *                 Added Cleaning Hooks
         *            1.0  First Version
         */
        function balanceTags( $text )
        {
            
            $tagstack = array(); $stacksize = 0; $tagqueue = ''; $newtext = '';
        
            # WP bug fix for LOVE <3 (and other situations with '<' before a number)
            $text = preg_replace('#<([0-9]{1})#', '&lt;$1', $text);
        
            while (preg_match("/<(\/?\w*)\s*([^>]*)>/",$text,$regex)) {
                $newtext = $newtext . $tagqueue;
        
                $i = strpos($text,$regex[0]);
                $l = strlen($tagqueue) + strlen($regex[0]);
        
                // clear the shifter
                $tagqueue = '';
                // Pop or Push
                if ($regex[1][0] == "/") { // End Tag
                    $tag = strtolower(substr($regex[1],1));
                    // if too many closing tags
                    if($stacksize <= 0) { 
                        $tag = '';
                        //or close to be safe $tag = '/' . $tag;
                    }
                    // if stacktop value = tag close value then pop
                    else if ($tagstack[$stacksize - 1] == $tag) { // found closing tag
                        $tag = '</' . $tag . '>'; // Close Tag
                        // Pop
                        array_pop ($tagstack);
                        $stacksize--;
                    } else { // closing tag not at top, search for it
                        for ($j=$stacksize-1;$j>=0;$j--) {
                            if ($tagstack[$j] == $tag) {
                            // add tag to tagqueue
                                for ($k=$stacksize-1;$k>=$j;$k--){
                                    $tagqueue .= '</' . array_pop ($tagstack) . '>';
                                    $stacksize--;
                                }
                                break;
                            }
                        }
                        $tag = '';
                    }
                } else { // Begin Tag
                    $tag = strtolower($regex[1]);
        
                    // Tag Cleaning
        
                    // Push if not img or br or hr
                    if($tag != 'br' && $tag != 'img' && $tag != 'hr') {
                        $stacksize = array_push ($tagstack, $tag);
                    }
        
                    // Attributes
                    // $attributes = $regex[2];
                    $attributes = $regex[2];
                    if($attributes) {
                        $attributes = ' '.$attributes;
                    }
                    $tag = '<'.$tag.$attributes.'>';
                }
                $newtext .= substr($text,0,$i) . $tag;
                $text = substr($text,$i+$l);
            }  
        
            // Clear Tag Queue
            $newtext = $newtext . $tagqueue;
        
            // Add Remaining text
            $newtext .= $text;
        
            // Empty Stack
            while($x = array_pop($tagstack)) {
                $newtext = $newtext . '</' . $x . '>'; // Add remaining tags to close      
            }
        
            return $newtext;
        }		
		
		/**
		 * xhtml-izes a string
		 *
		 * @param string
		 * @return the xhtml-ized string
		 */
		function xhtmlize( $string )
		{
		      // use kses in the "xhtml converter" mode
		      $config =& Config::getConfig();
		      if( $config->getValue( "xhtml_converter_enabled" )) {
		          $kses = new kses( true, $config->getValue( "xhtml_converter_aggresive_mode_enabled"));
		          $result = $kses->Parse( $string );
		          
		          // if balanceTags wasn't broken, we could use it...
		          //$result = Textfilter::balanceTags( $result );		          
		      }
		      else
		          $result = $string;
		      
		      return $result;
		}
    }
?>