<?php

	include_once( PLOG_CLASS_PATH."class/object/object.class.php" );

    /**
     * This is an implementation of the 'Strategy' pattern as it can be seen
     * http://www.phppatterns.com/index.php/article/articleview/13/1/1/. Here we use
     * this pattern to validate data received from forms. Its is useful since for example
     * we check in many places if a 'postId' is valid or not. We can put the
     * checkings inside the class and simply reuse this class wherever we want. If we ever
     *`change the format of the postId parameter, we only have to change the code of the
     * class that validates it and it will be automatically used everywhere.
     */
    class Validator extends Object {

    	var $_valid;
        var $_message;

    	/**
         * The constructor does nothing.
         */
    	function Validator()
        {
        	$this->Object();

            $this->_valid = false;
            $this->_message = "";
        }

        /**
         * Validates the data. Does nothing here and it must be reimplemented by
         * every child class.
         */
        function validate()
        {
        	// nothing
        }

        /**
         * Returns true if the condition imposed on the parameters is true. This method
         * must be called after calling the validate() method.
         *
         * @return True if the parameter is correct or false otherwhise.
         */
        function isValid()
        {
        	return $this->_valid;
        }

        /**
         * Returns the error message set by the validate() method.
         *
         * @return The error message set by the validate() method.
         */
        function getMessage()
        {
        	return $this->_message;
        }

        /**
         * Sets the error message that will be returned in case there is an error.
         *
         * @param message The error message.
         */
        function setMessage( $message )
        {
        	$this->_message = $message;
        }
    }
?>