<?php

	include_once( PLOG_CLASS_PATH."class/object/object.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/mimetype.class.php" );
    include_once( PLOG_CLASS_PATH."class/gallery/data/galleryresourceimagemetadatareader.class.php" );
    include_once( PLOG_CLASS_PATH."class/gallery/data/galleryresourcesoundmetadatareader.class.php" );
    include_once( PLOG_CLASS_PATH."class/gallery/data/galleryresourcevideometadatareader.class.php" );
    include_once( PLOG_CLASS_PATH."class/gallery/data/galleryresourcezipmetadatareader.class.php" );
    include_once( PLOG_CLASS_PATH."class/gallery/galleryconstants.php" );
    include_once( PLOG_CLASS_PATH."class/gallery/dao/galleryresourcestorage.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/timestamp.class.php" );
	include_once( PLOG_CLASS_PATH."class/gallery/resizers/galleryresizer.class.php" );
	include_once( PLOG_CLASS_PATH."class/data/stringutils.class.php" );

    /**
     * Encapsulates a resource from our database.
     */
    class GalleryResource extends Object
    {

    	var $_id;
        var $_ownerId;
        var $_albumId;
        var $_date;
        var $_description;
        var $_flags;
        var $_resourceType;
        var $_filePath;
        var $_fileName;
        var $_metadata;
        var $_album;
		var $_thumbnailFormat;

        function GalleryResource( $ownerId, $albumId, $description, $flags, $resourceType,
                                  $filePath, $fileName, $metadata, $date, $thumbnailFormat, $id = -1 )
        {
        	$this->_ownerId = $ownerId;
            $this->_albumId = $albumId;
            $this->_description = $description;
            $this->_flags = $flags;
            $this->_resourceType = $resourceType;
            $this->_filePath = $filePath;
            $this->_fileName = $fileName;
            $this->_metadata = $metadata;
            $this->_date = $date;
			$this->_thumbnailFormat = $thumbnailFormat;
            $this->_id = $id;
        }

        function getId()
        {
        	return $this->_id;
        }

        function getOwnerId()
        {
        	return $this->_ownerId;
        }

        function getAlbumId()
        {
        	return $this->_albumId;
        }

        function getDate()
        {
        	return $this->_date;
        }

        function getTimestamp()
        {
        	return new Timestamp($this->_date);
        }

        function getMetadata()
        {
        	return $this->_metadata;
        }

        function getFlags()
        {
        	return $this->_flags;
        }

        function getFilePath()
        {
        	return $this->_filePath;
        }

        function getFileName()
        {
        	return $this->_fileName;
        }
        
        function getDescription()
        {
        	return $this->_description;
        }

        function getMimeType()
        {
        	$mimeType = new MimeType();
        	return $mimeType->getType( $this->_fileName );
        }
		
		function getThumbnailMimeType()
		{
			if( $this->getThumbnailFormat() == "same" )
				return $this->getMimeType();
			else {
				$mimeType = new MimeType();
				return $mimeType->getType( $this->getThumbnailFormat());
			}
		}

        function getResourceType()
        {
        	return $this->_resourceType;
        }

        function getData()
        {
        	$fileParts = explode( ".", $this->getFileName());
            $fileExt = $fileParts[count($fileParts)-1];

			$resourceStorageFolder = GalleryResourceStorage::getResourcesStorageFolder();
            $fileName = $resourceStorageFolder.$this->getOwnerId()."/".$this->getOwnerId()."-".$this->getId().".".$fileExt;

        	$file = new File( $fileName );
            $file->open( "rb" );

            $size = $file->getSize();

            return $file->read( $size );
        }

        function setAlbumId( $albumId )
        {
        	$this->_albumId = $albumId;
        }

        function setAlbum( $album )
        {
        	$this->_album = $album;
        }
		
		function getAlbum()
		{
			return $this->_album;
		}

        function setDescription( $description )
        {
        	$this->_description = $description;
        }
		
		function getThumbnailFormat()
		{
			return $this->_thumbnailFormat;
		}
		
		function setThumbnailFormat( $format )
		{
			$this->_thumbnailFormat = $format;
		}

        function getMetadataReader()
        {
        	switch( $this->getResourceType()) {
            	case GALLERY_RESOURCE_IMAGE:
                	$reader = new GalleryResourceImageMetadataReader( $this );
                    break;
                case GALLERY_RESOURCE_SOUND:
                	$reader = new GalleryResourceSoundMetadataReader( $this );
                    break;
                case GALLERY_RESOURCE_VIDEO:
                	$reader = new GalleryResourceVideoMetadataReader( $this );
                    break;
                case GALLERY_RESOURCE_ZIP:
                	$reader = new GalleryResourceZipMetadataReader( $this );
                    break;
                default:
                	$reader = new GalleryResourceBaseMetadataReader( $this );
                    break;
            }

            return $reader;
        }

        /**
         * Returns true if this resource has a preview or false if not.
         *
         * @return True if preview available or false otherwise.
         */
        function hasPreview()
        {
        	return( $this->_flags & GALLERY_RESOURCE_PREVIEW_AVAILABLE );
        }

        /**
         * Returns the binary data corresponding to the preview.
         *
         * @return A string of binary data.
         */
        function getPreview()
        {
        	if( !$this->hasPreview())
            	return false;

			if( $this->getThumbnailFormat() == THUMBNAIL_OUTPUT_FORMAT_SAME_AS_IMAGE ) {
				$fileParts = explode( ".", $this->getFileName());
				$fileExt = $fileParts[count($fileParts)-1];
				
				$fileName = $this->getOwnerId()."-".$this->getId().".".strtolower($fileExt);
				$previewFile = GalleryResourceStorage::getPreviewsFolder( $this->getOwnerId()).$fileName;
			}
			else {
				$previewType = $this->getThumbnailFormat();
				$fileName = $this->getOwnerId()."-".$this->getId().".".strtolower($previewType);
				$previewFile = GalleryResourceStorage::getPreviewsFolder( $this->getOwnerId()).$fileName;				
			}

        	$file = new File( $previewFile );
            $file->open( "rb" );

            $size = $file->getSize();

            return $file->read( $size );
        }

        function isImage()
        {
        	return( $this->_resourceType == GALLERY_RESOURCE_IMAGE );
        }

        function isSound()
        {
        	return( $this->_resourceType == GALLERY_RESOURCE_SOUND );
        }

        function isVideo()
        {
        	return( $this->_resourceType == GALLERY_RESOURCE_VIDEO );
        }

        function isZip()
        {
        	return( $this->_resourceType == GALLERY_RESOURCE_ZIP );
        }
    }
?>
