<?php

	include_once( PLOG_CLASS_PATH."class/dao/model.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/timestamp.class.php" );
    include_once( PLOG_CLASS_PATH."class/gallery/dao/galleryresource.class.php" );
    include_once( PLOG_CLASS_PATH."class/file/fileupload.class.php" );
    include_once( PLOG_CLASS_PATH."class/gallery/dao/galleryresourcestorage.class.php" );
    include_once( PLOG_CLASS_PATH."class/gallery/getid3/getid3.php" );
    include_once( PLOG_CLASS_PATH."class/gallery/galleryconstants.php" );
    include_once( PLOG_CLASS_PATH."class/gallery/resizers/galleryresizer.class.php" );
    include_once( PLOG_CLASS_PATH."class/gallery/dao/galleryalbums.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/uploadvalidator.class.php" );
	include_once( PLOG_CLASS_PATH."class/config/config.class.php" );

    /**
     * database access for GalleryResource objects
     */
    class GalleryResources extends Model
    {
    	var $albums;

    	/**
         * maps extensions to resource types
         */
       var $_extensionToType = Array(
       	"jpg" => GALLERY_RESOURCE_IMAGE,
        "jpeg" => GALLERY_RESOURCE_IMAGE,
        "png" => GALLERY_RESOURCE_IMAGE,
        "gif" => GALLERY_RESOURCE_IMAGE,
        "bmp" => GALLERY_RESOURCE_IMAGE,
        "mp3" => GALLERY_RESOURCE_SOUND,
		"mp2" => GALLERY_RESOURCE_SOUND,
        "wav" => GALLERY_RESOURCE_SOUND,
        "au" => GALLERY_RESOURCE_SOUND,
		"aac" => GALLERY_RESOURCE_SOUND,
		"mp4" => GALLERY_RESOURCE_SOUND,
        "wma" => GALLERY_RESOURCE_SOUND,
		"ogg" => GALLERY_RESOURCE_SOUND,
		"mod" => GALLERY_RESOURCE_SOUND,
		"mid" => GALLERY_RESOURCE_SOUND,
		"midi" => GALLERY_RESOURCE_SOUND,
        "avi" => GALLERY_RESOURCE_VIDEO,
		"mpg" => GALLERY_RESOURCE_VIDEO,
		"mpeg" => GALLERY_RESOURCE_VIDEO,
        "wmv" => GALLERY_RESOURCE_VIDEO,
		"asf" => GALLERY_RESOURCE_VIDEO,
        "mov" => GALLERY_RESOURCE_VIDEO,
        "divx" => GALLERY_RESOURCE_VIDEO,
		"rm" => GALLERY_RESOURCE_VIDEO,
		"swf" => GALLERY_RESOURCE_VIDEO,
		"qt" => GALLERY_RESOURCE_VIDEO,
        "pdf" => GALLERY_RESOURCE_DOCUMENT,
        "zip" => GALLERY_RESOURCE_ZIP
       );

    	function GalleryResources()
        {
        	$this->Model();

            $this->albums = new GalleryAlbums();
			
			//$this->_db->debug=true;
        }

        /**
         * Fetches GalleryResource objects from the database
         *
         * @param resourceId The id of the resource we'd like to fetch
         * @return Returns a GalleryResource object representing the resource
         */
        function getResource( $resourceId, $ownerId = -1 )
        {
        	$query = "SELECT * FROM ".$this->getPrefix()."gallery_resources
                      WHERE id = $resourceId";
            if( $ownerId > 0 )
            	$query .= " AND owner_id = $ownerId";

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;

            if( $result->RecordCount() == 0 )
            	return false;

            $row = $result->FetchRow();
            return $this->_fillResourceInformation( $row );
        }

        /**
         * Returns all the resources that belong to a blog
         *
         * @param blogId The blog to which the resources belong
         * @param albumId Filters by album
         * @return Returns an array of GalleryResource objects with all
         * the resources that match the given conditions, or empty
         * if none could be found.
         */
        function getUserResources( $ownerId, $albumId = -1 )
        {
        	$query = "SELECT * FROM ".$this->getPrefix()."gallery_resources
                      WHERE owner_id = $ownerId ";
            if( $albumId >= 0 )
            	$query .= " AND album_id = $albumId";
			$query .= " ORDER BY album_id, date ASC";

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return Array();

            $resources = Array();
            while( $row = $result->FetchRow()) {
            	array_push( $resources, $this->_fillResourceInformation( $row ));
            }

            return $resources;
        }

        /**
         * adds a resource to the database
         */
        function addResource( $ownerId, $albumId, $description, $upload )
        {
        	// first of all, validate the file using the
            // upload validator class. It can return
            // UPLOAD_VALIDATOR_ERROR_UPLOAD_TOO_BIG (-1)
            // or
            // UPLOAD_VALIDATOR_ERROR_FORBIDDEN_EXTENSION (-2)
            // in case the file is not valid.
            $uploadValidator = new UploadValidator();
            $error = $uploadValidator->validate( $upload );
            if( $error < 0 ) {
            	return $error;
            }
			_debug( "New resource validated ok!" );
			
            // get the metadata
			$getId3 = new GetID3();
			$metadata = $getId3->analyze( $upload->getTmpName());
			_debug( "Resource metadata fetched!" );

            // get the resource type
            $fileParts = explode( ".", $upload->getFileName());
            $fileExt = strtolower($fileParts[count($fileParts)-1]);
            if( array_key_exists( $fileExt, $this->_extensionToType ))
            	$resourceType = $this->_extensionToType[ $fileExt ];
            else
            	$resourceType = GALLERY_RESOURCE_UNKNOWN;
			_debug( "Resource type = $resourceType" );

            // set the flags
            $flags = 0;
            if( $resourceType == GALLERY_RESOURCE_IMAGE )
            	$flags = $flags|GALLERY_RESOURCE_PREVIEW_AVAILABLE;

            // add the record to the database
            $fileName = $upload->getFileName();

            $serMetadata = Db::qstr(serialize( $metadata ));
			
			$config =& Config::getConfig();
			$thumbnailFormat = $config->getValue( "thumbnail_format" );

            $query = "INSERT INTO ".$this->getPrefix()."gallery_resources(
                      owner_id, album_id, description, flags, resource_type,
                      file_path, file_name, metadata, thumbnail_format) VALUES (
                      $ownerId, $albumId, '".Db::qstr($description)."', $flags, $resourceType,
                      '$filePath', '$fileName', '$serMetadata', '$thumbnailFormat')";
            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;
			
            // get the id that was given to the record
            $resourceId = $this->_db->Insert_ID();
			_debug( "Resource added to db successfully! Resource ID = $resourceId" );			

            // check if we have two resources with the same filename now
            // check if there already exists a file with the same name
            //
            // if that's the case, then we should rename the one we just
            // added with some random prefix, to make it different from the
            // other one...
            if( $this->isDuplicatedFilename( $fileName )) {
            	$query = "UPDATE ".$this->getPrefix()."gallery_resources
                          SET file_name = '$resourceId-$fileName'
                          WHERE id = $resourceId";

                $this->_db->Execute( $query );

            }

            // and finally move the file to the right place in disk
        	// move the file to disk
            $storage = new GalleryResourceStorage();
            $resFile = $storage->store( $resourceId, $ownerId, $upload );
			_debug("res file = $resFile");
			
			// if the file cannot be read, we will also remove the record from the
			// database so that we don't screw up
			$fileReadable = File::isReadable( $resFile );

            if( !$resFile || $resFile < 0 || !$fileReadable ) {
            	// if something went wrong, we should not keep the record in the db
                $query = "DELETE FROM ".$this->getPrefix()."gallery_resources
                          WHERE id = $resourceId";

                $this->_db->Execute( $query );

                return $resFile;
            }

            // and finally, we can generate the thumbnail
            if( $resourceType == GALLERY_RESOURCE_IMAGE ) {
                // get some configuration settings regarding the size of the
                // thumbnails, and also the default format for thumbnails
                $config =& Config::getConfig();
                $previewHeight = $config->getValue( "thumbnail_height" );
                $previewWidth  = $config->getValue( "thumbnail_width" );
                $previewKeepAspectRatio = $config->getValue( "thumbnails_keep_aspect_ratio" );

                // build the filename
                $fileName = $ownerId."-".$resourceId.".".$fileExt;

                // and start the resizing process
                $resizer = new GalleryResizer( $resFile );
                GalleryResourceStorage::checkPreviewsStorageFolder( $ownerId );
                $outFile = GalleryResourceStorage::getPreviewsFolder( $ownerId ).$fileName;

                // and finally, we can generate the preview!
                $result = $resizer->generate( $outFile, $previewHeight, $previewWidth, $previewKeepAspectRatio );
            }

            return true;
        }

        /**
         * retrieves a resource, given its filename and its owner
         *
         * @param ownerId
         * @param fileName
         * @return Returns a GalleryResource object containing the given resource
         * or false if the resource doesn't exist.
         */
        function getResourceFile( $ownerId, $fileName )
        {
        	$query = "SELECT * FROM ".$this->getPrefix()."gallery_resources
                      WHERE owner_id = $ownerId AND
                      file_name = '$fileName'";

            $result = $this->_db->Execute( $query );
            if( !$result )
            	return false;

            if( $result->RecordCount() == 0 )
            	return false;

            $row = $result->FetchRow();

            return $this->_fillResourceInformation( $row );
        }

        /**
         * updates a resource in the database.
         *
         * @param resource A GalleryResource object with the information of the
         * resource we'd like to update.
         */
        function updateResource( $resource ) {
        	$query = "UPDATE ".$this->getPrefix()."gallery_resources
                      SET album_id = ".$resource->getAlbumId().",
                      description = '".Db::qstr($resource->getDescription())."',
                      flags = ".$resource->getFlags().",
                      resource_type = ".$resource->getResourceType().",
                      file_path = '".$resource->getFilePath()."',
                      file_name = '".$resource->getFileName()."',
                      metadata = '".Db::qstr(serialize($resource->getMetadata()))."',
					  thumbnail_format ='".$resource->getThumbnailFormat()."',
                      date = '".$resource->getDate()."'
                      WHERE id = ".$resource->getId();

            $result = $this->_db->Execute( $query );

            if( !$result )
            	return false;
            else
            	return true;
        }

        /**
         * removes a resource from the database and disk
         *
         * @param resourceId The identifier of the resource we'd like to remove
         * @param ownerId Identifier of the owner of the resource. Optional.
         * @return Returns true if resource deleted ok or false otherwise.
         */
        function deleteResource( $resourceId, $ownerId = -1 )
        {
        	// first, get informaiton abotu the resource
            $resource = $this->getResource( $resourceId, $ownerId );

            // now, remove it from the db
            $query = "DELETE FROM ".$this->getPrefix()."gallery_resources
                      WHERE id = $resourceId";
            if( $ownerId > 0 )
            	$query .= " AND owner_id = $ownerId";

            $result = $this->_db->Execute( $query );

            // if there was an error, we quit here
            if( !$result )
            	return false;

            // otherwise, proceed and remove the file from disk
            $storage = new GalleryResourceStorage();
            return $storage->remove( $resource );
        }

        function isDuplicatedFilename( $fileName )
        {
        	$query = "SELECT COUNT(*) AS total FROM ".$this->getPrefix()."gallery_resources
                      WHERE file_name = '$fileName'";

            $result = $this->_db->Execute( $query );

            $row = $result->FetchRow();
            if( $row["total"] == 0 || $row["total"] == 1 )
            	return false;
            else
            	return true;
        }


        /**
         * @private
         */
        function _fillResourceInformation( $row )
        {
        	$res = new GalleryResource( $row["owner_id"],
                                        $row["album_id"],
                                        $row["description"],
                                        $row["flags"],
                                        $row["resource_type"],
                                        $row["file_path"],
                                        $row["file_name"],
                                        unserialize($row["metadata"]),
                                        $row["date"],
										$row["thumbnail_format"],
                                        $row["id"] );

             // we should also try to get some information about
             // to which album this resource belongs
             $album = $this->albums->getAlbum( $row["album_id"], $row["owner_id"], false );
             $res->setAlbum( $album );

             return $res;
        }
    }
?>
