<?php

	include_once( PLOG_CLASS_PATH."class/object/object.class.php" );
    include_once( PLOG_CLASS_PATH."class/file/fileupload.class.php" );
    include_once( PLOG_CLASS_PATH."class/file/file.class.php" );
    include_once( PLOG_CLASS_PATH."class/file/fileuploads.class.php" );
    include_once( PLOG_CLASS_PATH."class/config/config.class.php" );
    include_once( PLOG_CLASS_PATH."class/gallery/galleryconstants.php" );

    /**
     * Takes care of dealing with resource files on disk
     */
    class GalleryResourceStorage extends Object
    {

    	function GalleryResourceStorage()
        {
        	$this->Object();
        }
		
		/**
		 * Returns the path to the folder where resources are stored. This method can 
		 * be used as static.
		 *
		 * @static
		 * @return Returns a string containing the base path to the resource storage folder.
		 */
		function getResourcesStorageFolder()
		{
			$config =& Config::getConfig();
			$resourcesStorageFolder = $config->getValue( "resources_folder" );
			
			// just in case...
			if( $resourcesStorageFolder == "" )
				$resourcesStorageFolder = DEFAULT_RESOURCES_STORAGE_FOLDER;
				
			// append a forward slash to the folder if we forgot about it...
			if( $resourcesStorageFolder[strlen($resourcesStorageFolder)-1] != '/')
				$resourcesStorageFolder .= "/";
				
			
			return $resourcesStorageFolder;
		}
		
        /**
         * @private
         */
        function _checkBaseStorageFolder()
        {
			$baseFolder = GalleryResourceStorage::getResourcesStorageFolder();
			if( $baseFolder[strlen($baseFolder)-1] == "/") {
        	   $baseFolder = substr($baseFolder,0,strlen($baseFolder)-1);
        	   _debug("base folder = $baseFolder");
            }			
						
			_debug( "Checking if base gallery folder exists = $baseFolder" );
        	if( !File::isDir( $baseFolder )) {
            	// folder does not exist, so we should try to create it
                if( !File::createDir( $baseFolder, 0755 )) {
					_debug( "There was an error creating the folder!" );
                	throw( new Exception( "Could not create storage folder for resources: ".$baseFolder));
					return false;
                    //die();
                }
				
				_debug( "Folder created ok!" );
            }

            if( !File::isReadable( $baseFolder )) {
				_debug( "Folder exists but it is not readable!" );
            	throw( new Exception( $baseFolder." storage folder exists but it is not readable!" ));
				return false;
                //die();
            }
			
			_debug( "Base folder exists and it is readable." );

            return true;
        }
		
		/**
		 * a nicer function that the one above. And it is also meant to be used
		 * by external classes
		 *
		 * @static
		 * @return Returns true if the base storage folder has been created and
		 * it is readable.
		 */
        function checkBaseStorageFolder()
        {
			return GalleryResourceStorage::_checkBaseStorageFolder();
        }		
				
        /**
         * @private
         */
        function getUserFolder( $ownerId )
        {
        	return GalleryResourceStorage::getResourcesStorageFolder().$ownerId."/";
        }

        /**
         * @public
         */
        function getPreviewsFolder( $ownerId )
        {
        	return GalleryResourceStorage::getResourcesStorageFolder().$ownerId."/previews/";
        }

        /**
         * @private
         */
        function _checkUserStorageFolder( $ownerId )
        {
			$baseFolder = GalleryResourceStorage::getResourcesStorageFolder();
			if( $baseFolder[strlen($baseFolder)-1] == "/") {
        	   $baseFolder = substr($baseFolder,0,strlen($baseFolder)-1);
        	   _debug("base folder = $baseFolder");
            }			

        	$userFolder = GalleryResourceStorage::getUserFolder( $ownerId );
        	if( $userFolder[strlen($userFolder)-1] == "/") {
        	   $userFolder = substr($userFolder,0,strlen($userFolder)-1);
            }
			
			_debug("user folder = $userFolder");			
        	if( !File::isDir( $userFolder )) {
            	// folder does not exist, so we should try to create it
                if( !File::createDir( $userFolder, 0755 )) {
					_debug("could not create folder!");
                	throw( new Exception( "Could not create user storage folder for resources: ".$userFolder ));
					return false;
                    //die();
                }
            }

            if( !File::isReadable( $userFolder )) {
				_debug( "User storage folder exists but it is not readable!" );
            	throw( new Exception( $userFolder." user storage folder exists but it is not readable!" ));
				return false;
                //die();
            }
			
            return true;
        }
		
		function checkUserStorageFolder( $ownerId )
		{
			return GalleryResourceStorage::_checkUserStorageFolder( $ownerId );
		}

        /**
         * @public
         */
        function checkPreviewsStorageFolder( $ownerId )
        {
            $previewsFolder = GalleryResourceStorage::getPreviewsFolder( $ownerId );
        	if( $previewsFolder[strlen($previewsFolder)-1] == "/") {
        	   $previewsFolder = substr($previewsFolder,0,strlen($previewsFolder)-1);
				_debug("previews folder = $previewsFolder");			   
            }

			_debug( "Checking if previews folder exists = $previewsFolder" );
        	if( !File::isDir( $previewsFolder )) {
            	// folder does not exist, so we should try to create it
                if( !File::createDir( $previewsFolder, 0755 )) {
					_debug("Error creating $previewsFolder!");
                	throw( new Exception( "Could not create user storage folder for previews: ".$previewsFolder ));
                    //die();
					return false;
                }
				
				_debug( "Previews folder created ok!" );
            }

            if( !File::isReadable( $previewsFolder )) {
            	throw( new Exception( $previewsFolder." user previews storage folder exists but it is not readable!" ));
                //die();
				return false;
            }

			_debug( "Previews folder checked ok!" );
            return true;
        }

        /**
         * stores a new resource in disk
         *
         * @param ownerId The id of the owner of this file
         * @param albumId The album id to which the
         * @param upload a FileUpload object with information about the
         * uploaded file
         */
        function store( $resourceId, $ownerId, $upload )
        {
        	// check that the folders exist
            if( !$this->_checkBaseStorageFolder())
				return false;
            if( !$this->_checkUserStorageFolder( $ownerId ))
				return false;

            // new name for the file
            $fileParts = explode( ".", $upload->getFileName());
            $fileExt = $fileParts[count($fileParts)-1];
            $fileName = "$ownerId-$resourceId.$fileExt";
            $filePath = $this->getUserFolder( $ownerId );
			
			_debug( "fileExt = $fileExt - fileName = $fileName - filePath = $filePath" );

            // move the file to the temporaray folder first
            $config =& Config::getConfig();
            $tmpFolder = $config->getValue( "temp_folder" );
            $files = HttpVars::getFiles();
            $uploads = new FileUploads( $files );
            $result = $uploads->process( $tmpFolder );

            if( $result < 0 ) {
            	return $result;
            }

            // rename it while it's there
			$destFile = $filePath.$fileName;			
            _debug("moving ".$tmpFolder."/".$upload->getFileName()." to $destFile");
            $res = File::rename( $tmpFolder."/".$upload->getFileName(), $destFile );
            if( !$res ) {
				_debug( "There was an error moving the file!" );
            	return false;
			}			
			// check that the permissions are correct
			_debug( "chmodding file to 0777" );
			File::chMod( $destFile, 0777 );

            return $destFile;
        }

        /**
         * removes a file from disk
         *
         * @param resource A GalleryResource object, representing the resource
         * we'd like to delete.
         * @return Returns ok if file was successfully deleted ok or false otherwise.
         */
        function remove( $resource )
        {
        	if( $resource ) {
        		$fileParts = explode( ".", $resource->getFileName());
                $fileExt = $fileParts[count($fileParts)-1];
                $fileName = $resource->getOwnerId()."-".$resource->getId().".".$fileExt;
                $filePath = $this->getUserFolder( $resource->getOwnerId());
				$fullName = $filePath.$fileName;

                _debug( "removing resource ".$fullName );
				if( File::isReadable( $fullName)) {
					$result = File::delete( $fullName );
				}
            }
            else
            	$result = false;

            return $result;
        }
		
		/**
		 * returns the path of a GalleryResource object within the storage area
		 *
		 * @param $resource A GalleryResource object
		 * @return A string containing the path to the file relative to the storage area.
		 */
		function getResourcePath( $resource )
		{
			$fileParts = explode( ".", $resource->getFileName());
			$fileExt = $fileParts[count($fileParts)-1];		
			$filePath = $this->getUserFolder( $resource->getOwnerId());
			$fileName = $resource->getOwnerId()."-".$resource->getId().".".$fileExt;
			
			return $filePath.$fileName;
		}
    }
?>
