<?php

    include_once( PLOG_CLASS_PATH."class/gallery/resizers/galleryabstractresizer.class.php" );
	include_once( PLOG_CLASS_PATH."class/gallery/resizers/gddetector.class.php" );
	
	/**
     * Generates functions using the built-in GD library functionality.
     *
     * Based off Shiege Iseng's (shiegege@yahoo.com) Resize Class found somewhere in
     * the net, but heavily modified specially concerning error situations.
     *
     */
	class GalleryGDResizer extends GalleryAbstractResizer {

    	var $img;

    	function GalleryGDResizer( $image, $outputMethod )
        {
        	$this->GalleryAbstractResizer( $image, $outputMethod );
        }

        function generate( $outFile, $width, $height )
        {
        	//
        	// generate the thubmanil but check for errors every time
            //
			
			// also, check if GD is available because otherwise we would get
			// all sorts of nasty errors...
			if( !GdDetector::detectGd())
				return false;
			
            if( !$this->thumbnail( $this->_image ))
            	return false;

			if( ($this->img["lebar"] < $width) || ($this->img["tinggi"] < $height) ) {
				$this->img["lebar_thumb"] = $this->img["lebar"];
				$this->img["tinggi_thumb"] = $this->img["tinggi"];
			}
			else {
				$this->size_height( $height );
				$this->size_width( $width );
			}

            if( !$this->save( $outFile ))
            	return false;

            return $outFile;
        }
        
        /**
         * returns true if gd2 is available or false otherwise.
         * Based on a comment found in http://fi2.php.net/imagecreatetruecolor by aaron at aaron-wright dot com
         * (credit is due where it is due :)
         */
        function isGD2Available()
        {
            // maybe the blog has been configured to use the gd1 routines no matter
            // if gd2 is detected or not... in that case, we don't go ahead and simply
            // say that gd2 is not available
            $config =& Config::getConfig();
            if( $config->getValue( "thumbnail_generator_force_use_gd1" )) {
                _debug("Forcing GD1");
                return false;
            }
            
            // if not, we still check in case the user made a mistake...
            $testGD = get_extension_funcs("gd"); // Grab function list
            if ( !$testGD ) { 
                _debug("There is no GD installed!");
                return false;
            }
            if (in_array ("imagegd2",$testGD)) {
                _debug("Using GD2");
                return true;
            }
            else { 
                _debug("Using GD1");
                return false; 
            }
        }

        function thumbnail($imgfile)
        {
        	//detect image format
            $this->img["format"]=ereg_replace(".*\.(.*)$","\\1",$imgfile);
            $this->img["format"]=strtoupper($this->img["format"]);
            if ($this->img["format"]=="JPG" || $this->img["format"]=="JPEG") {
                $this->img["format"]="JPEG";
                $this->img["src"] = @ImageCreateFromJPEG ($imgfile);
            }
            elseif ($this->img["format"]=="PNG") {
                $this->img["format"]="PNG";
                $this->img["src"] = @ImageCreateFromPNG ($imgfile);
            }
            elseif ($this->img["format"]=="GIF") {
                $this->img["format"]="GIF";
				if( function_exists("imagecreatefromgif")) {
                $this->img["src"] = @ImageCreateFromGIF ($imgfile);
				}
				else {
					return false;
				}

            }
            else {
            	// not a recognized format
                throw( new Exception( "Trying to generate a thumbnail of an unsupported format!"));
                //die();
            }

            // check for errors
            if( !$this->img["src"] )
            	return false;

            // if no errors, continue
            @$this->img["lebar"] = imagesx($this->img["src"]);
            @$this->img["tinggi"] = imagesy($this->img["src"]);
            //default quality jpeg
            $this->img["quality"]=85;

            return true;
        }

        function size_height($size=100)
        {
        	//height
			if( $this->img["lebar"] > $size ) {
				$this->img["tinggi_thumb"]=$size;
				@$this->img["lebar_thumb"] = ($this->img["tinggi_thumb"]/$this->img["tinggi"])*$this->img["lebar"];
			}
			else {
				$this->img["tinggi_thumb"]=$size;
				$this->img["lebar_thumb"]=$this->img["lebar"]; 
			}

            return true;
        }

        function size_width($size=100)
        {
        	//width
			if( $this->img["tinggi"] > $size ) {
				$this->img["lebar_thumb"]=$size;
				@$this->img["tinggi_thumb"] = ($this->img["lebar_thumb"]/$this->img["lebar"])*$this->img["tinggi"];
			}
			else {
				$this->img["lebar_thumb"] = $size;
				$this->img["tinggi_thumb"] = $this->img["tinggi"];
			}

            return true;
        }

        function size_auto($size=100)
        {
        	//size
            if ($this->img["lebar"]>=$this->img["tinggi"]) {
            	$this->img["lebar_thumb"]=$size;
                @$this->img["tinggi_thumb"] = ($this->img["lebar_thumb"]/$this->img["lebar"])*$this->img["tinggi"];
            }
            else {
            	$this->img["tinggi_thumb"]=$size;
                @$this->img["lebar_thumb"] = ($this->img["tinggi_thumb"]/$this->img["tinggi"])*$this->img["lebar"];
            }

            return true;
        }

        function jpeg_quality($quality=75)
        {
        	//jpeg quality
            $this->img["quality"]=$quality;

            return true;
        }
        
        function save( $save = "" )
        {
			$fileParts = explode( ".", $save );
			$fileNoExt = implode( ".", $fileParts );
			$fileExt = $fileParts[count($fileParts)-1];
		
            //if( function_exists("imagecreatetruecolor")) {
            if( $this->isGD2Available()) {
                _debug("using GD2");
            	$this->img["des"] = @ImageCreateTrueColor($this->img["lebar_thumb"],$this->img["tinggi_thumb"]);
            }
            else {
                _debug("Using GD1");
            	$this->img["des"] = @ImageCreate($this->img["lebar_thumb"],$this->img["tinggi_thumb"]);
            }

            // check for errors and stop if any, or continue otherwise
            if( $this->img["des"] == "" )
            	return false;
            	
            imagecopyresized(&$this->img["des"], $this->img["src"], 0, 0, 0, 0, $this->img["lebar_thumb"], $this->img["tinggi_thumb"], $this->img["lebar"], $this->img["tinggi"] );

			// format for thumbnails is the same as the image
			//if( $this->_outputMethod == THUMBNAIL_OUTPUT_FORMAT_SAME_AS_IMAGE || $this->_outputMethod == "" ) {
				if ($fileExt=="jpg" || $fileExt=="jpeg") {
				    _debug("saving as jpeg");
					$result = imageJPEG($this->img["des"],"$save",$this->img["quality"]);
				}
				elseif ($fileExt=="png") {
				    _debug("saving as png");
					$result = imagePNG($this->img["des"],"$save");
					
				}
				elseif ($fileExt=="gif") {
				    _debug("Saving as GIF");
					if( function_exists("imagegif")) {
						$result = imageGIF($this->img["des"],"$save");
					}
					else {
					   _debug("saving as none??");
						$result = false;
					}
				}

            return $result;
        }
	}
?>
