<?php

	include_once( PLOG_CLASS_PATH."class/object/object.class.php" );
    include_once( PLOG_CLASS_PATH."class/net/httpvars.class.php" );


    /**
     * Generates HTTP GET requests in a transparent way:
     *
     * <i>
     * $g = new RequestGenerator()<br>
     * $g->addParameter( "op", "ViewArticle" );<br>
     * $g->addParameter( "articleId", 1 );<br>
     * $request = $g->getRequest();<br>
     * </i><br>
     * Doing so we can easily change the format of the urls in the future if necessary.
     */
	class BaseRequestGenerator extends Object {

    	var $_params;
        var $_SERVER;
        var $_blogInfo;
        var $_baseUrl;
		var $_xhtmlEnabled;

        /**
         * Constructor.
         *
         * @param mode The mode we are going to use.
         * @param xhtml Wether or not requests have to be XHTML compliant. That is, all
         * '&' will be turned into '&amp;'
         */
    	function BaseRequestGenerator( $blogInfo = null )
        {
        	$this->Object();

            $this->_params = Array();

            $this->_SERVER = HttpVars::getServer();

            $this->_blogInfo = $blogInfo;

            $config =& Config::getConfig();
            $this->_baseUrl = $config->getValue( "base_url" );
			
			// enable the xhtml mode by default, but it can be turned off
			// via the setXHTML() method
			$this->_xhtmlEnabled = true;
        }

        function getBaseUrl( $abs = true )
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }

        function getIndexUrl( $abs = true )
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }

        function getRssUrl( $abs = true )
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }

        function getTrackbackUrl( $abs = true )
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }

        function getUrl( $res, $abs = true )
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }

        /**
         * Adds a parameter to the request
         *
         * @param paramName Name of the parameter
         * @param paramValue Value given to the parameter
         */
        function addParameter( $paramName, $paramValue )
        {
        	$this->_params[$paramName] = $paramValue;
        }

        function reset()
        {
        	$this->_params = Array();
        }

        /**
         * Returns a string representing the request
         *
         * @return A String object representing the request
         */
        function getRequest()
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }

    	/**
         * Returns the permalink for a post
         *
         * @param post The Article object
         * @return The link for the permalink
         */
		function postPermalink( $post )
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }

        /**
         * Returns the comment link for a post
         *
         * @param post The Article object
         * @return The correct string
         */
        function postCommentLink( $post )
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }

        /**
         * Returns the link for the post
         *
         * @param post The Article object
         * @return The link for the post
         */
        function postLink( $post )
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }

        /**
         * Returns the link for the post. This is kind of a dirty trick... :( This is only meant to be
         * used in the template that generates the rss feed for a blog.
         *
         * @param post The Article object
         * @return The link for the post
         */
        function postRssLink( $post )
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }

        /**
         * Returns the url for the link tracker feature
         *
         * @return The url that shows the results of the link tracker.
         */
        function linkTrackerLink()
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }

        /**
         * Returns the link of a category.
         *
         * @param post The post from which we'll fetch the category and then generate the right link.
         * @return The url pointing to the page with only the posts belonging to that category.
         * @see Article
         * @see categoryLink
         */
        function postCategoryLink( $post )
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }

        /**
         * Returns the link but given a category. Does the same as postCategoryLink but this time we don't need
         * a post but an ArticleCategory object.
         *
         * @see postCategoryLink
         * @see ArticleCategory
         * @param An ArticleCategory object containing the information regarding the category.
         * @return A string with the correct url pointing to the page that will show only the posts that belong
         * to the given category.
         */
        function categoryLink( $category )
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }

        /**
         * Returns a link to only the articles of the user
         *
         * @param user The user whose posts we would like to see
         * @param category Optionally, we can specify an ArticleCategory object
         * @return A string containing the right url to only the posts of the user.
         * @see UserInfo
         * @see ArticleCategory
         */
        function postUserLink( $user, $category = null )
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }

        /**
         * Returns the url of the host where the blog is running
         *
         * @return Returns the url where the blog is running.
         */
        function blogLink( $blogInfo = null )
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }

        /**
         * Returns the url where the rss feed is running
         *
         * @param blogInfo A BlogInfo object containing information about the blog.
         * @return The url pointing to the rss feed of the journal.
         * @see BlogInfo
         */
        function rssLink( $blogInfo )
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }

        /**
         * Returns the url to reply to a given comment.
         *
         * @param post An Article object with information about the post
         * @param commen A UserComment object containing information about the post we'd like to reply to.
         * @return The right url to reply to this comment.
         * @see UserComment
         * @see Article
         */
        function replyCommentLink( $post, $comment )
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }

        /**
         * Manually adds the "show more" link in a post.
         *
         * @param post The post we are going to cut.
         * @param maxWords Amount of words we'd like to allow.
         * @param linkText Text we are going to show.
         * @return The modified link.
         */
        function addShowMoreLink( $post, $maxWords, $linkText )
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }

        /**
         * Returns the trackback link for a given post
         *
         * @param post The post with the necessary information.
         * @return A string representing the rdf trackback link.
         */
        function postTrackbackLink( $post )
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }

        function getArchiveLink( $date )
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }


        /**
         * Returns the link to the page showing the trackback statistics for a given post.
         *
         * @param post The post with the information.
         * @return Returns a string with the valid link.
         */
        function postTrackbackStatsLink( $post )
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }

        /**
         * Returns the link to an album. If the parameter is null or zero, then a link to the top level
		 * album will be returned.
         *
         * @param album The GalleryAlbum object.
         */
        function albumLink( $album = null )
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }

        /**
         * Returns the link to a resource
         *
         * @param album Generates the correct link to fetch a resource
         */
        function resourceLink( $resource )
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }

        /**
         * Returns the link to a resource preview
         *
         * @param album Generates the correct link to fetch a resource preview
         */
        function resourcePreviewLink( $resource )
        {
        	throw( new Exception( "This function must be implemented by child classes." ));
        }
		
		/**
		 * whether we should generate valid xhtml requests or not
		 * (used for example when sending out messages, as some email clients will
		 * go to the wrong url when using xhtml message
		 *
		 * @param enable Whether to enable xhtml or not (enabled by default)
		 * @return Always true
		 */
		function setXHTML( $enabled = true )
		{
			$this->_xhtmlEnabled = $enabled;
		}
		
		/**
		 * whether xhtml mode is enabled or not
		 *
 		 * @return True if enabled or false otherwise
		 */
		function isXHTML()
		{
			return $this->_xhtmlEnabled;
		}
		
		/**
		 * generates the correct path to a file in the template folder, without having to worry
		 * whether the template was installed in /templates/ or in /templates/blog_X/
		 *
		 * @param file
		 * @return A string
		 */
		function getTemplateFile( $file )
		{
        	throw( new Exception( "This function must be implemented by child classes." ));		
        }		
    }
?>
