<?php

	include_once( PLOG_CLASS_PATH."class/net/baserequestgenerator.class.php" );
	include_once( PLOG_CLASS_PATH."class/dao/users.class.php" );

    /**
     * Generates 'pretty' URLs
     */
	class PrettyRequestGenerator extends BaseRequestGenerator {

    	/*
         * Constructor.
         */
    	function PrettyRequestGenerator( $blogInfo )
        {
        	$this->BaseRequestGenerator( $blogInfo );

        }

        function getBaseUrl( $abs = true )
        {
            $url = $this->_baseUrl;

            return $url;
        }

        function getIndexUrl( $abs = true )
        {
            $url = $this->getBaseUrl( $abs )."/index.php";

            return $url;
        }

        function getRssUrl( $abs = true )
        {
            $url = $this->getBaseUrl( $abs )."/rss";

            return $url;
        }

        function getTrackbackUrl( $abs = true )
        {
            $url = $this->getBaseUrl( $abs )."/trackback.php";

            return $url;
        }

        function getResourceServerUrl( $abs = true )
        {
            $url = $this->getBaseUrl( $abs )."/resserver.php";

            return $url;
        }

        function getUrl( $res, $abs = true )
        {
        	$baseUrl = $this->getBaseUrl( $abs );
            $url = $baseUrl.$res;

            return $url;
        }

    	/**
         * Returns the permalink for a post
         *
         * @param post The Article object
         * @return The link for the permalink
         */
		function postPermalink( $post )
        {
            $permaLink = $this->getBaseUrl()."/post/".$this->_blogInfo->getId()."/".$post->getId();

            return $permaLink;
        }

        function getArchiveLink( $date )
        {
        	$archiveLink = $this->getBaseUrl()."/archives";
            if( $this->_blogInfo != null )
            	$archiveLink .= "/".$this->_blogInfo->getId();
            $archiveLink .= "/$date";

            return $archiveLink;
        }

        /**
         * Returns the comment link for a post
         *
         * @param post The Article object
         * @return The correct string
         */
        function postCommentLink( $post )
        {
            $commentLink = $this->getBaseUrl()."/comment/".$this->_blogInfo->getId()."/".$post->getId();

            return $commentLink;
        }

        /**
         * Returns the link for the post
         *
         * @param post The Article object
         * @return The link for the post
         */
        function postLink( $post )
        {

            $postLink = $this->postPermalink( $post );

            return $postLink;
        }

        /**
         * Returns the link for the post. This is kind of a dirty trick... :( This is only meant to be
         * used in the template that generates the rss feed for a blog.
         *
         * @param post The Article object
         * @return The link for the post
         */
        function postRssLink( $post )
        {
        	$postLink = $this->postPermalink($post);

            return $postLink;
        }

        /**
         * Returns the url for the link tracker feature
         *
         * @return The url that shows the results of the link tracker.
         */
        function linkTrackerLink()
        {

            $linkTracker = $this->getBaseUrl()."/linktracker/".$this->_blogInfo->getId();

            return $linkTracker;
        }

        /**
         * Returns the link of a category.
         *
         * @param post The post from which we'll fetch the category and then generate the right link.
         * @return The url pointing to the page with only the posts belonging to that category.
         * @see Article
         * @see categoryLink
         */
        function postCategoryLink( $post )
        {
            $categoryLink = $this->getBaseUrl()."/category/".$this->_blogInfo->getId()."/".$post->getCategoryId();

            return $categoryLink;
        }

        /**
         * Returns the link but given a category. Does the same as postCategoryLink but this time we don't need
         * a post but an ArticleCategory object.
         *
         * @see postCategoryLink
         * @see ArticleCategory
         * @param An ArticleCategory object containing the information regarding the category.
         * @return A string with the correct url pointing to the page that will show only the posts that belong
         * to the given category.
         */
        function categoryLink( $category )
        {
            $categoryLink = $this->getBaseUrl()."/category/".$this->_blogInfo->getId()."/".$category->getId();

            return $categoryLink;
        }

        /**
         * Returns a link to only the articles of the user
         *
         * @param user The user whose posts we would like to see
         * @param category Optionally, we can specify an ArticleCategory object
         * @return A string containing the right url to only the posts of the user.
         * @see UserInfo
         * @see ArticleCategory
         */
        function postUserLink( $user, $category = null )
        {
            $userLink = $this->getBaseUrl()."/user/".$user->getId();

            return $userLink;
        }

        /**
         * Returns the url of the host where the blog is running
         *
         * @return Returns the url where the blog is running.
         */
        function blogLink( $blogInfo = null )
        {
			$config =& Config::getConfig();
			$disableApacheErrorHandler = $config->getValue( "disable_apache_error_handler" );
			if( $disableApacheErrorHandler )
				$path = "/blog/";
			else
				$path = "/";
				
		
            if( $blogInfo == null ) {
                if( $config->getValue( "pretty_urls_force_use_username" )) {
                    $users = new Users();
                    $userInfo = $users->getUserInfoFromId( $this->_blogInfo->getOwner());
                    $link = $this->getBaseUrl().$path.$userInfo->getUsername();
                }
                else {
            	   $link = $this->getBaseUrl().$path.$this->_blogInfo->getId();
                }
            }
            else {
            	$link = $this->getBaseUrl().$path.$blogInfo->getId();
            }

            return $link;
        }

        /**
         * Returns the url where the rss feed is running
         *
         * @param blogInfo A BlogInfo object containing information about the blog.
         * @param profile The RSS profile we'd like to use. It defaults to none.
         * @return The url pointing to the rss feed of the journal.
         * @see BlogInfo
         */
        function rssLink( $profile = "", $blogInfo = null )
        {
        	$rssBase = $this->getBaseUrl()."/rss/";
            if( $profile != "" )
            	$rssBase .= $profile."/";

        	if( $blogInfo == null )
            	$rssLink = $rssBase.$this->_blogInfo->getId();
            else
            	$rssLink = $rssBase.$blogInfo->getId();

            return $rssLink;
        }

        /**
         * Returns the url for the rss feed of a category
         *
         * @param category The ArticleCategory object with information about the category
         * whose RSS feed we'd like to generate
         * @aram profile The profile we'd like to generate: RSS 0.90, RSS 1.0, RSS 2.0
         * or XML.
         * @param blogInfo A BlogInfo object containing information about the blog.
         * @return The url pointing to the rss feed of the journal.
         * @see BlogInfo
         */
        function categoryRssLink( $category, $profile = "", $blogInfo = null )
        {
        	$rssBase = $this->getBaseUrl()."/rss/";

            if( $profile != "" )
            	$rssBase .= $profile."/";

        	if( $blogInfo == null )
            	$rssLink = $rssBase.$this->_blogInfo->getId();
            else
            	$rssLink = $rssBase.$blogInfo->getId();

            $rssLink .= "/".$category->getId();

            return $rssLink;
        }

        function postTrackbackLink( $post )
        {
        	$rdfHeader = '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
                                   xmlns:dc="http://purl.org/dc/elements/1.1/"
                                   xmlns:trackback="http://madskills.com/public/xml/rss/module/trackback/">';

            $trackbackLink = $this->getTrackbackUrl()."?id=".$post->getId();
            $rdfBody = "<rdf:Description
                             rdf:about=\"".$this->postLink($post)."\"
                             dc:identifier=\"".$this->postLink($post)."\"
                             dc:title=\"".$post->getTopic()."\"
                             trackback:ping=\"".$trackbackLink."\"/>";
            $rdfFooter = "</rdf:RDF>";

            return $rdfHeader.$rdfBody.$rdfFooter;
        }

        /**
         * Returns the url to reply to a given comment.
         *
         * @param post An Article object with information about the post
         * @param commen A UserComment object containing information about the post we'd like to reply to.
         * @return The right url to reply to this comment.
         * @see UserComment
         * @see Article
         */
        function replyCommentLink( $post, $comment )
        {
            $replyCommentLink = $this->getIndexUrl()."?op=Comment&amp;articleId=".$post->getId()."&amp;parentId=".$comment->getId()."&amp;blogId=".$this->_blogInfo->getId();

            return $replyCommentLink;
        }

        /**
         * Returns the link to the page showing the trackback statistics for a given post.
         *
         * @param post The post with the information.
         * @return Returns a string with the valid link.
         */
        function postTrackbackStatsLink( $post )
        {
            $tbStatsLink = $this->getBaseUrl()."/trackbacks/".$this->_blogInfo->getId()."/".$post->getId();

            return $tbStatsLink;
        }

        /**
         * Returns the link to an album
         *
         * @param album The GalleryAlbum object.
         */
        function albumLink( $album = null )
        {
			if( $album == null )
				$albumLink = $this->getBaseUrl()."/album/".$this->_blogInfo->getId()."/0";
			else 
				$albumLink = $this->getBaseUrl()."/album/".$this->_blogInfo->getId()."/".$album->getId();

            return $albumLink;
        }

        function parentAlbumLink( $album )
        {
        	$albumLink = $this->getBaseUrl()."/album/".$this->_blogInfo->getId()."/".$album->getParentId();

            return $albumLink;
        }

        function templatePage( $template )
        {
        	$templatePage = $this->getBaseUrl()."/static/".$this->_blogInfo->getId()."/".$template;

            return $templatePage;
        }

        /**
         * Returns the link to a resource
         *
         * @param album Generates the correct link to fetch a resource
         */
        function resourceLink( $resource )
        {
        	return $resourceLink = $this->getBaseUrl()."/resource/".$this->_blogInfo->getId()."/".$resource->getId();
        }

        /**
         * Returns the link to a resource preview
         *
         * @param album Generates the correct link to fetch a resource preview
         */
        function resourcePreviewLink( $resource )
        {
        	return $resourceLink = $this->getBaseUrl()."/resserver.php?blogId=".$this->_blogInfo->getId()."&amp;resource=".rawurlencode($resource->getFileName())."&amp;mode=preview";
        }

        /**
         * Returns the link to a resource
         *
         * @param resource Generates the correct link to fetch a resource
         */
        function resourceDownloadLink( $resource )
        {
        	return $resourceLink = $this->getBaseUrl()."/get/".$this->_blogInfo->getId()."/".$resource->getFileName();
        }


        /**
         * Returns a string representing the request
         *
         * @return A String object representing the request
         */
        function getRequest()
        {
        	throw( new Exception( "PrettyRequestGenerator::getRequest: function not implemented" ));
            die();
        }
        
		/**
		 * generates the correct path to a file in the template folder, without having to worry
		 * whether the template was installed in /templates/ or in /templates/blog_X/
		 *
		 * @param file
		 * @return A string
		 */
		function getTemplateFile( $file )
		{
		    // get the current template set
		    $blogSettings = $this->_blogInfo->getSettings();
		    $template = $blogSettings->getValue( "template" );
		    
		    // define this couple of things
        	$baseUrl = $this->getBaseUrl( $abs );
            $url = "$baseUrl/templates/";
		    		    
		    // is it a blog template?
		    $blogTemplates = $blogSettings->getValue( "blog_templates" );
		    
		    if( !is_array($blogTemplates ))
		        $url .= "$template/$file";
		    else {
		        if( in_array( $template, $blogTemplates ))
		          $url .= "blog_".$this->_blogInfo->getId()."/$template/$file";
		        else
		          $url .= "$template/$file";
		    }
		    
            return $url;		    
		}        
    }
?>
