<?php

	include_once( PLOG_CLASS_PATH."class/net/baserequestgenerator.class.php" );

    /**
     * Generates HTTP GET requests in a transparent way:
     *
     * <i>
     * $g = new RequestGenerator()<br>
     * $g->addParameter( "op", "ViewArticle" );<br>
     * $g->addParameter( "articleId", 1 );<br>
     * $request = $g->getRequest();<br>
     * </i><br>
     * Doing so we can easily change the format of the urls in the future if necessary.
     */
	class RawRequestGenerator extends BaseRequestGenerator {

        /**
         * Constructor.
         */
    	function RawRequestGenerator( $blogInfo )
        {
        	$this->BaseRequestGenerator( $blogInfo );
        }

        function getBaseUrl( $abs = true )
        {
        	$scriptName = $this->_SERVER["PHP_SELF"];
            $url    = preg_replace( "/(.*)\/(.+)\.php/", "\\1", $scriptName );

            if( $abs )
            	$url = "http://".$this->_SERVER["SERVER_NAME"].$url;

            //return $url;
	    return $this->_baseUrl;
        }

        function getIndexUrl( $abs = true )
        {
            $url = $this->getBaseUrl( $abs )."/index.php";

            return $url;
        }

        function getRssUrl( $abs = true )
        {
            $url = $this->getBaseUrl( $abs )."/rss.php";

            return $url;
        }

        function getTrackbackUrl( $abs = true )
        {
            $url = $this->getBaseUrl( $abs )."/trackback.php";

            return $url;
        }

        function getResourceServerUrl( $abs = true )
        {
            $url = $this->getBaseUrl( $abs )."/resserver.php";

            return $url;
        }

        function getUrl( $res, $abs = true )
        {
        	$baseUrl = $this->getBaseUrl( $abs );
            $url = $baseUrl.$res;

            return $url;
        }

        /**
         * Adds a parameter to the request
         *
         * @param paramName Name of the parameter
         * @param paramValue Value given to the parameter
         */
        function addParameter( $paramName, $paramValue )
        {
        	$this->_params[$paramName] = $paramValue;
        }

    	/**
         * Returns the permalink for a post
         *
         * @param post The Article object
         * @return The link for the permalink
         */
		function postPermalink( $post )
        {
            $this->addParameter( "op", "ViewArticle" );
            $this->addParameter( "articleId", $post->getId());
            if( $this->_blogInfo != null )
            	$this->addParameter( "blogId", $this->_blogInfo->getId());

            $permaLink = $this->getIndexUrl().$this->getRequest();

            return $permaLink;
        }

        /**
         * Returns the comment link for a post
         *
         * @param post The Article object
         * @return The correct string
         */
        function postCommentLink( $post )
        {
            $this->addParameter( "op", "Comment" );
            $this->addParameter( "articleId", $post->getId());
            if( $this->_blogInfo != null )
            	$this->addParameter( "blogId", $this->_blogInfo->getId());

            $commentLink = $this->getIndexUrl().$this->getRequest();

            return $commentLink;
        }

        /**
         * Returns the link for the post
         *
         * @param post The Article object
         * @return The link for the post
         */
        function postLink( $post )
        {
            $this->addParameter( "op", "ViewArticle" );
            $this->addParameter( "articleId", $post->getId());
            if( $this->_blogInfo != null )
            	$this->addParameter( "blogId", $this->_blogInfo->getId());

            //$postLink = $_SERVER["PHP_SELF"].$rg->getRequest();
            $postLink = $this->getIndexUrl().$this->getRequest();

            return $postLink;
        }

        /**
         * Returns the link for the post. This is kind of a dirty trick... :( This is only meant to be
         * used in the template that generates the rss feed for a blog.
         *
         * @param post The Article object
         * @return The link for the post
         */
        function postRssLink( $post )
        {
        	$postLink = $this->postPermalink($post);

            return $postLink;
        }

        /**
         * Returns the url for the link tracker feature
         *
         * @return The url that shows the results of the link tracker.
         */
        function linkTrackerLink()
        {
            $this->addParameter( "op", "LinkTracker" );
            if( $this->_blogInfo != null )
            	$this->addParameter( "blogId", $this->_blogInfo->getId());

            //$linkTracker = $_SERVER["PHP_SELF"].$rg->getRequest();
            $linkTracker = $this->getIndexUrl().$this->getRequest();

            return $linkTracker;
        }

        /**
         * Returns the link of a category.
         *
         * @param post The post from which we'll fetch the category and then generate the right link.
         * @return The url pointing to the page with only the posts belonging to that category.
         * @see Article
         * @see categoryLink
         */
        function postCategoryLink( $post )
        {
             $this->addParameter( "op", "Default" );
             $this->addParameter( "postCategoryId", $post->getCategoryId());
             if( $this->_blogInfo != null )
               $this->addParameter( "blogId", $this->_blogInfo->getId());
 
             //$categoryLink = $_SERVER["PHP_SELF"].$rg->getRequest();
             $categoryLink = $this->getIndexUrl().$this->getRequest();
 
             return $categoryLink;
        }

        /**
         * Returns the link but given a category. Does the same as postCategoryLink but this time we don't need
         * a post but an ArticleCategory object.
         *
         * @see postCategoryLink
         * @see ArticleCategory
         * @param An ArticleCategory object containing the information regarding the category.
         * @return A string with the correct url pointing to the page that will show only the posts that belong
         * to the given category.
         */
        function categoryLink( $category )
        {
            $this->addParameter( "op", "Default" );
            $this->addParameter( "postCategoryId", $category->getId());
            if( $this->_blogInfo != null )
            	$this->addParameter( "blogId", $this->_blogInfo->getId());

            //$categoryLink = $_SERVER["PHP_SELF"].$rg->getRequest();
            $categoryLink = $this->getIndexUrl().$this->getRequest();

            return $categoryLink;
        }

        /**
         * Returns a link to only the articles of the user
         *
         * @param user The user whose posts we would like to see
         * @param category Optionally, we can specify an ArticleCategory object
         * @return A string containing the right url to only the posts of the user.
         * @see UserInfo
         * @see ArticleCategory
         */
        function postUserLink( $user, $category = null )
        {
            $this->addParameter( "op", "Default" );
            $this->addParameter( "userId", $user->getId());
            if( $category != null )
            	$this->addParameter( "postCategoryId", $category->getId());
            if( $this->_blogInfo != null )
            	$this->addParameter( "blogId", $this->_blogInfo->getId());

            //$userLink = $_SERVER["PHP_SELF"].$rg->getRequest();
            //$userLink = "index.php".$rg->getRequest();
            $userLink = $this->getIndexUrl().$this->getRequest();

            return $userLink;
        }

        /**
         * Returns the url of the host where the blog is running
         *
         * @return Returns the url where the blog is running.
         */
        function blogLink( $blogInfo = null )
        {
        	if( $blogInfo == null ) {
            	$this->addParameter( "blogId", $this->_blogInfo->getId());
                $link = $this->getIndexUrl().$this->getRequest();
            }
            else {
                $this->addParameter( "blogId", $blogInfo->getId());
                $link = $this->getIndexUrl().$this->getRequest();
            }

            return $link;
        }

        /**
         * Returns the url where the rss feed is running
         *
         * @param blogInfo A BlogInfo object containing information about the blog.
         * @param profile The RSS profile we'd like to use. It defaults to none.
         * @return The url pointing to the rss feed of the journal.
         * @see BlogInfo
         */
        function rssLink( $profile = "", $blogInfo = null )
        {
        	if( $blogInfo == null )
            	$this->addParameter( "blogId", $this->_blogInfo->getId());
            else
            	$this->addParameter( "blogId", $blogInfo->getId());

            if( $profile != "" )
            	$this->addParameter( "profile", $profile );

            $rssLink = $this->getRssUrl().$this->getRequest();

            return $rssLink;
        }

        /**
         * Returns the url for the rss feed of a category
         *
         * @param category The ArticleCategory object with information about the category
         * whose RSS feed we'd like to generate
         * @aram profile The profile we'd like to generate: RSS 0.90, RSS 1.0, RSS 2.0
         * or XML.
         * @param blogInfo A BlogInfo object containing information about the blog.
         * @return The url pointing to the rss feed of the journal.
         * @see BlogInfo
         */
        function categoryRssLink( $category, $profile = "", $blogInfo = null )
        {
        	$this->addParameter( "categoryId", $category->getId());

        	if( $blogInfo == null )
            	$this->addParameter( "blogId", $this->_blogInfo->getId());
            else
            	$this->addParameter( "blogId", $blogInfo->getId());

            if( $profile != "" )
            	$this->addParameter( "profile", $profile );

            $rssLink = $this->getRssUrl().$this->getRequest();

            return $rssLink;
        }

        function postTrackbackLink( $post )
        {
        	$rdfHeader = '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
                                   xmlns:dc="http://purl.org/dc/elements/1.1/"
                                   xmlns:trackback="http://madskills.com/public/xml/rss/module/trackback/">';

            $trackbackLink = $this->getTrackbackUrl()."?id=".$post->getId();
            $rdfBody = "<rdf:Description
                             rdf:about=\"".$this->postLink($post)."\"
                             dc:identifier=\"".$this->postLink($post)."\"
                             dc:title=\"".$post->getTopic()."\"
                             trackback:ping=\"".$trackbackLink."\"/>";
            $rdfFooter = "</rdf:RDF>";

            return $rdfHeader.$rdfBody.$rdfFooter;
        }

        /**
         * Returns the url to reply to a given comment.
         *
         * @param post An Article object with information about the post
         * @param commen A UserComment object containing information about the post we'd like to reply to.
         * @return The right url to reply to this comment.
         * @see UserComment
         * @see Article
         */
        function replyCommentLink( $post, $comment )
        {
            $this->addParameter( "op", "Comment" );
            $this->addParameter( "articleId", $post->getId());
            $this->addParameter( "parentId", $comment->getId());
            if( $this->_blogInfo != null )
            	$this->addParameter( "blogId", $this->_blogInfo->getId());

            //$replyCommentLink = $_SERVER["PHP_SELF"].$rg->getRequest();
            $replyCommentLink = $this->getIndexUrl().$this->getRequest();

            return $replyCommentLink;
        }

        function getArchiveLink( $date )
        {
        	$this->addParameter( "op", "Default" );
            $this->addParameter( "Date", $date);
            if( $this->_blogInfo != null )
            	$this->addParameter( "blogId", $this->_blogInfo->getId());

            $archiveUrl = $this->getIndexUrl().$this->getRequest();

            return $archiveUrl;
        }

        /**
         * Returns the link to the page showing the trackback statistics for a given post.
         *
         * @param post The post with the information.
         * @return Returns a string with the valid link.
         */
        function postTrackbackStatsLink( $post )
        {
            $this->addParameter( "op", "Trackbacks" );
            $this->addParameter( "articleId", $post->getId());
            $this->addParameter( "blogId", $this->_blogInfo->getId());

            //$tbStatsLink = $_SERVER["PHP_SELF"].$rg->getRequest();
            $tbStatsLink = $this->getIndexUrl().$this->getRequest();

            return $tbStatsLink;
        }

        /**
         * Returns the link to an album
         *
         * @param album The GalleryAlbum object.
         */
        function albumLink( $album = null )
        {
        	$this->addParameter( "op", "ViewAlbum" );
			if( $album == null )
				$this->addParameter( "albumId", "0" );
			else
				$this->addParameter( "albumId", $album->getId());
            $this->addParameter( "blogId", $this->_blogInfo->getId());

            $albumLink = $this->getIndexUrl().$this->getRequest();

            return $albumLink;
        }

        function parentAlbumLink( $album )
        {
        	$this->addParameter( "op", "ViewAlbum" );
            $this->addParameter( "albumId", $album->getParentId());
            $this->addParameter( "blogId", $this->_blogInfo->getId());

            $albumLink = $this->getIndexUrl().$this->getRequest();

            return $albumLink;
        }

        function templatePage( $template )
        {
        	$this->addParameter( "op", "Template" );
            $this->addParameter( "blogId", $this->_blogInfo->getId());
            $this->addParameter( "show", $template );

            $templatePage = $this->getIndexUrl().$this->getRequest();

            return $templatePage;
        }

        /**
         * Returns the link to a resource
         *
         * @param album Generates the correct link to fetch a resource
         */
        function resourceLink( $resource )
        {
            $this->addParameter( "op", "ViewResource" );
            $this->addParameter( "blogId", $this->_blogInfo->getId());
            $this->addParameter( "resource", $resource->getFileName());

            $resourceLink = $this->getIndexUrl().$this->getRequest();

            return $resourceLink;
        }

        /**
         * Returns the link to a resource preview
         *
         * @param album Generates the correct link to fetch a resource preview
         */
        function resourcePreviewLink( $resource )
        {
            $this->addParameter( "blogId", $this->_blogInfo->getId());
            $this->addParameter( "resource", $resource->getFileName());
            $this->addParameter( "mode", "preview" );

            $resourceLink = $this->getResourceServerUrl().$this->getRequest();

            return $resourceLink;
        }

        /**
         * Returns the link to a resource
         *
         * @param resource Generates the correct link to fetch a resource
         */
        function resourceDownloadLink( $resource )
        {
            $this->addParameter( "blogId", $this->_blogInfo->getId());
            $this->addParameter( "resource", $resource->getFileName());

            $resourceLink = $this->getResourceServerUrl().$this->getRequest();

            return $resourceLink;
        }

        /**
         * Returns a string representing the request
         *
         * @return A String object representing the request
         */
        function getRequest()
        {
        	$request = "";

			if( $this->isXHTML()) 
				$amp = "&amp;";
			else
				$amp = "&";

        	foreach( $this->_params as $name => $value )
            {
               	if( $request == "" )
                	$request .= "?";
                else
                	$request .= $amp;

                $request .= urlencode($name) ."=".urlencode($value);
            }

            $this->reset();

            return $request;
        }
        
		/**
		 * generates the correct path to a file in the template folder, without having to worry
		 * whether the template was installed in /templates/ or in /templates/blog_X/
		 *
		 * @param file
		 * @return A string
		 */
		function getTemplateFile( $file )
		{
		    // get the current template set
		    $blogSettings = $this->_blogInfo->getSettings();
		    $template = $blogSettings->getValue( "template" );
		    
		    // define this couple of things
        	$baseUrl = $this->getBaseUrl( $abs );
            $url = "$baseUrl/templates/";
		    		    
		    // is it a blog template?
		    $blogTemplates = $blogSettings->getValue( "blog_templates" );
		    
		    if( !is_array($blogTemplates ))
		        $url .= "$template/$file";
		    else {
		        if( in_array( $template, $blogTemplates ))
		          $url .= "blog_".$this->_blogInfo->getId()."/$template/$file";
		        else
		          $url .= "$template/$file";
		    }
		    
            return $url;		    
		}        
    }
?>
