<?php

	include_once( PLOG_CLASS_PATH."class/config/config.class.php" );
	include_once( PLOG_CLASS_PATH."class/net/baserequestgenerator.class.php" );
    include_once( PLOG_CLASS_PATH."class/net/prettyrequestgenerator.class.php" );
    include_once( PLOG_CLASS_PATH."class/net/rawrequestgenerator.class.php" );
    include_once( PLOG_CLASS_PATH."class/net/modrewriterequestgenerator.class.php" );

    define( CHECK_CONFIG_REQUEST_MODE, 0 );
    define( NORMAL_REQUEST_MODE, 1 );
    define( SEARCH_ENGINE_FRIENDLY_MODE, 2);
    define( MODREWRITE_MODE, 3);

	class RequestGenerator extends Object {

    	var $_mode;

    	function RequestGenerator( $blogInfo = null, $mode = CHECK_CONFIG_REQUEST_MODE )
        {
        	/*if( $blogInfo == null ) {
            	throw( new Exception( "blogInfo can't be null." ));
                die();
            }*/

            // check the mode
        	if( $mode == CHECK_CONFIG_REQUEST_MODE ) {
            	$config =& Config::getConfig();
                $mode   = $config->getValue( "request_format_mode" );
            }

            $this->_mode  = $mode;

            if( $this->_mode == SEARCH_ENGINE_FRIENDLY_MODE ) {
            	$this->_rg = new PrettyRequestGenerator( $blogInfo );
                //echo "pretty";

            } elseif( $this->_mode == MODREWRITE_MODE ) {
            	$this->_rg = new ModRewriteRequestGenerator( $blogInfo );
                //echo "mod";

            } else {
                $this->_rg = new RawRequestGenerator( $blogInfo );
                //echo "raw";
            }

        }

        function getBaseUrl( $abs = true )
        {
        	return $this->_rg->getBaseUrl( $abs );
        }

        function getIndexUrl( $abs = true )
        {
        	return $this->_rg->getIndexUrl( $abs );
        }

        function getRssUrl( $abs = true )
        {
        	return $this->_rg->getRssUrl( $abs );
        }

        function getTrackbackUrl( $abs = true )
        {
        	return $this->_rg->getTrackbackUrl( $abs );
        }

        function getUrl( $res, $abs = true )
        {
        	return $this->_rg->getUrl( $res, $abs );
        }

        /**
         * Adds a parameter to the request
         *
         * @param paramName Name of the parameter
         * @param paramValue Value given to the parameter
         */
        function addParameter( $paramName, $paramValue )
        {
        	$this->_rg->addParameter( $paramName, $paramValue );
        }

        function getRequest()
        {
        	return $this->_rg->getRequest( $abs );
        }

        /**
         * Returns the permalink for a post
         *
         * @param post The Article object
         * @return The link for the permalink
         */
		function postPermalink( $post, $blogInfo = null )
        {
        	if( $blogInfo != null ) $this->_blogInfo = $blogInfo;
        	return $this->_rg->postPermalink( $post );
        }

        /**
         * Returns the comment link for a post
         *
         * @param post The Article object
         * @return The correct string
         */
        function postCommentLink( $post )
        {
        	return $this->_rg->postCommentLink( $post );
        }

        /**
         * Returns the link for the post
         *
         * @param post The Article object
         * @return The link for the post
         */
        function postLink( $post )
        {
            return $this->_rg->postLink( $post );
        }

        /**
         * Returns the link for the post. This is kind of a dirty trick... :( This is only meant to be
         * used in the template that generates the rss feed for a blog.
         *
         * @param post The Article object
         * @return The link for the post
         */
        function postRssLink( $post )
        {
        	return $this->_rg->postRssLink( $post );
        }

        /**
         * Returns the url for the link tracker feature
         *
         * @return The url that shows the results of the link tracker.
         */
        function linkTrackerLink()
        {
        	return $this->_rg->linkTrackerLink();
        }

        /**
         * Returns the link of a category.
         *
         * @param post The post from which we'll fetch the category and then generate the right link.
         * @return The url pointing to the page with only the posts belonging to that category.
         * @see Article
         * @see categoryLink
         */
        function postCategoryLink( $post )
        {
        	return $this->_rg->postCategoryLink( $post );
        }

        /**
         * Returns the link but given a category. Does the same as postCategoryLink but this time we don't need
         * a post but an ArticleCategory object.
         *
         * @see postCategoryLink
         * @see ArticleCategory
         * @param An ArticleCategory object containing the information regarding the category.
         * @return A string with the correct url pointing to the page that will show only the posts that belong
         * to the given category.
         */
        function categoryLink( $category )
        {
        	return $this->_rg->categoryLink( $category );
        }

        /**
         * Returns a link to only the articles of the user
         *
         * @param user The user whose posts we would like to see
         * @param category Optionally, we can specify an ArticleCategory object
         * @return A string containing the right url to only the posts of the user.
         * @see UserInfo
         * @see ArticleCategory
         */
        function postUserLink( $user, $category = null )
        {
        	return $this->_rg->postUserLink( $user, $category );
        }

        /**
         * Returns the url of the host where the blog is running
         *
         * @return Returns the url where the blog is running.
         */
        function blogLink( $blogInfo = null )
        {
        	return $this->_rg->blogLink( $blogInfo );
        }

        function getArchiveLink( $date )
        {
        	return $this->_rg->getArchiveLink( $date );
        }

        /**
         * Returns the url for the rss feed of a category
         *
         * @param category The ArticleCategory object with information about the category
         * whose RSS feed we'd like to generate
         * @aram profile The profile we'd like to generate: RSS 0.90, RSS 1.0, RSS 2.0
         * or XML.
         * @param blogInfo A BlogInfo object containing information about the blog.
         * @return The url pointing to the rss feed of the journal.
         * @see BlogInfo
         */
        function categoryRssLink( $category, $profile = "", $blogInfo = null )
        {
        	return $this->_rg->categoryRssLink( $category, $profile, $blogInfo );
        }

        /**
         * Returns the url where the rss feed is running
         *
         * @param blogInfo A BlogInfo object containing information about the blog.
         * @param profile The RSS profile we'd like to use. It defaults to none.
         * @return The url pointing to the rss feed of the journal.
         * @see BlogInfo
         */
        function rssLink( $profile = "", $blogInfo = null )
        {
        	return $this->_rg->rssLink( $profile, $blogInfo );
        }

        /**
         * Returns the url to reply to a given comment.
         *
         * @param post An Article object with information about the post
         * @param commen A UserComment object containing information about the post we'd like to reply to.
         * @return The right url to reply to this comment.
         * @see UserComment
         * @see Article
         */
        function replyCommentLink( $post, $comment )
        {
        	return $this->_rg->replyCommentLink( $post, $comment );
        }

        /**
         * Manually adds the "show more" link in a post.
         *
         * @param post The post we are going to cut.
         * @param maxWords Amount of words we'd like to allow.
         * @param linkText Text we are going to show.
         * @return The modified link.
         */
        function addShowMoreLink( $post, $maxWords, $linkText )
        {
        	return $this->_rg->addShowMoreLink( $post, $maxWords, $linkText );
        }

        /**
         * Returns the trackback link for a given post
         *
         * @param post The post with the necessary information.
         * @return A string representing the rdf trackback link.
         */
        function postTrackbackLink( $post )
        {
        	return $this->_rg->postTrackbackLink( $post );
        }

        /**
         * Returns the link to the page showing the trackback statistics for a given post.
         *
         * @param post The post with the information.
         * @return Returns a string with the valid link.
         */
        function postTrackbackStatsLink( $post )
        {
        	return $this->_rg->postTrackbackStatsLink( $post );
        }

        /**
         * Returns the link to an album
         *
         * @param album The GalleryAlbum object.
         */
        function albumLink( $album = null )
        {
        	return $this->_rg->albumLink( $album );
        }

        function parentAlbumLink( $album )
        {
        	return $this->_rg->parentAlbumLink( $album );
        }

        function templatePage( $template )
        {
        	return $this->_rg->templatePage( $template );
        }

        /**
         * Returns the link to a resource
         *
         * @param resource Generates the correct link to fetch a resource
         */
        function resourceLink( $resource )
        {
        	return $this->_rg->resourceLink( $resource );
        }

        /**
         * Returns the link to download a resource
         *
         * @param resource The resource
         */
        function resourceDownloadLink( $resource )
        {
        	return $this->_rg->resourceDownloadLink( $resource );
        }

        /**
         * Returns the link to a resource preview
         *
         * @param album Generates the correct link to fetch a resource preview
         */
        function resourcePreviewLink( $resource )
        {
        	return $this->_rg->resourcePreviewLink( $resource );
        }
		
		/**
		 * whether we should generate valid xhtml requests or not
		 * (used for example when sending out messages, as some email clients will
		 * go to the wrong url when using xhtml message
		 *
		 * @param enable Whether to enable xhtml or not (enabled by default)
		 * @return Always true
		 */
		function setXHTML( $enabled = true )
		{
			return $this->_rg->setXHTML( $enabled );
		}
		
		/**
		 * whether xhtml mode is enabled or not
		 *
 		 * @return True if enabled or false otherwise
		 */
		function isXHTML()
		{
			return $this->_rg->isXHTML();
		}
		
		/**
		 * generates the correct path to a file in the template folder, without having to worry
		 * whether the template was installed in /templates/ or in /templates/blog_X/
		 *
		 * @param file
		 * @return A string
		 */
		function getTemplateFile( $file )
		{
		    return $this->_rg->getTemplateFile( $file );  
		}
    }
?>
