<?php

	include_once( PLOG_CLASS_PATH."class/net/baserequestgenerator.class.php" );
	include_once( PLOG_CLASS_PATH."class/net/requestgenerator.class.php" );

    /**
     * This class is like any other request generator but the difference
     * is that we do not require a BlogInfo object in order to generate the requests
     * but it is enough that we pass the blog identifier for every operation.
     * It is only meant to be used in places where we do not have a BlogInfo object
     * available such as the summary page. Outside that, it should *not* be used.
     *
     * It doesn't implement all the methods, not because we can't but because we
     * don't need them all in the summary page ;)
     *
     * @see BaseRequestGenerator
     */
	class VerySimpleRequestGenerator extends BaseRequestGenerator {

    	var $_params;
        var $_SERVER;
        var $_blogInfo;
        var $_baseUrl;
		var $_mode;

    	function VerySimpleRequestGenerator( $blogInfo = null )
        {
        	$this->BaseRequestGenerator();

            $this->_params = Array();

            $this->_SERVER = HttpVars::getServer();

            $this->_blogInfo = $blogInfo;

            $config =& Config::getConfig();
            $this->_baseUrl = $config->getValue( "base_url" );
			$this->_mode = $config->getValue( "request_format_mode" );
        }

        function getBaseUrl( $abs = true )
        {
        	return $this->_baseUrl;
        }

        function getIndexUrl( $abs = true )
        {
        	throw( new Exception( "Sorry, this method has not been implemented by this class." ));
        }

        function getRssUrl( $abs = true )
        {
        	throw( new Exception( "Sorry, this method has not been implemented by this class." ));
        }

        function getTrackbackUrl( $abs = true )
        {
        	throw( new Exception( "Sorry, this method has not been implemented by this class." ));
        }

        function getUrl( $res, $abs = true )
        {
        	throw( new Exception( "Sorry, this method has not been implemented by this class." ));
        }

        function addParameter( $paramName, $paramValue )
        {
        	$this->_params[$paramName] = $paramValue;
        }

        function reset()
        {
        	$this->_params = Array();
        }

        function getRequest()
        {
        	throw( new Exception( "Sorry, this method has not been implemented by this class." ));
        }

		function postPermalink( $post )
        {
            $blogId = $post->getBlog();
			if( $this->_mode == SEARCH_ENGINE_FRIENDLY_MODE ) 
				$url = $this->_baseUrl."/post/$blogId/".$post->getId();
			else
				$url = $this->_baseUrl."/index.php?op=ViewArticle&amp;blogId=$blogId&amp;articleId=".$post->getId();
				
			return $url;
        }


        function postCommentLink( $post )
        {
            $blogId = $post->getBlog();
			if( $this->_mode == SEARCH_ENGINE_FRIENDLY_MODE )
				$url = $this->_baseUrl."/comment/$blogId/".$post->getId();
			else
				$url = $this->_baseUrl."/index.php?op=CommentArticle&amp;blogId=$blogId&amp;articleId=".$post->getId();
				
			return $url;
        }


        function postLink( $post )
        {
        	throw( new Exception( "Sorry, this method has not been implemented by this class." ));
        }

        function postRssLink( $post )
        {
        	throw( new Exception( "Sorry, this method has not been implemented by this class." ));
        }

        function linkTrackerLink()
        {
        	throw( new Exception( "Sorry, this method has not been implemented by this class." ));
        }

        function postCategoryLink( $post )
        {
            $blogId = $post->getBlog();
			if( $this->_mode == SEARCH_ENGINE_FRIENDLY_MODE ) 		
				$url = $this->_baseUrl."/category/$blogId/".$post->getCategoryId();
			else
				$url = $this->_baseUrl."/index.php?op=Default&amp;blogId=$blogId&amp;postCategoryId=".$post->getCategoryId();
				
			return $url;
        }

        function categoryLink( $blogId, $category )
        {
			if( $this->_mode == SEARCH_ENGINE_FRIENDLY_MODE ) 		
				$url = $this->_baseUrl."/category/$blogId/".$category->getId();
			else
				$url = $this->_baseUrl."/index.php?op=Default&amp;blogId=$blogId&amp;postCategoryId=".$category->getId();
			
			return $url;
        }

        function postUserLink( $user, $category = null )
        {
        	throw( new Exception( "Sorry, this method has not been implemented by this class." ));
        }

        function blogLink( $blog )
        {
			if( $this->_mode == SEARCH_ENGINE_FRIENDLY_MODE ) {
			    $config =& Config::getConfig();
                if( $config->getValue( "pretty_urls_force_use_username" )) {
                    $users = new Users();
                    $userInfo = $users->getUserInfoFromId( $blog->getOwner());
                    $url = $this->getBaseUrl()."/".$userInfo->getUsername();
                }
                else {
				    $url = $this->getBaseUrl()."/".$blog->getId();
				}
            }
			else
				$url = $this->getBaseUrl()."/index.php?blogId=".$blog->getId();
				
			return $url;
        }

        function rssLink( $blog )
        {
            $blogId = $blog->getId();
			if( $this->_mode == SEARCH_ENGINE_FRIENDLY_MODE ) 
				$url = $this->_baseUrl."/rss/".$blogId;
			else
				$url = $this->_baseUrl."/rss.php?blogId=$blogId";
				
			return $url;
        }

        function replyCommentLink( $post, $comment )
        {
        	throw( new Exception( "Sorry, this method has not been implemented by this class." ));
        }

        function addShowMoreLink( $post, $maxWords, $linkText )
        {
        	throw( new Exception( "Sorry, this method has not been implemented by this class." ));
        }

        function postTrackbackLink( $post )
        {
        	throw( new Exception( "Sorry, this method has not been implemented by this class." ));
        }

        function getArchiveLink( $date )
        {
        	throw( new Exception( "Sorry, this method has not been implemented by this class." ));
        }

        function postTrackbackStatsLink( $post )
        {
            $blogId = $post->getBlog();
			if( $this->_mode == SEARCH_ENGINE_FRIENDLY_MODE ) 			
				$url = $this->_baseUrl."/trackbacks/$blogId/".$post->getId();
			else
				$url = $this->_baseUrl."/index.php?op=Trackbacks&amp;blogId=$blogId&amp;articleId=".$post->getId();
				
			return $url;
        }
    }
?>
