<?php

	include_once( PLOG_CLASS_PATH."class/object/object.class.php" );
    include_once( PLOG_CLASS_PATH."class/database/adodb/adodb.inc.php" );
    include_once( PLOG_CLASS_PATH."class/config/config.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/blogs.class.php" );

    /**
     * This is the base class from which every plugin object should inherit.
     *
     * Plugins can also offer a configuration interface in case there is anything
     * that can be configured. Plugins can set the $this->configMessage to the desired
     * html code that represents the interface or use the setConfigurationMessage
     * method.
     *
     * The most usual kind of interface will be a form if the plugin is expecting something
     * from the user. The form can have as many fields as needed but the following two are
     * mandatory if we are to make the admin interface recognize that the user is trying to
     * configure a plugin:
     * <ul>
     * <li>op = updatePluginConfig</li>
     * <li>pluginId = the plugin's own identifier</li>
     * </ul>
     *
     * The first one will help the admin interface recognize what we are trying to  do
     * (any other value will be ignored or can have unexpected results) and the second
     * one will identify which plugin we are trying to configure.
     *
     * Once the user presses the "Submit" button, the plugin manager will call the
     * configure() method of the plugin using the data received from the form as the
     * parameter.
     *
     * Plugins have also now two more additional objects: blogInfo and blogSettings. The first
     * one is a BlogInfo object containing information about the blog that is executing this
     * plugin. blogSettings is an object of the BlogSettings class that contains all the settings
     * of this blog. Use with care :)
     *
     * Finally, just mention that it is not mandatory to generate an html form. It could also
     * be that we are running a statistics plugin and we could use this feature to generate the
     * report.
     *
     * Please see the configure() method and the PluginTest class (plugintest.class.php)
     * for a real-life sample.
     *
     * @see BlogInfo
     * @see BlogSettings
     * @see PluginTest
     */
    class PluginBase extends Object {

    	/** ADOConnection object to connect to the database
        */
    	var $db;

        /**
         * This should be filled by the objects that extend this class to provide
         * some help on how to use the plugin.
         */
        var $desc = "This plugin has no description";

        /**
         * This should be filled with the author of the plugin.
         */
        var $author = "This plugin has no author";

        /**
         * This should not be modified as it is set by the plugin manager.
         * @private
         */
        var $_id = "This plugin has no identifier";

        /**
         * Set this to the html code that you would like to display when
         * configuring this plugin.
         */
        var $configMessage = "No configuration options available";

        /**
         * Error message, if any, that happened when configuring the plugin.
         */
        var $errorMessage = "No error message";

        /**
         * This contains information about the blog that is currently executing or configuring
         * this plugin.
         * @see BlogInfo
         */
        var $blogInfo;

        /**
         * This one contains the blog specific settings
         * @see BlogSettings
         */
        var $blogSettings;

	/**
	 * User who is running the plugin.
	 */
	var $userInfo;

    	/**
         * Constructor. Feel free to do here whatever you need to.
         */
    	function PluginBase()
        {
        	$this->Object();
        }

        /**
         * Returns a string with some information about the plugin.
         *
         * @return Returns a string describing the plugin.
         */
        function getDescription()
        {
        	return $this->desc;
        }

        /**
         * Returns the string describing the author of this plugin.
         *
         * @return The author of the plugin.
         */
        function getAuthor()
        {
        	return $this->author;
        }

        /**
         * Returns the identifier of the plugin, the name which can be used
         * in templates.
         *
         * @return The plugin identifier.
         */
        function getId()
        {
        	return $this->_id;
        }

        /**
         * This function is called only once when the plugin is registered.
         *
         * @return Nothing.
         */
        function register()
        {

        }

        /**
         * Connects to the database. If no parameters are provided, the default settings
         * will be used.
         *
         * @param user Username used to connect to the database.
         * @param password Password used to connect to the database.
         * @param host Database server.
         * @param database Database where we are going to connect.
         * @return Returns true if connection successful.
         */
        function connectDb( $user = null, $password = null, $host = null, $database = null )
        {
        	if( $user == null && $password == null && $host == null && $database == null ) {
            	$config   = Config::getConfig();
				$user     = $config->getValue( "db_username" );
                $password = $config->getValue( "db_password" );
                $host     = $config->getValue( "db_host" );
                $database = $config->getValue( "db_database" );
            }

            // connect to the database
            $this->db = NewADOConnection( 'mysql' );
            if( !$this->db->Connect( $host, $user, $password, $database ))
            	return false;

            $this->db->SetFetchMode( ADODB_FETCH_ASSOC );
 
            return true;
        }

        /**
         * @private
         */
        function setBlogInfo( $blogInfo )
        {
        	$this->blogInfo = $blogInfo;
            $this->blogSettings = $blogInfo->getSettings();
        }

        /**
         * @private
         */
        function setUserInfo( $userInfo )
        {
			$this->userInfo = $userInfo;
        }

        /**
         * Saves the blog settings back to the database so that they are permanent. This function
         * can be called anytime but it is specially useful from inside the config() method,
         * when the user has just configured something in the admin interface
         *
         * @return Returns true if the settings were saved ok or false otherwise.
         */
        function updateBlogSettings()
        {
        	$blogs = new Blogs();

            $result = $blogs->updateBlogSettings( $this->blogInfo->getId(), $this->blogSettings);

            return $result;
        }

        /**
         * Returns the configuration dialog that the plugin uses. Returns by default the
         * value of the $configMessage attribute of the class, but it can also be overwritten
         * by child classes to do anything else.
         *
         * @return String with html code that will be displayed to the user wanting to configure
         * the plugin.
         */
        function getConfigurationMessage()
        {
        	return $this->configMessage;
        }

        /**
         * This function must be overwritten by child classes wishing to configure anything.
         * It will be called from within the administration interface right after the user
         * submitted the plugin configuration settings.
         *
         * This method will receive a Request object as it was sent by the browser if
         * the plugin defined any configuration view and it included a "Submit" button, poiting
         * back to the administration interface.
         *
         * This method will have to get the values needed and store them using the
         * $this->blogSettings object. The blogSettings object offers two very useful methods:
         * getValue and setValue. Using setValue(...) we can store as many values as needed
         * as long as we don't forget to call the updateBlogSettings method to make those
         * permanent by storing them in the database.
         *
         * The method can return true if everything went fine or false if there was any error.
         * In case there was an error, we can optionally set an error message that will be
         * reported to the user using the setConfigurationErrorMessage method or by simply
         * assining the string to $this->errorMessage.
         *
         * @param params An assiociative array containing the parameters configured by the user.
         * @return Returns true if everything went fine or false otherwise.
         * @see Request
         */
        function configure( $params )
        {
        	return true;
        }

        /**
         * This function should be called when configure() returned false and we would like to
         * exactly know what happened.
         *
         * @return Returns a string with the error.
         */
        function getConfigurationErrorMessage()
        {
        	return $this->errorMessage;
        }

        /**
         * Used to set the error message. It is an commodity function for
         * $this->errorMessage = $message.
         *
         * @param message The error message that we'd like the user to see.
         */
        function setConfigurationErrorMessage( $message )
        {
        	$this->errorMessage = $message;
        }
    }
?>
