<?php

	include_once( PLOG_CLASS_PATH."class/object/object.class.php" );
	include_once( PLOG_CLASS_PATH."class/config/config.class.php" );
    include_once( PLOG_CLASS_PATH."class/misc/glob.class.php" );

    $_pluginManager_pluginList;

    /**
     * Implements a 'plugin manager' a class that takes care of loading plugins,
     * objects subclassing the Plugin class which add some additional features to
     * the program
     */    
    class PluginManager extends Object {

    	var $_pluginDir;
        var $_filePattern;
        var $_blogInfo;

        /**
         * Constructor.
         *
         * @param pluginDir Specifies from which folders templates should
         * be loaded from.
         */
    	function PluginManager( &$blogInfo, &$userInfo, $pluginDir = "./plugins/", $filePattern = "plugin*.class.php" )
        {
            global $_pluginManager_pluginList;
			$config = Config::getConfig();
            $this->_enabled = $config->getValue( "plugin_manager_enabled" );
			if( !$this->_enabled ) {
				// in case the plugin manager is disabled, we won't return an empty array
				// when asking for which plugins were loaded...
				$_pluginManager_pluginList =  Array();			
				return;
			}
        	$this->_pluginDir = $pluginDir;
            $this->_filePattern = $filePattern;

            $this->_blogInfo = $blogInfo;
            $this->_userInfo = $userInfo;

            $_pluginManager_pluginList =  Array();

            $this->_loadPlugins();
        }

        /**
         * Returns wether the plugin manager is enabled or not.
         *
         * @return A boolean value telling wether the plugin manager is enabled or not.
         */
        function isEnabled()
        {
        	return $this->_enabled;
        }

        /**
         * Loads all the plugins from disk
         *
         * @private
         */
        function _loadPlugins()
        {
        	if( File::isReadable( $this->_pluginDir )) {
            	foreach(Glob::glob($this->_pluginDir, $this->_filePattern) as $pluginFile ) {
                	include_once( PLOG_CLASS_PATH.$pluginFile );
            	}
            }
        }

        /**
         * Returns the folder used to store the plugins.
         *
         * @return The folder used to store the plugins.
         */
        function getPluginDir()
        {
        	return $this->_pluginDir;
        }

        /**
         * Static function to register a plugin.
         */
        function registerPlugin( $name, $class )
        {
    		global $_pluginManager_pluginList;

            // instantiate a new plugin object of that type
            $pluginClass = new $class();

            // set its identifier and save it in our array of registered plugins
            $_pluginManager_pluginList["$name"] = &$pluginClass;
            $_pluginManager_pluginList["$name"]->setBlogInfo( $this->_blogInfo );
            $_pluginManager_pluginList["$name"]->setUserInfo( $this->_userInfo );
        	$_pluginManager_pluginList["$name"]->register();
            $pluginClass->_id = $name;
            // set the identifier field
        	return true;
    	}

        /**
         * Returns the array of plugins.
         *
         * @return An array of PluginBase objects.
         */
        function getPlugins()
        {
    		global $_pluginManager_pluginList;

        	return $_pluginManager_pluginList;
        }

        function getPluginConfigurationMessage( $pluginId )
        {
        	global $_pluginManager_pluginList;

            $configMessage = "";

            if( isset($_pluginManager_pluginList["$pluginId"])) {
            	$configMessage = $_pluginManager_pluginList["$pluginId"]->getConfigurationMessage();
            }

            return $configMessage;
        }

        function configurePlugin( $pluginId, $params )
        {
        	global $_pluginManager_pluginList;

            $result = false;

            if( isset($_pluginManager_pluginList["$pluginId"])) {
            	$pluginObj =& $_pluginManager_pluginList["$pluginId"];
                $result = $pluginObj->configure( $params );
                //print("after configuring: ".htmlspecialchars($pluginObj->toString()));
            }

            return $result;
        }

        function getPluginConfigurationErrorMessage( $pluginId )
        {
        	global $_pluginManager_pluginList;

            $errorMessage = "";

            if( isset($_pluginManager_pluginList["$pluginId"])) {
            	$pluginObj = $_pluginManager_pluginList["$pluginId"];
            	$errorMessage = $pluginObj->getConfigurationErrorMessage();
            }

            return $errorMessage;
        }
    }
?>
