<?php

	include_once( PLOG_CLASS_PATH."class/object/object.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/bloginfo.class.php" );

    include_once( PLOG_CLASS_PATH."class/config/config.class.php" );

    include_once( PLOG_CLASS_PATH."class/security/nullpipelinefilter.class.php" );
    include_once( PLOG_CLASS_PATH."class/security/hostsblacklist.class.php" );
    include_once( PLOG_CLASS_PATH."class/security/contentfilter.class.php" );
    include_once( PLOG_CLASS_PATH."class/security/commentfilter.class.php" );
    include_once( PLOG_CLASS_PATH."class/security/bayesianfilter.class.php" );

    /**
     * Implementation of a basic security framework based on a
     * pipeline. Every element of the pipeline implements a simple
     * security mechanism. When one of the filters in the pipeline
     * find a condition that is matched by the incoming request, the
     * request will be blocked
     */
	class Pipeline extends Object {

    	/**
         * HTTP request that will be used if the filter is doing
         * some content filtering
         */
    	var $_httpRequest;

        /**
         * the BlogInfo object that has information about the blog
         * that is currently processing the incoming request
         */
        var $_blogInfo;

        /**
         * Array with the name of the classes that implement filters
         */
        var $_pipelineFilters = Array (
                                        "NullPipelineFilter",
                                        "HostsBlacklist",
                                        "ContentFilter",
                                        "CommentFilter",
                                        "BayesianFilter",
                                      );

        /**
         * Array of objects that implement the PipelineFilter interface
         */
        var $_filters = Array();

        var $_result;

        /**
         * Constructor
         *
         * @param httpRequest The HTTP request
         * @param blogInfo The BlogInfo object with information about the blog
         * that is currently executing this pipeline
         */
    	function Pipeline( $httpRequest, $blogInfo = null )
        {
        	$this->Object();

            $this->_httpRequest = $httpRequest;
            $this->_blogInfo    = $blogInfo;

            $this->_addFilters();

            $this->_result;
        }

        /**
         * Loads all the filters
         * @private
         */
        function _addFilters()
        {
        	$pipelineRequest = new PipelineRequest( $this->_httpRequest, $this->_blogInfo );

        	foreach( $this->_pipelineFilters as $filterClass ) {
            	array_push( $this->_filters, new $filterClass( $pipelineRequest ));
            }

            return true;
        }

        /**
         * Processes the pipeline, using the request and blogInfo
         * objects as given in the constructor.
         */
        function process()
        {
        	// check if the pipeline is enabled
            $config =& Config::getConfig();
            if( $config->getValue( "security_pipeline_enabled" ) == false ) {
            	// pipeline is disabled, so everything's fine
            	return new PipelineResult( true );
            }

            // if enabled, then check all the filters
        	foreach( $this->_filters as $filter ) {
            	$result = $filter->filter();
                // if there was an error, we better say so now
                // and quite, making sure that we're keeping the
                // error code
                if( !$result->isValid()) {                
                	$this->_result = $result;
                	return $result;
                }
            }

            $this->_result = $result;
            return $result;
        }
    }
?>
