<?php

	include_once( PLOG_CLASS_PATH."class/object/object.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/article.class.php" );
    include_once( PLOG_CLASS_PATH."class/net/requestgenerator.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/bloginfo.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/articlecategory.class.php" );
    include_once( PLOG_CLASS_PATH."class/dao/userinfo.class.php" );
    include_once( PLOG_CLASS_PATH."class/net/httpvars.class.php" );
    include_once( PLOG_CLASS_PATH."class/net/prettyrequestgenerator.class.php" );
    include_once( PLOG_CLASS_PATH."class/net/rawrequestgenerator.class.php" );

    /**
     * This class provides methods to easily carry out some operations while inside the templates.
     * For example, it is quite painful to generate some of the urls that are used by pLog, so this
     * class provides methods to easily generate those. If, say, we want to get the permalink url for
     * a post, we do not have to know the format ourselves but rather, call the postPermalink function
     * that will generate it for us. This gives us the chance to change the format of these urls
     * in the future without -hopefully- having to modifiy the templates.
     * <br>
     * Of course the use of this class is optional but doing so, we will ensure future compatibility and
     * also readibility of our templates.
     */
    class TemplateUtils extends Object {

		var $_blogInfo;
        var $_rg;
        var $_mode;

    	/**
         * Constructor. We need the blogInfo object to get some information
         * about the current blog and so on
         */
    	function TemplateUtils( $blogInfo )
        {
        	$this->Object();
        	$this->_blogInfo = $blogInfo;

            $this->_rg = new RequestGenerator( $blogInfo );
        }

        function getUrl( $res, $abs = true )
        {
        	return $this->_rg->getUrl( $res, $abs );
        }

    	/**
         * Returns the permalink for a post
         *
         * @param post The Article object
         * @return The link for the permalink
         */
		function postPermalink( $post )
        {
        	return $this->_rg->postPermalink( $post );
        }

        /**
         * Returns the comment link for a post
         *
         * @param post The Article object
         * @return The correct string
         */
        function postCommentLink( $post )
        {
        	return $this->_rg->postCommentLink( $post );
        }

        /**
         * Returns the link for the post
         *
         * @param post The Article object
         * @return The link for the post
         */
        function postLink( $post )
        {
            return $this->_rg->postLink( $post );
        }

        /**
         * Returns the link for the post. This is kind of a dirty trick... :( This is only meant to be
         * used in the template that generates the rss feed for a blog.
         *
         * @param post The Article object
         * @return The link for the post
         */
        function postRssLink( $post )
        {
        	return $this->_rg->postRssLink( $post );
        }

        /**
         * Returns the url for the link tracker feature
         *
         * @return The url that shows the results of the link tracker.
         */
        function linkTrackerLink()
        {
        	return $this->_rg->linkTrackerLink();
        }

        /**
         * Returns the link of a category.
         *
         * @param post The post from which we'll fetch the category and then generate the right link.
         * @return The url pointing to the page with only the posts belonging to that category.
         * @see Article
         * @see categoryLink
         */
        function postCategoryLink( $post )
        {
        	return $this->_rg->postCategoryLink( $post );
        }

        /**
         * Returns the link but given a category. Does the same as postCategoryLink but this time we don't need
         * a post but an ArticleCategory object.
         *
         * @see postCategoryLink
         * @see ArticleCategory
         * @param An ArticleCategory object containing the information regarding the category.
         * @return A string with the correct url pointing to the page that will show only the posts that belong
         * to the given category.
         */
        function categoryLink( $category )
        {
        	return $this->_rg->categoryLink( $category );
        }

        /**
         * Returns a link to only the articles of the user
         *
         * @param user The user whose posts we would like to see
         * @param category Optionally, we can specify an ArticleCategory object
         * @return A string containing the right url to only the posts of the user.
         * @see UserInfo
         * @see ArticleCategory
         */
        function postUserLink( $user, $category = null )
        {
        	return $this->_rg->postUserLink( $user, $category );
        }

        /**
         * Returns the url of the host where the blog is running
         *
         * @return Returns the url where the blog is running.
         */
        function blogLink( $blogInfo = null )
        {
        	return $this->_rg->blogLink( $blogInfo );
        }

        /**
         * Returns the url where the rss feed is running
         *
         * @param blogInfo A BlogInfo object containing information about the blog.
         * @return The url pointing to the rss feed of the journal.
         * @see BlogInfo
         */
        function rssLink( $blogInfo )
        {
        	return $this->_rg->rssLink( $blogInfo );
        }

        /**
         * Returns the url to reply to a given comment.
         *
         * @param post An Article object with information about the post
         * @param commen A UserComment object containing information about the post we'd like to reply to.
         * @return The right url to reply to this comment.
         * @see UserComment
         * @see Article
         */
        function replyCommentLink( $post, $comment )
        {
        	return $this->_rg->replyCommentLink( $post, $comment );
        }

        /**
         * Changes the content type header from within the template.
         * <b>NOTE: </b> does not work properly yet...
         *
         * @param mymeType A valid MIME type identifier.
         */
        function setContentType( $mimeType )
        {
        	header( $mimeType );
        }

        /**
         * Transforms some characters into their HTML entities
         *
         * @param string The string we'd like to translate
         * @return Returns the translated string.
         */
        function htmlTranslate( $string )
        {
        	return htmlspecialchars( $string );
        }

        /**
         * Manually adds the "show more" link in a post.
         *
         * @param post The post we are going to cut.
         * @param maxWords Amount of words we'd like to allow.
         * @param linkText Text we are going to show.
         * @return The modified link.
         */
        function addShowMoreLink( $post, $maxWords, $linkText )
        {
        	$textFilter = new TextFilter();
            $result = $textFilter->cutText( $post->getText(), $maxWords );
            if( $result != $post->getText()) {
            	$rg = new RequestGenerator();
                $rg->addParameter( "op", "ViewArticle" );
                $rg->addParameter( "articleId", $post->getId());
                $rg->addParameter( "blogId", $this->_blogInfo->getId());
                $showMoreLink = " <a href=\"index.php".$rg->getRequest()."\">".$linkText."</a>";
                $result .= $showMoreLink;
            }

            return $result;
        }

        /**
         * Returns the trackback link for a given post
         *
         * @param post The post with the necessary information.
         * @return A string representing the rdf trackback link.
         */
        function postTrackbackLink( $post )
        {
        	return $this->_rg->postTrackbackLink( $post );
        }

        /**
         * Returns the link to the page showing the trackback statistics for a given post.
         *
         * @param post The post with the information.
         * @return Returns a string with the valid link.
         */
        function postTrackbackStatsLink( $post )
        {
        	return $this->_rg->postTrackbackStatsLink( $post );
        }

        function getArchiveLink( $date )
        {
        	return $this->_rg->getArchiveLink( $date );
        }

        /**
         * Cuts a string after 'n' characters.
         *
         * @param string The string.
         * @return The string cut to 'n' characters.
         */
        function cutString( $string, $n )
        {
        	return substr( $string, 0, $n );
        }

        /**
         * Returns an array with all the links in a string.
         *
         * @param string The string
         * @return An array with the links in the string.
         */
        function getLinks( $string )
        {
        	$regexp = "|<a href=\"(.+)\">(.+)</a>|U";
            $result = Array();

            if( preg_match_all( $regexp, $string, $out, PREG_PATTERN_ORDER )) {
            	foreach( $out[1] as $link ) {
             		array_push( $result, $link );
            	}
            }

            return $result;
        }
    }
?>
