--
-- This file will update the database schema from 0.2 to 0.3
--

--
-- New tables
--
CREATE TABLE plog_filtered_content (
   id int(10) NOT NULL auto_increment,
   reg_exp text,
   blog_id int(10) NOT NULL default '0',
   reason text,
   date timestamp(14) NOT NULL,
   PRIMARY KEY  (id)
   ) TYPE=MyISAM;

CREATE TABLE plog_host_blocking_rules (
   id int(10) NOT NULL auto_increment,
   reason text,
   date timestamp(14) NOT NULL,
   blog_id int(10) NOT NULL default '0',
   block_type int(1) default '1',
   list_type int(1) default '1',
   mask int(2) default '0',
   host varchar(15) default '0.0.0.0',
   PRIMARY KEY  (id)
   ) TYPE=MyISAM;

CREATE TABLE plog_gallery_resources (
   id int(10) NOT NULL auto_increment,
   owner_id int(10) NOT NULL default '0',
   album_id int(10) NOT NULL default '0',
   description text,
   date timestamp(14) NOT NULL,
   flags int(10) default '0',
   resource_type int(3) default NULL,
   file_path varchar(255) default '',
   file_name varchar(255) default '',
   metadata text,
   thumbnail_format varchar(4) not null default 'same',
   PRIMARY KEY  (id)
   ) TYPE=MyISAM;

CREATE TABLE plog_gallery_albums (
   id int(10) NOT NULL auto_increment,
   owner_id int(10) NOT NULL default '0',
   description text NOT NULL,
   name varchar(255) NOT NULL default '',
   flags int(10) NOT NULL default '0',
   parent_id int(10) NOT NULL default '0',
   date timestamp(14) NOT NULL,
   properties text NOT NULL DEFAULT '',
   show_album TINYINT(1) DEFAULT 1,
   PRIMARY KEY  (id)
   ) TYPE=MyISAM;

CREATE TABLE plog_bayesian_filter_info (
   id int(10) unsigned NOT NULL auto_increment,
   blog_id int(10) unsigned default NULL,
   total_spam int(10) unsigned default NULL,
   total_nonspam int(10) unsigned default NULL,
   PRIMARY KEY  (id)
   ) TYPE=MyISAM;

CREATE TABLE plog_bayesian_tokens (
   id int(10) unsigned NOT NULL auto_increment,
   blog_id int(10) unsigned default NULL,
   token char(100) default NULL,
   spam_occurrences int(10) unsigned default NULL,
   nonspam_occurrences int(10) unsigned default NULL,
   prob float default NULL,
   PRIMARY KEY  (id),
   KEY blog_id (blog_id),
   KEY token (token)
   ) TYPE=MyISAM;

--
-- Changes made in old tables
-- 
ALTER TABLE plog_articles ADD COLUMN properties TEXT NOT NULL default '';
ALTER TABLE plog_articles_categories ADD COLUMN in_main_page TINYINT(1) default '1';
ALTER TABLE plog_articles_comments ADD COLUMN status TINYINT(2) default '0';
ALTER TABLE plog_articles_comments ADD COLUMN spam_rate FLOAT default '0';

--
-- New configuration settings
--
INSERT INTO plog_config (config_key, config_value, value_type) VALUES ('security_pipeline_enabled', 1, 1);
INSERT INTO plog_config (config_key, config_value, value_type) VALUES ('maximum_comment_size', 0, 1 );
INSERT INTO plog_config (config_key, config_value, value_type) VALUES ('resources_enabled', 1, 1 );
INSERT INTO plog_config (config_key, config_value, value_type) VALUES ('thumbnail_method', 'gd', 3);
INSERT INTO plog_config (config_key, config_value, value_type) VALUES ('path_to_convert', '/usr/bin/convert', 3);
INSERT INTO plog_config (config_key, config_value, value_type) VALUES ('thumbnail_height', 120, 1);
INSERT INTO plog_config (config_key, config_value, value_type) VALUES ('thumbnail_width', 120, 1);
INSERT INTO plog_config (config_key, config_value, value_type) VALUES ('thumbnails_keep_aspect_ratio', 1, 1);
INSERT INTO plog_config (config_key, config_value, value_type) VALUES ('xmlrpc_api_enabled', 1, 1);
INSERT INTO plog_config (config_key, config_value, value_type) VALUES ('uploads_enabled', 1, 1);
INSERT INTO plog_config (config_key, config_value, value_type) VALUES ('default_rss_profile', 'rss090', 3);
INSERT INTO plog_config (config_key, config_value, value_type) VALUES ('bayesian_filter_enabled', 1, 2);
INSERT INTO plog_config (config_key, config_value, value_type) VALUES ('bayesian_filter_spam_probability_treshold', '0.9', 6);
INSERT INTO plog_config (config_key, config_value, value_type) VALUES ('bayesian_filter_nonspam_probability_treshold', '0.2', 6);
INSERT INTO plog_config (config_key, config_value, value_type) VALUES ('bayesian_filter_min_length_token', '3', 1);
INSERT INTO plog_config (config_key, config_value, value_type) VALUES ('bayesian_filter_max_length_token', '100', 1);
INSERT INTO plog_config (config_key, config_value, value_type) VALUES ('bayesian_filter_number_significant_tokens', 15, 1);
INSERT INTO plog_config (config_key, config_value, value_type) VALUES ('bayesian_filter_spam_comments_action', 0, 1 );
INSERT INTO plog_config (config_key, config_value, value_type) VALUES ('ip_address_filter_enabled', 1, 1 );
INSERT INTO plog_config (config_key, config_value, value_type) VALUES ('content_filter_enabled', 0, 1 );
INSERT INTO plog_config (config_key, config_value, value_type) VALUES ('thumbnail_format','same',3);
INSERT INTO plog_config (config_key, config_value, value_type) VALUES ('resources_folder','./gallery/',3);

--
-- Change the value of some old fields
--
UPDATE plog_config SET config_value = 'a:3:{i:0;s:7:\"blueish\";i:1;s:4:\"grey\";i:2;s:6:\"lemons\";}' WHERE config_key = 'templates';
UPDATE plog_config SET config_value = 'blueish' WHERE config_key = 'default_template';

--
-- This is for the bayesian filter... this rows must have been initialized beforehand
-- The wizard does it automatically but there is no way to do it here, so we initialize 10
-- of them here... if by any chance we have more than 10 blogs, we'll have to do it manually
-- or else the bayesian filter won't work!
--
INSERT INTO plog_bayesian_filter_info VALUES ( 1, 1, 0, 0 );
INSERT INTO plog_bayesian_filter_info VALUES ( 2, 2, 0, 0 );
INSERT INTO plog_bayesian_filter_info VALUES ( 3, 3, 0, 0 );
INSERT INTO plog_bayesian_filter_info VALUES ( 4, 4, 0, 0 );
INSERT INTO plog_bayesian_filter_info VALUES ( 5, 5, 0, 0 );
INSERT INTO plog_bayesian_filter_info VALUES ( 6, 6, 0, 0 );
INSERT INTO plog_bayesian_filter_info VALUES ( 7, 7, 0, 0 );
INSERT INTO plog_bayesian_filter_info VALUES ( 8, 8, 0, 0 );
INSERT INTO plog_bayesian_filter_info VALUES ( 9, 9, 0, 0 );
INSERT INTO plog_bayesian_filter_info VALUES ( 10, 10, 0, 0 );