<?php

	if (!defined( "PLOG_CLASS_PATH" )) {
    	define( "PLOG_CLASS_PATH", dirname(__FILE__)."/");
    }
    
	/**
     * Script that shows a summary page with the 'n' most recent additions
     * of all blogs in one page. Therefore, it is called a "Summary" :)
     */

     include_once( PLOG_CLASS_PATH."class/action/action.class.php" );
     include_once( PLOG_CLASS_PATH."class/view/view.class.php" );
     include_once( PLOG_CLASS_PATH."class/controller/controller.class.php" );
     include_once( PLOG_CLASS_PATH."class/template/templateutils.class.php" );
     include_once( PLOG_CLASS_PATH."class/template/templateservice.class.php" );
     include_once( PLOG_CLASS_PATH."class/dao/blogs.class.php" );
     include_once( PLOG_CLASS_PATH."class/dao/articles.class.php" );
     include_once( PLOG_CLASS_PATH."class/dao/summarystats.class.php" );     
     include_once( PLOG_CLASS_PATH."class/data/timestamp.class.php" );
     include_once( PLOG_CLASS_PATH."class/locale/locales.class.php" );
     include_once( PLOG_CLASS_PATH."class/config/config.class.php" );
     include_once( PLOG_CLASS_PATH."class/net/httpvars.class.php" );
     include_once( PLOG_CLASS_PATH."class/net/verysimplerequestgenerator.class.php" );
     include_once( PLOG_CLASS_PATH."class/misc/version.class.php" );

     /**
      * This is the one and only default action. It simply fetches all the most recent
      * posts from the database and shows them. The default locale is the one specified
      * in the configuration file and the amount of posts shown in this page is also
      * configurable through the config file.
      */
     class SummaryAction extends Action 
     {

     	var $_locale;
        var $_numPosts;

     	function SummaryAction( $actionInfo, $request )
        {
        	$this->Action( $actionInfo, $request );

            $config =& Config::getConfig();
            $this->_locale   = Locales::getLocale( $config->getValue("default_locale" ));
            $this->_numPosts = $config->getValue( "summary_page_show_max" );
        }

        /**
         * Loads the posts and shows them.
         */
     	function perform()
        {
            $t           = new Timestamp();
        	$articles    = new Articles();
            $blogs       = new Blogs();
            $stats       = new SummaryStats();

        	// load the posts
            $recentPosts = $stats->getRecentArticles( $this->_numPosts );
            // get all the blogs           
			$siteBlogs   = $blogs->getAllBlogs( true );
            $recentBlogs = $stats->getRecentBlogs( $this->_numPosts );

	        $activeBlogs    = $stats->getMostActiveBlogs( $this->_numPosts, $t->getYear().$t->getMonth().$t->getDay());
			$commentedPosts = $stats->getMostCommentedArticles( $this->_numPosts, $t->getYear().$t->getMonth().$t->getDay());
			$readestBlogs   = $stats->getMostReadArticles( $this->_numPosts, $t->getYear().$t->getMonth().$t->getDay());
			
            if( !$recentPosts ) {
            	// if there was an error, show the error view
                $this->_view = new SummaryView( "error" );
                $this->_view->setValue( "message", "There are no posts yet" );
            }
            else {
            	$this->_view = new SummaryView( "index" );
                $this->_view->setValue( "posts", $recentPosts );
                $this->_view->setValue( "recentBlogs", $recentBlogs );
                $this->_view->setValue( "activeBlogs", $activeBlogs );
                $this->_view->setValue( "commentedPosts", $commentedPosts );
				$this->_view->setValue( "readestBlogs", $readestBlogs );
				$this->_view->setValue( "blogs", $siteBlogs );
                $this->_view->setValue( "url", new VerySimpleRequestGenerator());
            }

            $this->_view->setValue( "locale", $this->_locale );

            return true;
        }
     }

     class BlogListAction extends Action 
     {

     	var $_locale;
        var $_numPosts;

     	function BlogListAction( $actionInfo, $request )
        {
        	$this->Action( $actionInfo, $request );

            $config =& Config::getConfig();
            $this->_locale   = Locales::getLocale( $config->getValue("default_locale" ));
            $this->_numPosts = $config->getValue( "summary_page_show_max" );
        }

        /**
         * Loads the posts and shows them.
         */
     	function perform()
        {
            // get all the blogs
            $blogs     = new Blogs();            
			$siteBlogs = $blogs->getAllBlogs( true );
			
            if( !$siteBlogs ) {
            	// if there was an error, show the error view
                $this->_view = new SummaryView( "error" );
                $this->_view->setValue( "message", "There are no posts yet" );
            }
            else {
            	$this->_view = new SummaryView( "blogslist" );
				$this->_view->setValue( "blogs", $siteBlogs );
                $this->_view->setValue( "url", new VerySimpleRequestGenerator());
            }

            $this->_view->setValue( "locale", $this->_locale );

            return true;
        }
     }

     class FaqAction extends Action 
     {
     	var $_locale;

     	function FaqAction( $actionInfo, $request )
        {
        	$this->Action( $actionInfo, $request );

            $config =& Config::getConfig();
            $this->_locale   = Locales::getLocale( $config->getValue("default_locale" ));
        }

        /**
         * Loads the posts and shows them.
         */
     	function perform()
        {
            $show = $this->_request->getValue( "show" );

            if (!$show)
            {
                $show = "faq";
            }

        	$this->_view = new SummaryView( $show );
            $this->_view->setValue( "url", new VerySimpleRequestGenerator());
            $this->_view->setValue( "locale", $this->_locale );

            return true;
        }
     }

	class SummaryView extends View 
	{

    	var $_templateName;

    	function SummaryView( $templateName )
        {
        	$this->View();
            $this->_templateName = $templateName;
        }

        function render()
        {
        	$templateService = new TemplateService();
            $template = $templateService->Template( $this->_templateName, "summary" );
            $this->_params->setValue( "version", new Version());
            $template->assign( $this->_params->getAsArray());
            print $template->fetch();
        }
    }

    //// map to associate actions with classes ////
    $_actionMap["Default"]  = "SummaryAction";
    $_actionMap["BlogList"] = "BlogListAction";
    $_actionMap["Faq"]      = "FaqAction";

    //// main part ////
    $controller = new Controller( $_actionMap, "op", false );
    $controller->process( HttpVars::getRequest());
?>