<?php

	if (!defined( "PLOG_CLASS_PATH" )) {
    	define( "PLOG_CLASS_PATH", dirname(__FILE__)."/");
    }

	include_once( PLOG_CLASS_PATH."class/dao/trackbacks.class.php" );
    include_once( PLOG_CLASS_PATH."class/config/properties.class.php" );
    include_once( PLOG_CLASS_PATH."class/config/config.class.php" );
    include_once( PLOG_CLASS_PATH."class/data/textfilter.class.php" );
    include_once( PLOG_CLASS_PATH."class/net/httpvars.class.php" );
    include_once( PLOG_CLASS_PATH."class/logger/logger.class.php" );

    //
    // set this to 'true' if you want this script to log whatever it is
    // doing... (useful for debugging)
    //
    define( "TRACKBACK_DEBUG_ENABLED", false );    
    
    class TrackbackLogger extends Logger
    {
        /**
         * logs a single string to the log file
         */
        function log( $string, $fileName = null )
        {
            if( TRACKBACK_DEBUG_ENABLED ) {
               return Logger::log( $string, $fileName );
            }
        }
        
        /**
         * dumps the whole http request to the file...
         */
        function logRequest( $request, $fileName = null )
        {
            TrackbackLogger::log( "-- dump of trackback request --" );
            $params = $request->getAsArray();
            foreach( $params as $key => $value ) 
            {
                TrackbackLogger::log( "  $key = $value" );
            }
            TrackbackLogger::log( "-- end of dump --" );
            
            return true;            
        }
    }

    function errorResponse( $message )
    {
    	$result = '<?xml version="1.0" encoding="iso-8859-1"?>';
    	$result .= '<response>';
    	$result .= '<error>1</error>';
        $result .= '<message>'.$message.'</message>';
    	$result .= '</response>';

        return $result;
    }

    // prepare everything...    
    $config =& Config::getConfig();
    TrackbackLogger::setLogFilePath( $config->getValue( "temp_folder" ));

    // get the post and get parameters
    TrackbackLogger::log( "** Request received" );
    $params = new Properties( HttpVars::getRequest());
    TrackbackLogger::logRequest( $params );    

    // check that they are correct and quit if they're not
    if( $params->getValue("id") == "" || $params->getValue("id") <= 0 ) {
    	$result = errorResponse( "Incorrect or missing id parameter." );
        print($result);
        TrackbackLogger::log( "Sending error response: $result" );
        TrackbackLogger::log( "** End" );
        die;
    }

    if( $params->getValue( "url" ) == "" ) {
    	$result = errorResponse( "The url parameter must be present." );
        print($result);
        TrackbackLogger::log( "Sending error response: $result" );
        TrackbackLogger::log( "** End" );        
        die;
    }

    if( $config->getValue( "trackback_server_enabled" )) {
    	// receives the request and adds it to the database

    	$trackbacks = new TrackBacks();
        // for security, we will strip _ANY_ html tag from the tags
        $tf = new TextFilter();
        $blogName = $tf->filterAllHTML( $params->getValue( "blog_name" ));
        $excerpt  = $tf->filterAllHTML( $params->getValue( "excerpt" ));
        $title    = $tf->filterAllHTML( $params->getValue( "title" ));
        // and now add it to our database
    	$trackbacks->addTrackBack( $params->getValue("id"), $params->getValue("url"), $title,
                                   $blogName, $excerpt);

    	// result
    	$result = '<?xml version="1.0" encoding="iso-8859-1"?>';
    	$result .= '<response>';
    	$result .= '<error>0</error>';
    	$result .= '</response>';
    }
    else {
    	$result = errorResponse( "Trackback feature has been disabled by the administrator." );
    }

    // return the result
    print($result);
    
    TrackbackLogger::log( "Sending response: $result" );
    TrackbackLogger::log( "** End" );
    
?>